@@ -38,8 +38,7 @@
  * GStreamer borrows heavily from both the <ulink
  * url="http://www.cse.ogi.edu/sysl/">OGI media pipeline</ulink> and
  * Microsoft's DirectShow, hopefully taking the best of both and leaving the
- * cruft behind.  Its interface is still very fluid and thus can be changed
- * to increase the sanity/noise ratio.
+ * cruft behind. Its interface is slowly getting stable.
  *
  * The <application>GStreamer</application> library should be initialized with
  * gst_init() before it can be used. You should pass pointers to the main argc
@@ -98,7 +97,7 @@
  * by <application>GStreamer</application>. It is mostly used in unit tests 
  * to check for leaks.
  *
- * Last reviewed on 2005-11-23 (0.9.5)
+ * Last reviewed on 2006-08-11 (0.10.10)
  */
 
 #include "gst_private.h"
@@ -107,8 +106,10 @@
 #include <sys/types.h>
 #ifdef HAVE_FORK
 #include <sys/wait.h>
-#endif //HAVE_FORK
+#endif /* HAVE_FORK */
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include "gst-i18n-lib.h"
 #include <locale.h>             /* for LC_ALL */
@@ -128,13 +129,25 @@ static GList *plugin_paths = NULL;      /* for delayed processing in post_init *
 
 extern gint _gst_trace_on;
 
-/* set to TRUE when segfaults need to be left as is */
+/* defaults */
+#ifdef HAVE_FORK
+#define DEFAULT_FORK TRUE
+#else
+#define DEFAULT_FORK FALSE
+#endif /* HAVE_FORK */
+
+/* set to TRUE when segfaults need to be left as is, FIXME, this variable is
+ * global. */
 gboolean _gst_disable_segtrap = FALSE;
 
+/* control the behaviour of registry rebuild */
+static gboolean _gst_enable_registry_fork = DEFAULT_FORK;
 
 static void load_plugin_func (gpointer data, gpointer user_data);
-static gboolean init_pre (void);
-static gboolean init_post (void);
+static gboolean init_pre (GOptionContext * context, GOptionGroup * group,
+    gpointer data, GError ** error);
+static gboolean init_post (GOptionContext * context, GOptionGroup * group,
+    gpointer data, GError ** error);
 static gboolean parse_goption_arg (const gchar * s_opt,
     const gchar * arg, gpointer data, GError ** err);
 
@@ -166,7 +179,8 @@ enum
   ARG_PLUGIN_SPEW,
   ARG_PLUGIN_PATH,
   ARG_PLUGIN_LOAD,
-  ARG_SEGTRAP_DISABLE
+  ARG_SEGTRAP_DISABLE,
+  ARG_REGISTRY_FORK_DISABLE
 };
 
 /* debug-spec ::= category-spec [, category-spec]*
@@ -270,42 +284,53 @@ GOptionGroup *
 gst_init_get_option_group (void)
 {
   GOptionGroup *group;
-  static GOptionEntry gst_args[] = {
+  const static GOptionEntry gst_args[] = {
     {"gst-version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Print the GStreamer version"), NULL},
+        (gpointer) parse_goption_arg, N_("Print the GStreamer version"), NULL},
     {"gst-fatal-warnings", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Make all warnings fatal"), NULL},
+        (gpointer) parse_goption_arg, N_("Make all warnings fatal"), NULL},
 #ifndef GST_DISABLE_GST_DEBUG
     {"gst-debug-help", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-          parse_goption_arg, N_("Print available debug categories and exit"),
+          (gpointer) parse_goption_arg,
+          N_("Print available debug categories and exit"),
         NULL},
-    {"gst-debug-level", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-debug-level", 0, 0, G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
           N_("Default debug level from 1 (only error) to 5 (anything) or "
               "0 for no output"),
         N_("LEVEL")},
-    {"gst-debug", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-debug", 0, 0, G_OPTION_ARG_CALLBACK, (gpointer) parse_goption_arg,
           N_("Comma-separated list of category_name:level pairs to set "
               "specific levels for the individual categories. Example: "
               "GST_AUTOPLUG:5,GST_ELEMENT_*:3"),
         N_("LIST")},
     {"gst-debug-no-color", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Disable colored debugging output"), NULL},
+          (gpointer) parse_goption_arg, N_("Disable colored debugging output"),
+        NULL},
     {"gst-debug-disable", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Disable debugging"), NULL},
+        (gpointer) parse_goption_arg, N_("Disable debugging"), NULL},
 #endif
     {"gst-plugin-spew", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-          parse_goption_arg, N_("Enable verbose plugin loading diagnostics"),
+          (gpointer) parse_goption_arg,
+          N_("Enable verbose plugin loading diagnostics"),
         NULL},
-    {"gst-plugin-path", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-plugin-path", 0, 0, G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
         N_("Colon-separated paths containing plugins"), N_("PATHS")},
-    {"gst-plugin-load", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-plugin-load", 0, 0, G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
           N_("Comma-separated list of plugins to preload in addition to the "
               "list stored in environment variable GST_PLUGIN_PATH"),
         N_("PLUGINS")},
     {"gst-disable-segtrap", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-          parse_goption_arg,
+          (gpointer) parse_goption_arg,
           N_("Disable trapping of segmentation faults during plugin loading"),
         NULL},
+    {"gst-disable-registry-fork", 0, G_OPTION_FLAG_NO_ARG,
+          G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
+          N_("Disable the use of fork() while scanning the registry"),
+        NULL},
     {NULL}
   };
 
@@ -342,6 +367,8 @@ gst_init_check (int *argc, char **argv[], GError ** err)
   GOptionContext *ctx;
   gboolean res;
 
+  GST_INFO ("initializing GStreamer");
+
   if (gst_initialized) {
     GST_DEBUG ("already initialized gst");
     return TRUE;
@@ -354,8 +381,12 @@ gst_init_check (int *argc, char **argv[], GError ** err)
   res = g_option_context_parse (ctx, argc, argv, err);
   g_option_context_free (ctx);
 
+  gst_initialized = res;
+
   if (res) {
-    gst_initialized = TRUE;
+    GST_INFO ("initialized GStreamer successfully");
+  } else {
+    GST_INFO ("failed to initialize GStreamer");
   }
 
   return res;
@@ -386,7 +417,7 @@ gst_init (int *argc, char **argv[])
   GError *err = NULL;
 
   if (!gst_init_check (argc, argv, &err)) {
-    g_print ("Could not initialized GStreamer: %s\n",
+    g_print ("Could not initialize GStreamer: %s\n",
         err ? err->message : "unknown error occurred");
     if (err) {
       g_error_free (err);
@@ -407,7 +438,7 @@ add_path_func (gpointer data, gpointer user_data)
 static void
 prepare_for_load_plugin_func (gpointer data, gpointer user_data)
 {
-  preload_plugins = g_slist_prepend (preload_plugins, data);
+  preload_plugins = g_slist_prepend (preload_plugins, g_strdup (data));
 }
 
 static void
@@ -455,7 +486,6 @@ split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
       iterator (strings[j], user_data);
       if (++j == MAX_PATH_SPLIT) {
         lastlist = g_strdup (strings[j]);
-        g_strfreev (strings);
         j = 0;
         break;
       }
@@ -466,7 +496,8 @@ split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
 
 /* we have no fail cases yet, but maybe in the future */
 static gboolean
-init_pre (void)
+init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
+    GError ** error)
 {
   /* GStreamer was built against a GLib >= 2.8 and is therefore not doing
    * the refcount hack. Check that it isn't being run against an older GLib */
@@ -543,6 +574,263 @@ static GstPluginDesc plugin_desc = {
   GST_PADDING_INIT
 };
 
+#ifndef GST_DISABLE_REGISTRY
+
+/*
+ * scan_and_update_registry:
+ * @default_registry: the #GstRegistry
+ * @registry_file: registry filename
+ * @write_changes: write registry if it has changed?
+ *
+ * Scans for registry changes and evntualy updates the registry cache. 
+ *
+ * Return: %TRUE if the registry could be updated
+ */
+static gboolean
+scan_and_update_registry (GstRegistry * default_registry,
+    const gchar * registry_file, gboolean write_changes, GError ** error)
+{
+  const gchar *plugin_path;
+  gboolean changed = FALSE;
+  GList *l;
+
+  GST_DEBUG ("reading registry cache: %s", registry_file);
+  gst_registry_xml_read_cache (default_registry, registry_file);
+
+  /* scan paths specified via --gst-plugin-path */
+  GST_DEBUG ("scanning paths added via --gst-plugin-path");
+  for (l = plugin_paths; l != NULL; l = l->next) {
+    GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
+    /* CHECKME: add changed |= here as well? */
+    gst_registry_scan_path (default_registry, (gchar *) l->data);
+    g_free (l->data);
+  }
+  g_list_free (plugin_paths);
+  plugin_paths = NULL;
+
+  /* GST_PLUGIN_PATH specifies a list of directories to scan for
+   * additional plugins.  These take precedence over the system plugins */
+  plugin_path = g_getenv ("GST_PLUGIN_PATH");
+  if (plugin_path) {
+    char **list;
+    int i;
+
+    GST_DEBUG ("GST_PLUGIN_PATH set to %s", plugin_path);
+    list = g_strsplit (plugin_path, G_SEARCHPATH_SEPARATOR_S, 0);
+    for (i = 0; list[i]; i++) {
+      changed |= gst_registry_scan_path (default_registry, list[i]);
+    }
+    g_strfreev (list);
+  } else {
+    GST_DEBUG ("GST_PLUGIN_PATH not set");
+  }
+
+  /* GST_PLUGIN_SYSTEM_PATH specifies a list of plugins that are always
+   * loaded by default.  If not set, this defaults to the system-installed
+   * path, and the plugins installed in the user's home directory */
+  plugin_path = g_getenv ("GST_PLUGIN_SYSTEM_PATH");
+  if (plugin_path == NULL) {
+    char *home_plugins;
+
+    GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH not set");
+
+    /* plugins in the user's home directory take precedence over
+     * system-installed ones */
+    home_plugins = g_build_filename (g_get_home_dir (),
+        ".gstreamer-" GST_MAJORMINOR, "plugins", NULL);
+    changed |= gst_registry_scan_path (default_registry, home_plugins);
+    g_free (home_plugins);
+
+    /* add the main (installed) library path */
+    changed |= gst_registry_scan_path (default_registry, PLUGINDIR);
+  } else {
+    gchar **list;
+    gint i;
+
+    GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH set to %s", plugin_path);
+    list = g_strsplit (plugin_path, G_SEARCHPATH_SEPARATOR_S, 0);
+    for (i = 0; list[i]; i++) {
+      changed |= gst_registry_scan_path (default_registry, list[i]);
+    }
+    g_strfreev (list);
+  }
+
+  /* Remove cached plugins so stale info is cleared. */
+  changed |= _priv_gst_registry_remove_cache_plugins (default_registry);
+
+  if (!changed) {
+    GST_INFO ("Registry cache has not changed");
+    return TRUE;
+  }
+
+  if (!write_changes) {
+    GST_INFO ("Registry cached changed, but writing is disabled. Not writing.");
+    return TRUE;
+  }
+
+  GST_INFO ("Registry cache changed. Writing new registry cache");
+  if (!gst_registry_xml_write_cache (default_registry, registry_file)) {
+    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+        _("Error writing registry cache to %s: %s"),
+        registry_file, g_strerror (errno));
+    return FALSE;
+  }
+
+  GST_INFO ("Registry cache written successfully");
+  return TRUE;
+}
+
+static gboolean
+ensure_current_registry_nonforking (GstRegistry * default_registry,
+    const gchar * registry_file, GError ** error)
+{
+  /* fork() not available */
+  GST_DEBUG ("Updating registry cache in-process");
+  scan_and_update_registry (default_registry, registry_file, TRUE, error);
+  return TRUE;
+}
+
+/* when forking is not available this function always does nothing but return
+ * TRUE immediatly */
+static gboolean
+ensure_current_registry_forking (GstRegistry * default_registry,
+    const gchar * registry_file, GError ** error)
+{
+#ifdef HAVE_FORK
+  pid_t pid;
+  int pfd[2];
+
+  /* We fork here, and let the child read and possibly rebuild the registry.
+   * After that, the parent will re-read the freshly generated registry. */
+  GST_DEBUG ("forking to update registry");
+
+  if (pipe (pfd) == -1) {
+    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+        _("Error re-scanning registry %s: %s"),
+        ", could not create pipes. Error", g_strerror (errno));
+    return FALSE;
+  }
+
+  pid = fork ();
+  if (pid == -1) {
+    GST_ERROR ("Failed to fork()");
+    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+        _("Error re-scanning registry %s: %s"),
+        ", failed to fork. Error", g_strerror (errno));
+    return FALSE;
+  }
+
+  if (pid == 0) {
+    gboolean res;
+    gchar res_byte;
+
+    /* this is the child. Close the read pipe */
+    close (pfd[0]);
+
+    GST_DEBUG ("child reading registry cache");
+    res =
+        scan_and_update_registry (default_registry, registry_file, TRUE, NULL);
+
+    /* need to use _exit, so that any exit handlers registered don't
+     * bring down the main program */
+    GST_DEBUG ("child exiting: %s", (res) ? "SUCCESS" : "FAILURE");
+
+    /* make valgrind happy (yes, you can call it insane) */
+    g_free ((char *) registry_file);
+
+    /* write a result byte to the pipe */
+    res_byte = res ? '1' : '0';
+    if (write (pfd[1], &res_byte, 1) != 1 || close (pfd[1]) != 0) {
+      /* could not write to pipe, probably means parent has exited before us */
+    }
+    _exit (0);
+  } else {
+    int ret;
+    gchar res_byte;
+
+    /* parent. Close write pipe */
+    close (pfd[1]);
+
+    /* Wait for result from the pipe */
+    GST_DEBUG ("Waiting for data from child");
+    ret = read (pfd[0], &res_byte, 1);
+    if (ret == -1) {
+      g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+          _("Error re-scanning registry %s: %s"),
+          ", read returned error", g_strerror (errno));
+      close (pfd[0]);
+      return FALSE;
+    }
+    close (pfd[0]);
+
+    /* Wait to ensure the child is reaped, but ignore the result */
+    GST_DEBUG ("parent waiting on child");
+    waitpid (pid, NULL, 0);
+    GST_DEBUG ("parent done waiting on child");
+
+    if (ret == 0) {
+      GST_ERROR ("child did not exit normally, terminated by signal");
+      g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+          _("Error re-scanning registry %s"), ", child terminated by signal");
+      return FALSE;
+    }
+
+    if (res_byte == '1') {
+      GST_DEBUG ("Child succeeded. Parent reading registry cache");
+      gst_registry_xml_read_cache (default_registry, registry_file);
+    } else {
+      GST_DEBUG ("Child failed. Parent re-scanning registry, ignoring errors.");
+      scan_and_update_registry (default_registry, registry_file, FALSE, NULL);
+    }
+  }
+#endif /* HAVE_FORK */
+  return TRUE;
+}
+
+static gboolean
+ensure_current_registry (GError ** error)
+{
+  char *registry_file;
+  GstRegistry *default_registry;
+  gboolean ret;
+  gboolean do_fork;
+
+  default_registry = gst_registry_get_default ();
+  registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
+  if (registry_file == NULL) {
+    registry_file = g_build_filename (g_get_home_dir (),
+        ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".xml", NULL);
+  }
+
+  /* first see if forking is enabled */
+  do_fork = _gst_enable_registry_fork;
+  if (do_fork) {
+    const gchar *fork_env;
+
+    /* forking enabled, see if it is disabled with an env var */
+    if ((fork_env = g_getenv ("GST_REGISTRY_FORK"))) {
+      /* fork enabled for any value different from "no" */
+      do_fork = strcmp (fork_env, "no") != 0;
+    }
+  }
+
+  /* now check registry with or without forking */
+  if (do_fork) {
+    GST_DEBUG ("forking for registry rebuild");
+    ret = ensure_current_registry_forking (default_registry, registry_file,
+        error);
+  } else {
+    GST_DEBUG ("requested not to fork for registry rebuild");
+    ret = ensure_current_registry_nonforking (default_registry, registry_file,
+        error);
+  }
+
+  g_free (registry_file);
+
+  return ret;
+}
+#endif /* GST_DISABLE_REGISTRY */
+
 /*
  * this bit handles:
  * - initalization of threads if we use them
@@ -555,7 +843,8 @@ static GstPluginDesc plugin_desc = {
  *   we might and then it's nice to be able to return that
  */
 static gboolean
-init_post (void)
+init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
+    GError ** error)
 {
   GLogLevelFlags llf;
 
@@ -566,6 +855,7 @@ init_post (void)
   llf = G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL;
   g_log_set_handler (g_log_domain_gstreamer, llf, debug_log_handler, NULL);
 
+  _priv_gst_quarks_initialize ();
   _gst_format_initialize ();
   _gst_query_initialize ();
   gst_object_get_type ();
@@ -595,137 +885,20 @@ init_post (void)
 
   _gst_plugin_initialize ();
 
+  /*
+   * Any errors happening below this point are non-fatal, we therefore mark
+   * gstreamer as being initialized, since it is the case from a plugin point of
+   * view.
+   *
+   * If anything fails, it will be put back to FALSE in gst_init_check().
+   * This allows some special plugins that would call gst_init() to not cause a
+   * looping effect (i.e. initializing GStreamer twice).
+   */
+  gst_initialized = TRUE;
+
 #ifndef GST_DISABLE_REGISTRY
-  {
-    char *registry_file;
-    const char *plugin_path;
-    GstRegistry *default_registry;
-    gboolean changed = FALSE;
-    GList *l;
-
-#ifdef HAVE_FORK
-    pid_t pid;
-#endif
-
-    for (l = plugin_paths; l != NULL; l = l->next) {
-      GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
-      /* CHECKME: add changed |= here as well? */
-      gst_registry_scan_path (gst_registry_get_default (), (gchar *) l->data);
-      g_free (l->data);
-    }
-    g_list_free (plugin_paths);
-    plugin_paths = NULL;
-
-    default_registry = gst_registry_get_default ();
-    registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
-    if (registry_file == NULL) {
-      registry_file = g_build_filename (g_get_home_dir (),
-          ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".xml", NULL);
-    }
-#ifdef HAVE_FORK
-    /* We fork here, and let the child read and possibly rebuild the registry.
-     * After that, the parent will re-read the freshly generated registry. */
-
-    GST_DEBUG ("forking");
-    pid = fork ();
-    if (pid == -1) {
-      GST_ERROR ("Failed to fork()");
-      return FALSE;
-    }
-
-    if (pid == 0) {
-      /* this is the child */
-#endif /* HAVE_FORK */
-
-      GST_DEBUG ("child reading registry cache");
-      gst_registry_xml_read_cache (default_registry, registry_file);
-
-      /* GST_PLUGIN_PATH specifies a list of directories to scan for
-       * additional plugins.  These take precedence over the system plugins */
-      plugin_path = g_getenv ("GST_PLUGIN_PATH");
-      if (plugin_path) {
-        char **list;
-        int i;
-
-        GST_DEBUG ("GST_PLUGIN_PATH set to %s", plugin_path);
-        list = g_strsplit (plugin_path, G_SEARCHPATH_SEPARATOR_S, 0);
-        for (i = 0; list[i]; i++) {
-          changed |= gst_registry_scan_path (default_registry, list[i]);
-        }
-        g_strfreev (list);
-      } else {
-        GST_DEBUG ("GST_PLUGIN_PATH not set");
-      }
-
-      /* GST_PLUGIN_SYSTEM_PATH specifies a list of plugins that are always
-       * loaded by default.  If not set, this defaults to the system-installed
-       * path, and the plugins installed in the user's home directory */
-      plugin_path = g_getenv ("GST_PLUGIN_SYSTEM_PATH");
-      if (plugin_path == NULL) {
-        char *home_plugins;
-
-        GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH not set");
-
-        /* plugins in the user's home directory take precedence over
-         * system-installed ones */
-        home_plugins = g_build_filename (g_get_home_dir (),
-            ".gstreamer-" GST_MAJORMINOR, "plugins", NULL);
-        changed |= gst_registry_scan_path (default_registry, home_plugins);
-        g_free (home_plugins);
-
-        /* add the main (installed) library path */
-        changed |= gst_registry_scan_path (default_registry, PLUGINDIR);
-      } else {
-        char **list;
-        int i;
-
-        GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH set to %s", plugin_path);
-        list = g_strsplit (plugin_path, G_SEARCHPATH_SEPARATOR_S, 0);
-        for (i = 0; list[i]; i++) {
-          changed |= gst_registry_scan_path (default_registry, list[i]);
-        }
-        g_strfreev (list);
-      }
-
-      if (changed) {
-        GST_DEBUG ("writing registry cache");
-        gst_registry_xml_write_cache (default_registry, registry_file);
-      }
-
-      _gst_registry_remove_cache_plugins (default_registry);
-
-#ifdef HAVE_FORK
-      /* need to use _exit, so that any exit handlers registered don't
-       * bring down the main program */
-      _exit (0);
-    } else {
-      /* parent */
-      int status;
-      pid_t ret;
-
-      GST_DEBUG ("parent waiting on child");
-      ret = waitpid (pid, &status, 0);
-      GST_DEBUG ("parent done waiting on child");
-      if (ret == -1) {
-        GST_ERROR ("error during waitpid: %s", g_strerror (errno));
-        return FALSE;
-      }
-
-      if (!WIFEXITED (status)) {
-        GST_ERROR ("child did not exit normally, status: %d", status);
-        return FALSE;
-      }
-
-      GST_DEBUG ("child exited normally with return value %d",
-          WEXITSTATUS (status));
-      GST_DEBUG ("parent reading registry cache");
-      gst_registry_xml_read_cache (default_registry, registry_file);
-    }
-#endif /* HAVE_FORK */
-
-    g_free (registry_file);
-  }
-
+  if (!ensure_current_registry (error))
+    return FALSE;
 #endif /* GST_DISABLE_REGISTRY */
 
   /* if we need to preload plugins */
@@ -764,7 +937,8 @@ gst_debug_help (void)
   GSList *list, *walk;
   GList *list2, *g;
 
-  if (!init_post ())
+  /* Need to ensure the registry is loaded to get debug categories */
+  if (!init_post (NULL, NULL, NULL, NULL))
     exit (1);
 
   list2 = gst_registry_plugin_filter (gst_registry_get_default (),
@@ -772,7 +946,7 @@ gst_debug_help (void)
 
   /* FIXME this is gross.  why don't debug have categories PluginFeatures? */
   for (g = list2; g; g = g_list_next (g)) {
-    GstPlugin *plugin = GST_PLUGIN (g->data);
+    GstPlugin *plugin = GST_PLUGIN_CAST (g->data);
 
     gst_plugin_load (plugin);
   }
@@ -862,6 +1036,9 @@ parse_one_option (gint opt, const gchar * arg, GError ** err)
     case ARG_SEGTRAP_DISABLE:
       _gst_disable_segtrap = TRUE;
       break;
+    case ARG_REGISTRY_FORK_DISABLE:
+      _gst_enable_registry_fork = FALSE;
+      break;
     default:
       g_set_error (err, G_OPTION_ERROR, G_OPTION_ERROR_UNKNOWN_OPTION,
           _("Unknown option"));
@@ -875,7 +1052,7 @@ static gboolean
 parse_goption_arg (const gchar * opt,
     const gchar * arg, gpointer data, GError ** err)
 {
-  const struct
+  static const struct
   {
     gchar *opt;
     int val;
@@ -896,6 +1073,7 @@ parse_goption_arg (const gchar * opt,
     "--gst-plugin-path", ARG_PLUGIN_PATH}, {
     "--gst-plugin-load", ARG_PLUGIN_LOAD}, {
     "--gst-disable-segtrap", ARG_SEGTRAP_DISABLE}, {
+    "--gst-disable-registry-fork", ARG_REGISTRY_FORK_DISABLE}, {
     NULL}
   };
   gint val = 0, n;
@@ -910,26 +1088,37 @@ parse_goption_arg (const gchar * opt,
   return parse_one_option (val, arg, err);
 }
 
+extern GstRegistry *_gst_registry_default;
+
 /**
  * gst_deinit:
  *
- * Clean up.
- * Call only once, before exiting.
- * After this call GStreamer should not be used anymore.
+ * Clean up any resources created by GStreamer in gst_init().
+ *
+ * It is normally not needed to call this function in a normal application
+ * as the resources will automatically be freed when the program terminates.
+ * This function is therefore mostly used by testsuites and other memory
+ * profiling tools.
+ *
+ * After this call GStreamer (including this method) should not be used anymore. 
  */
-
-extern GstRegistry *_gst_registry_default;
 void
 gst_deinit (void)
 {
   GstClock *clock;
 
   GST_INFO ("deinitializing GStreamer");
+
+  if (!gst_initialized) {
+    GST_DEBUG ("already deinitialized");
+    return;
+  }
+
   clock = gst_system_clock_obtain ();
   gst_object_unref (clock);
   gst_object_unref (clock);
 
-  _gst_registry_cleanup ();
+  _priv_gst_registry_cleanup ();
 
   gst_initialized = FALSE;
   GST_INFO ("deinitialized GStreamer");
@@ -981,3 +1170,80 @@ gst_version_string ()
     return g_strdup_printf ("GStreamer %d.%d.%d (prerelease)", major, minor,
         micro);
 }
+
+/**
+ * gst_segtrap_is_enabled:
+ *
+ * Some functions in the GStreamer core might install a custom SIGSEGV handler
+ * to better catch and report errors to the application. Currently this feature
+ * is enabled by default when loading plugins.
+ *
+ * Applications might want to disable this behaviour with the
+ * gst_segtrap_set_enabled() function. This is typically done if the application
+ * wants to install its own handler without GStreamer interfering.
+ *
+ * Returns: %TRUE if GStreamer is allowed to install a custom SIGSEGV handler.
+ *
+ * Since: 0.10.10
+ */
+gboolean
+gst_segtrap_is_enabled (void)
+{
+  /* yeps, it's enabled when it's not disabled */
+  return !_gst_disable_segtrap;
+}
+
+/**
+ * gst_segtrap_set_enabled:
+ * @enabled: whether a custom SIGSEGV handler should be installed.
+ *
+ * Applications might want to disable/enable the SIGSEGV handling of
+ * the GStreamer core. See gst_segtrap_is_enabled() for more information.
+ *
+ * Since: 0.10.10
+ */
+void
+gst_segtrap_set_enabled (gboolean enabled)
+{
+  _gst_disable_segtrap = !enabled;
+}
+
+/**
+ * gst_registry_fork_is_enabled:
+ *
+ * By default GStreamer will perform a fork() when scanning and rebuilding the
+ * registry file. 
+ *
+ * Applications might want to disable this behaviour with the
+ * gst_registry_fork_set_enabled() function. 
+ *
+ * Returns: %TRUE if GStreamer will use fork() when rebuilding the registry. On
+ * platforms without fork(), this function will always return %FALSE.
+ *
+ * Since: 0.10.10
+ */
+gboolean
+gst_registry_fork_is_enabled (void)
+{
+  return _gst_enable_registry_fork;
+}
+
+/**
+ * gst_registry_fork_set_enabled:
+ * @enabled: whether rebuilding the registry may fork
+ *
+ * Applications might want to disable/enable the usage of fork() when rebuilding
+ * the registry. See gst_registry_fork_is_enabled() for more information.
+ *
+ * On platforms without fork(), this function will have no effect on the return
+ * value of gst_registry_fork_is_enabled().
+ *
+ * Since: 0.10.10
+ */
+void
+gst_registry_fork_set_enabled (gboolean enabled)
+{
+#ifdef HAVE_FORK
+  _gst_enable_registry_fork = enabled;
+#endif /* HAVE_FORK */
+}
