@@ -164,6 +164,11 @@
 #include "gstutils.h"
 #include "gstchildproxy.h"
 
+/* enable for DURATION caching. 
+ * FIXME currently too many elements don't update
+ * their duration when it changes so we return inaccurate values.
+ * #define DURATION_CACHING */
+
 GST_DEBUG_CATEGORY_STATIC (bin_debug);
 #define GST_CAT_DEFAULT bin_debug
 
@@ -1239,8 +1244,8 @@ src_iterator_filter (GstElement * child, GstBin * bin)
  * gst_bin_iterate_sources:
  * @bin: a #GstBin
  *
- * Gets an iterator for all elements in the bin that have the
- * #GST_ELEMENT_IS_SRC flag set.
+ * Gets an iterator for all elements in the bin that have no sinkpads and have 
+ * the #GST_ELEMENT_IS_SINK flag unset.
  *
  * Each element yielded by the iterator will have its refcount increased, so
  * unref after use.
@@ -1412,7 +1417,8 @@ done:
       goto unknown_state;
   }
 
-  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "bin RETURN is now %d", ret);
+  GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin, "bin RETURN is now %s",
+      gst_element_state_change_return_get_name (ret));
 
   return;
 
@@ -1757,6 +1763,87 @@ done:
   return ret;
 }
 
+/* gst_iterator_fold functions for pads_activate
+ * Note how we don't stop the iterator when we fail an activation. This is
+ * probably a FIXME since when one pad activation fails, we don't want to
+ * continue our state change. */
+static gboolean
+activate_pads (GstPad * pad, GValue * ret, gboolean * active)
+{
+  if (!gst_pad_set_active (pad, *active))
+    g_value_set_boolean (ret, FALSE);
+  else if (!*active)
+    gst_pad_set_caps (pad, NULL);
+
+  /* unref the object that was reffed for us by _fold */
+  gst_object_unref (pad);
+  return TRUE;
+}
+
+/* returns false on error or early cutout (will never happen because the fold
+ * function always returns TRUE, see FIXME above) of the fold, true if all
+ * pads in @iter were (de)activated successfully. */
+static gboolean
+iterator_activate_fold_with_resync (GstIterator * iter, gpointer user_data)
+{
+  GstIteratorResult ires;
+  GValue ret = { 0 };
+
+  /* no need to unset this later, it's just a boolean */
+  g_value_init (&ret, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&ret, TRUE);
+
+  while (1) {
+    ires = gst_iterator_fold (iter, (GstIteratorFoldFunction) activate_pads,
+        &ret, user_data);
+    switch (ires) {
+      case GST_ITERATOR_RESYNC:
+        /* need to reset the result again */
+        g_value_set_boolean (&ret, TRUE);
+        gst_iterator_resync (iter);
+        break;
+      case GST_ITERATOR_DONE:
+        /* all pads iterated, return collected value */
+        goto done;
+      default:
+        /* iterator returned _ERROR or premature end with _OK, 
+         * mark an error and exit */
+        g_value_set_boolean (&ret, FALSE);
+        goto done;
+    }
+  }
+done:
+  /* return collected value */
+  return g_value_get_boolean (&ret);
+}
+
+/* is called with STATE_LOCK
+ */
+static gboolean
+gst_bin_src_pads_activate (GstBin * bin, gboolean active)
+{
+  GstIterator *iter;
+  gboolean fold_ok;
+
+  GST_DEBUG_OBJECT (bin, "src_pads_activate with active %d", active);
+
+  iter = gst_element_iterate_src_pads ((GstElement *) bin);
+  fold_ok = iterator_activate_fold_with_resync (iter, &active);
+  gst_iterator_free (iter);
+  if (G_UNLIKELY (!fold_ok))
+    goto failed;
+
+  GST_DEBUG_OBJECT (bin, "pads_activate successful");
+
+  return TRUE;
+
+  /* ERRORS */
+failed:
+  {
+    GST_DEBUG_OBJECT (bin, "source pads_activate failed");
+    return FALSE;
+  }
+}
 static GstStateChangeReturn
 gst_bin_change_state_func (GstElement * element, GstStateChange transition)
 {
@@ -1770,8 +1857,8 @@ gst_bin_change_state_func (GstElement * element, GstStateChange transition)
   gboolean done;
 
   /* we don't need to take the STATE_LOCK, it is already taken */
-  current = GST_STATE_TRANSITION_CURRENT (transition);
-  next = GST_STATE_TRANSITION_NEXT (transition);
+  current = (GstState) GST_STATE_TRANSITION_CURRENT (transition);
+  next = (GstState) GST_STATE_TRANSITION_NEXT (transition);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
       "changing state of children from %s to %s",
@@ -1786,6 +1873,9 @@ gst_bin_change_state_func (GstElement * element, GstStateChange transition)
       GST_DEBUG_OBJECT (element, "clearing EOS elements");
       bin_remove_messages (bin, NULL, GST_MESSAGE_EOS);
       GST_OBJECT_UNLOCK (bin);
+      if (current == GST_STATE_READY)
+        if (!(gst_bin_src_pads_activate (bin, TRUE)))
+          goto activate_failure;
       break;
     case GST_STATE_READY:
       /* Clear message list on next READY */
@@ -1793,6 +1883,14 @@ gst_bin_change_state_func (GstElement * element, GstStateChange transition)
       GST_DEBUG_OBJECT (element, "clearing all cached messages");
       bin_remove_messages (bin, NULL, GST_MESSAGE_ANY);
       GST_OBJECT_UNLOCK (bin);
+      if (current == GST_STATE_PAUSED)
+        if (!(gst_bin_src_pads_activate (bin, FALSE)))
+          goto activate_failure;
+      break;
+    case GST_STATE_NULL:
+      if (current == GST_STATE_READY)
+        if (!(gst_bin_src_pads_activate (bin, FALSE)))
+          goto activate_failure;
       break;
     default:
       break;
@@ -1892,6 +1990,11 @@ done:
       gst_element_state_get_name (GST_STATE (element)), ret);
 
   return ret;
+
+activate_failure:
+  GST_CAT_WARNING_OBJECT (GST_CAT_STATES, element,
+      "failure (de)activating src pads");
+  return GST_STATE_CHANGE_FAILURE;
 }
 
 /*
@@ -1980,6 +2083,9 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
 }
 
 /* handle child messages:
+ *
+ * This method is called synchronously when a child posts a message on
+ * the internal bus. 
  *
  * GST_MESSAGE_EOS: This message is only posted by sinks
  *     in the PLAYING state. If all sinks posted the EOS message, post
@@ -1987,7 +2093,8 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
  *
  * GST_MESSAGE_STATE_DIRTY: if we are the toplevel bin we do a state
  *     recalc. If we are not toplevel (we have a parent) we just post
- *     the message upwards.
+ *     the message upwards. This makes sure only the toplevel bin will
+ *     run over all its children to check if a state change happened.
  *
  * GST_MESSAGE_SEGMENT_START: just collect, never forward upwards. If an
  *     element posts segment_start twice, only the last message is kept.
@@ -2000,13 +2107,15 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
  *     Whenever someone performs a duration query on the bin, we store the
  *     result so we can answer it quicker the next time. Any element that
  *     changes its duration marks our cached values invalid.
- *     This message is also posted upwards.
+ *     This message is also posted upwards. This is currently disabled
+ *     because too many elements don't post DURATION messages when the
+ *     duration changes.
  *
  * GST_MESSAGE_CLOCK_LOST: This message is posted by an element when it
  *     can no longer provide a clock. The default bin behaviour is to
  *     check if the lost clock was the one provided by the bin. If so and
  *     we are currently in the PLAYING state, we forward the message to 
- *     our parent.
+ *     our parent. 
  *     This message is also generated when we remove a clock provider from
  *     a bin. If this message is received by the application, it should
  *     PAUSE the pipeline and set it back to PLAYING to force a new clock
@@ -2017,7 +2126,7 @@ bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
  *     provider to the bin. The default behaviour of the bin is to mark the
  *     currently selected clock as dirty, which will perform a clock
  *     recalculation the next time we are asked to provide a clock.
- *     This message is never sent tot the application but is forwarded to
+ *     This message is never sent to the application but is forwarded to
  *     the parent.
  *
  * OTHER: post upwards.
@@ -2254,7 +2363,7 @@ bin_query_duration_done (GstBin * bin, QueryFold * fold)
 
   GST_DEBUG_OBJECT (bin, "max duration %" G_GINT64_FORMAT, fold->max);
 
-#if 0
+#ifdef DURATION_CACHING
   /* and cache now */
   GST_OBJECT_LOCK (bin);
   bin->messages = g_list_prepend (bin->messages,
@@ -2326,7 +2435,7 @@ gst_bin_query (GstElement * element, GstQuery * query)
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_DURATION:
     {
-#if 0
+#ifdef DURATION_CACHING
       GList *cached;
       GstFormat qformat;
 
@@ -2358,6 +2467,7 @@ gst_bin_query (GstElement * element, GstQuery * query)
       }
       GST_OBJECT_UNLOCK (bin);
 #endif
+      /* no cached value found, iterate and collect durations */
       fold_func = (GstIteratorFoldFunction) bin_query_duration_fold;
       fold_init = bin_query_max_init;
       fold_done = bin_query_duration_done;
@@ -2510,9 +2620,10 @@ gst_bin_get_by_name_recurse_up (GstBin * bin, const gchar * name)
 static gint
 compare_interface (GstElement * element, gpointer interface)
 {
+  GType interface_type = (GType) interface;
   gint ret;
 
-  if (G_TYPE_CHECK_INSTANCE_TYPE (element, GPOINTER_TO_INT (interface))) {
+  if (G_TYPE_CHECK_INSTANCE_TYPE (element, interface_type)) {
     ret = 0;
   } else {
     /* we did not find the element, need to release the ref
@@ -2542,13 +2653,14 @@ GstElement *
 gst_bin_get_by_interface (GstBin * bin, GType interface)
 {
   GstIterator *children;
-  GstIterator *result;
+  gpointer result;
 
   g_return_val_if_fail (GST_IS_BIN (bin), NULL);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (interface), NULL);
 
   children = gst_bin_iterate_recurse (bin);
   result = gst_iterator_find_custom (children, (GCompareFunc) compare_interface,
-      GINT_TO_POINTER (interface));
+      (gpointer) interface);
   gst_iterator_free (children);
 
   return GST_ELEMENT_CAST (result);
@@ -2579,10 +2691,11 @@ gst_bin_iterate_all_by_interface (GstBin * bin, GType interface)
   GstIterator *result;
 
   g_return_val_if_fail (GST_IS_BIN (bin), NULL);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (interface), NULL);
 
   children = gst_bin_iterate_recurse (bin);
   result = gst_iterator_filter (children, (GCompareFunc) compare_interface,
-      GINT_TO_POINTER (interface));
+      (gpointer) interface);
 
   return result;
 }
@@ -2628,15 +2741,8 @@ gst_bin_restore_thyself (GstObject * object, xmlNodePtr self)
       childlist = field->xmlChildrenNode;
       while (childlist) {
         if (!strcmp ((char *) childlist->name, "element")) {
-          GstElement *element =
-              gst_xml_make_element (childlist, GST_OBJECT (bin));
-
-          /* it had to be parented to find the pads, now we ref and unparent so
-           * we can add it to the bin */
-          gst_object_ref (element);
-          gst_object_unparent (GST_OBJECT (element));
-
-          gst_bin_add (bin, element);
+          /* gst_xml_make_element will gst_bin_add() the element to ourself */
+          gst_xml_make_element (childlist, GST_OBJECT (bin));
         }
         childlist = childlist->next;
       }
