@@ -183,7 +183,14 @@ gst_bus_class_init (GstBusClass * klass)
    * @message: the message that has been posted synchronously
    *
    * A message has been posted on the bus. This signal is emitted from the
-   * thread that posted the message so one has to be carefull with locking.
+   * thread that posted the message so one has to be careful with locking.
+   *
+   * This signal will not be emitted by default, you have to set up
+   * gst_bus_sync_signal_handler() as a sync handler if you want this
+   * signal to be emitted when a message is posted on the bus, like this:
+   * <programlisting>
+   * gst_bus_set_sync_handler (bus, gst_bus_sync_signal_handler, yourdata);
+   * </programlisting>
    */
   gst_bus_signals[SYNC_MESSAGE] =
       g_signal_new ("sync-message", G_TYPE_FROM_CLASS (klass),
@@ -197,7 +204,7 @@ gst_bus_class_init (GstBusClass * klass)
    * @message: the message that has been posted asynchronously
    *
    * A message has been posted on the bus. This signal is emitted from a
-   * GSource added to the mainloop. this signal will only be emited when
+   * GSource added to the mainloop. this signal will only be emitted when
    * there is a mainloop running.
    */
   gst_bus_signals[ASYNC_MESSAGE] =
@@ -319,8 +326,8 @@ gst_bus_post (GstBus * bus, GstMessage * message)
   g_return_val_if_fail (GST_IS_BUS (bus), FALSE);
   g_return_val_if_fail (GST_IS_MESSAGE (message), FALSE);
 
-  GST_DEBUG_OBJECT (bus, "[msg %p] posting on bus, type %s",
-      message, GST_MESSAGE_TYPE_NAME (message));
+  GST_DEBUG_OBJECT (bus, "[msg %p] posting on bus, type %s, %" GST_PTR_FORMAT,
+      message, GST_MESSAGE_TYPE_NAME (message), message->structure);
 
   GST_OBJECT_LOCK (bus);
   /* check if the bus is flushing */
@@ -478,6 +485,8 @@ gst_bus_set_flushing (GstBus * bus, gboolean flushing)
  * Get a message from the bus.
  *
  * Returns: The #GstMessage that is on the bus, or NULL if the bus is empty.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
  *
  * MT safe.
  */
@@ -826,7 +835,8 @@ poll_destroy_timeout (GstBusPollData * poll_data)
  * polling.
  *
  * Returns: The message that was received, or NULL if the poll timed out.
- * The message is taken from the bus and needs to be unreffed after usage.
+ * The message is taken from the bus and needs to be unreffed with
+ * gst_message_unref() after usage.
  */
 GstMessage *
 gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
