@@ -86,7 +86,7 @@
  * slaved to another #GstClock with the gst_clock_set_master(). The clock will
  * then automatically be synchronized to this master clock by repeadedly
  * sampling the master clock and the slave clock and recalibrating the slave
- * clock with gst_clock_set_calibration(). This feature is mostly usefull for
+ * clock with gst_clock_set_calibration(). This feature is mostly useful for
  * plugins that have an internal clock but must operate with another clock
  * selected by the #GstPipeline.  They can track the offset and rate difference
  * of their internal clock relative to the master clock by using the
@@ -99,7 +99,7 @@
  * "window-threshold" defines the minimum number of samples before the 
  * calibration is performed.
  *
- * Last reviewed on 2006-03-09 (0.10.4)
+ * Last reviewed on 2006-08-11 (0.10.10)
  */
 
 
@@ -116,6 +116,14 @@
 static GstAllocTrace *_gst_clock_entry_trace;
 #endif
 
+#if GLIB_CHECK_VERSION (2, 10, 0)
+#define ALLOC_ENTRY()     g_slice_new (GstClockEntry)
+#define FREE_ENTRY(entry) g_slice_free (GstClockEntry, entry)
+#else
+#define ALLOC_ENTRY()     g_new (GstClockEntry, 1)
+#define FREE_ENTRY(entry) g_free (entry)
+#endif
+
 /* #define DEBUGGING_ENABLED */
 
 #define DEFAULT_STATS                   FALSE
@@ -154,9 +162,7 @@ gst_clock_entry_new (GstClock * clock, GstClockTime time,
 {
   GstClockEntry *entry;
 
-  /* FIXME, use g_slice, we do this a lot and potentially from
-   * different threads. */
-  entry = g_new0 (GstClockEntry, 1);
+  entry = ALLOC_ENTRY ();
 #ifndef GST_DISABLE_TRACE
   gst_alloc_trace_new (_gst_clock_entry_trace, entry);
 #endif
@@ -165,10 +171,12 @@ gst_clock_entry_new (GstClock * clock, GstClockTime time,
 
   gst_atomic_int_set (&entry->refcount, 1);
   entry->clock = clock;
+  entry->type = type;
   entry->time = time;
   entry->interval = interval;
-  entry->type = type;
   entry->status = GST_CLOCK_BUSY;
+  entry->func = NULL;
+  entry->user_data = NULL;
 
   return (GstClockID) entry;
 }
@@ -203,7 +211,7 @@ _gst_clock_id_free (GstClockID id)
 #ifndef GST_DISABLE_TRACE
   gst_alloc_trace_free (_gst_clock_entry_trace, id);
 #endif
-  g_free (id);
+  FREE_ENTRY (id);
 }
 
 /**
@@ -325,7 +333,6 @@ gst_clock_id_get_time (GstClockID id)
   return GST_CLOCK_ENTRY_TIME ((GstClockEntry *) id);
 }
 
-
 /**
  * gst_clock_id_wait
  * @id: The #GstClockID to wait on
@@ -339,9 +346,10 @@ gst_clock_id_get_time (GstClockID id)
  * If the @jitter argument is not NULL and this function returns #GST_CLOCK_OK
  * or #GST_CLOCK_EARLY, it will contain the difference
  * against the clock and the time of @id when this method was
- * called. Negative values indicate how late @id was relative to the clock
+ * called. 
+ * Positive values indicate how late @id was relative to the clock
  * (in which case this function will return #GST_CLOCK_EARLY). 
- * Positive values indicate how much time was spent waiting on the clock 
+ * Negative values indicate how much time was spent waiting on the clock 
  * before this function returned.
  *
  * Returns: the result of the blocking wait. #GST_CLOCK_EARLY will be returned
@@ -377,18 +385,29 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff * jitter)
 
   cclass = GST_CLOCK_GET_CLASS (clock);
 
-  if (G_UNLIKELY (cclass->wait == NULL))
-    goto not_supported;
-
   GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "waiting on clock entry %p", id);
 
-  /* jitter is the diff against the clock when this entry is scheduled */
-  if (jitter) {
-    GstClockTime now = gst_clock_get_time (clock);
+  /* if we have a wait_jitter function, use that */
+  if (G_LIKELY (cclass->wait_jitter)) {
+    res = cclass->wait_jitter (clock, entry, jitter);
+  } else {
+    /* check if we have a simple _wait function otherwise. The function without
+     * the jitter arg is less optimal as we need to do an additional _get_time()
+     * which is not atomic with the _wait() and a typical _wait() function does
+     * yet another _get_time() anyway. */
+    if (G_UNLIKELY (cclass->wait == NULL))
+      goto not_supported;
 
-    *jitter = GST_CLOCK_DIFF (requested, now);
+    if (jitter) {
+      GstClockTime now = gst_clock_get_time (clock);
+
+      /* jitter is the diff against the clock when this entry is scheduled. Negative
+       * values mean that the entry was in time, a positive value means that the
+       * entry was too late. */
+      *jitter = GST_CLOCK_DIFF (requested, now);
+    }
+    res = cclass->wait (clock, entry);
   }
-  res = cclass->wait (clock, entry);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
       "done waiting entry %p, res: %d", id, res);
@@ -658,7 +677,11 @@ gst_clock_finalize (GObject * object)
  * @clock: a #GstClock
  * @resolution: The resolution to set
  *
- * Set the accuracy of the clock.
+ * Set the accuracy of the clock. Some clocks have the possibility to operate
+ * with different accuracy at the expense of more resource usage. There is
+ * normally no need to change the default resolution of a clock. The resolution
+ * of a clock can only be changed if the clock has the
+ * #GST_CLOCK_FLAG_CAN_SET_RESOLUTION flag set.
  *
  * Returns: the new resolution of the clock.
  */
@@ -683,7 +706,8 @@ gst_clock_set_resolution (GstClock * clock, GstClockTime resolution)
  * gst_clock_get_resolution
  * @clock: a #GstClock
  *
- * Get the accuracy of the clock.
+ * Get the accuracy of the clock. The accuracy of the clock is the granularity
+ * of the values returned by gst_clock_get_time().
  *
  * Returns: the resolution of the clock in units of #GstClockTime.
  *
