@@ -25,9 +25,9 @@
  * @short_description: Abstract base class for all pipeline elements
  * @see_also: #GstElementFactory, #GstPad
  *
- * GstElement is the base class needed to construct an element that can be
- * used in a GStreamer pipeline.  As such, it is not a functional entity, and
- * cannot do anything when placed in a pipeline.
+ * GstElement is the abstract base class needed to construct an element that 
+ * can be used in a GStreamer pipeline. Please refer to the plugin writers
+ * guide for more information on creating #GstElement subclasses.
  *
  * The name of a #GstElement can be get with gst_element_get_name() and set with
  * gst_element_set_name().  For speed, GST_ELEMENT_NAME() can be used in the
@@ -73,7 +73,7 @@
  * toplevel #GstPipeline so the clock functions are only to be used in very
  * specific situations.
  *
- * Last reviewed on 2005-11-23 (0.9.5)
+ * Last reviewed on 2006-03-12 (0.10.5)
  */
 
 #include "gst_private.h"
@@ -93,7 +93,7 @@
 /* Element signals and args */
 enum
 {
-  NEW_PAD,
+  PAD_ADDED,
   PAD_REMOVED,
   NO_MORE_PADS,
   /* add more above */
@@ -171,7 +171,7 @@ gst_element_class_init (GstElementClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstobject_class = (GstObjectClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_OBJECT);
+  parent_class = g_type_class_peek_parent (klass);
 
   /**
    * GstElement::pad-added:
@@ -180,7 +180,7 @@ gst_element_class_init (GstElementClass * klass)
    *
    * a new #GstPad has been added to the element.
    */
-  gst_element_signals[NEW_PAD] =
+  gst_element_signals[PAD_ADDED] =
       g_signal_new ("pad-added", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstElementClass, pad_added), NULL, NULL,
       gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, G_TYPE_OBJECT);
@@ -303,6 +303,8 @@ gst_element_release_request_pad (GstElement * element, GstPad * pad)
 
   oclass = GST_ELEMENT_GET_CLASS (element);
 
+  /* if the element implements a custom release function we call that, else we
+   * simply remove the pad from the element */
   if (oclass->release_pad)
     (oclass->release_pad) (element, pad);
   else
@@ -315,7 +317,7 @@ gst_element_release_request_pad (GstElement * element, GstPad * pad)
  *
  * Query if the element requires a clock.
  *
- * Returns: TRUE if the element requires a clock
+ * Returns: %TRUE if the element requires a clock
  *
  * MT safe.
  */
@@ -337,8 +339,11 @@ gst_element_requires_clock (GstElement * element)
  *
  * Query if the element provides a clock. A #GstClock provided by an
  * element can be used as the global #GstClock for the pipeline. 
+ * An element that can provide a clock is only required to do so in the PAUSED
+ * state, this means when it is fully negotiated and has allocated the resources
+ * to operate the clock.
  *
- * Returns: TRUE if the element provides a clock
+ * Returns: %TRUE if the element provides a clock
  *
  * MT safe.
  */
@@ -358,10 +363,12 @@ gst_element_provides_clock (GstElement * element)
  * gst_element_provide_clock:
  * @element: a #GstElement to query
  *
- * Get the clock provided by the given element.
+ * Get the clock provided by the given element. 
+ * <note>An element is only required to provide a clock in the PAUSED
+ * state. Some elements can provide a clock in other states.</note>
  *
  * Returns: the GstClock provided by the element or NULL
- * if no clock could be provided. Unref after usage.
+ * if no clock could be provided.  Unref after usage.
  *
  * MT safe.
  */
@@ -390,7 +397,9 @@ gst_element_provide_clock (GstElement * element)
  * refcount on the clock. Any previously set clock on the object
  * is unreffed.
  *
- * Returns: TRUE if the element accepted the clock.
+ * Returns: %TRUE if the element accepted the clock. An element can refuse a
+ * clock when it, for example, is not able to slave its internal clock to the
+ * @clock or when it requires a specific clock to operate.
  *
  * MT safe.
  */
@@ -399,6 +408,7 @@ gst_element_set_clock (GstElement * element, GstClock * clock)
 {
   GstElementClass *oclass;
   gboolean res = TRUE;
+  GstClock **clock_p;
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
@@ -410,8 +420,10 @@ gst_element_set_clock (GstElement * element, GstClock * clock)
     res = oclass->set_clock (element, clock);
 
   if (res) {
+    /* only update the clock pointer if the element accepted the clock */
     GST_OBJECT_LOCK (element);
-    gst_object_replace ((GstObject **) & element->clock, (GstObject *) clock);
+    clock_p = &element->clock;
+    gst_object_replace ((GstObject **) clock_p, (GstObject *) clock);
     GST_OBJECT_UNLOCK (element);
   }
   return res;
@@ -421,7 +433,8 @@ gst_element_set_clock (GstElement * element, GstClock * clock)
  * gst_element_get_clock:
  * @element: a #GstElement to get the clock of.
  *
- * Gets the currently configured clock of the element.
+ * Gets the currently configured clock of the element. This is the clock as was
+ * last set with gst_element_set_clock().
  *
  * Returns: the #GstClock of the element. unref after usage.
  *
@@ -470,7 +483,7 @@ gst_element_set_base_time (GstElement * element, GstClockTime time)
  *
  * Returns the base time of the element. The base time is the
  * absolute time of the clock when this element was last put to
- * PLAYING. Substracting the base time from the clock time gives
+ * PLAYING. Subtracting the base time from the clock time gives
  * the stream time of the element.
  *
  * Returns: the base time of the element.
@@ -553,15 +566,16 @@ GstIndex *
 gst_element_get_index (GstElement * element)
 {
   GstElementClass *oclass;
+  GstIndex *result = NULL;
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
 
   oclass = GST_ELEMENT_GET_CLASS (element);
 
   if (oclass->get_index)
-    return oclass->get_index (element);
+    result = oclass->get_index (element);
 
-  return NULL;
+  return result;
 }
 #endif
 
@@ -575,11 +589,13 @@ gst_element_get_index (GstElement * element)
  *
  * Pads are not automatically activated so elements should perform the needed
  * steps to activate the pad in case this pad is added in the PAUSED or PLAYING
- * state.
+ * state. See gst_pad_set_active() for more information about activating pads.
  *
  * The pad and the element should be unlocked when calling this function.
  *
- * Returns: TRUE if the pad could be added. This function can fail when
+ * This function will emit the #GstElement::pad-added signal on the element.
+ *
+ * Returns: %TRUE if the pad could be added. This function can fail when
  * a pad with the same name already existed or the pad already had another
  * parent.
  *
@@ -630,8 +646,8 @@ gst_element_add_pad (GstElement * element, GstPad * pad)
   element->pads_cookie++;
   GST_OBJECT_UNLOCK (element);
 
-  /* emit the NEW_PAD signal */
-  g_signal_emit (G_OBJECT (element), gst_element_signals[NEW_PAD], 0, pad);
+  /* emit the PAD_ADDED signal */
+  g_signal_emit (element, gst_element_signals[PAD_ADDED], 0, pad);
 
   return TRUE;
 
@@ -671,10 +687,23 @@ no_direction:
  * @pad: the #GstPad to remove from the element.
  *
  * Removes @pad from @element. @pad will be destroyed if it has not been
- * referenced elsewhere.
+ * referenced elsewhere using gst_object_unparent().
  *
- * Returns: TRUE if the pad could be removed. Can return FALSE if the
- * pad is not belonging to the provided element.
+ * This function is used by plugin developers and should not be used
+ * by applications. Pads that were dynamically requested from elements
+ * with gst_element_get_request_pad() should be released with the
+ * gst_element_release_request_pad() function instead.
+ *
+ * Pads are not automatically deactivated so elements should perform the needed
+ * steps to deactivate the pad in case this pad is removed in the PAUSED or PLAYING
+ * state. See gst_pad_set_active() for more information about deactivating pads.
+ *
+ * The pad and the element should be unlocked when calling this function.
+ *
+ * This function will emit the #GstElement::pad-removed signal on the element.
+ *
+ * Returns: %TRUE if the pad could be removed. Can return %FALSE if the
+ * pad does not belong to the provided element.
  *
  * MT safe.
  */
@@ -728,19 +757,20 @@ gst_element_remove_pad (GstElement * element, GstPad * pad)
   element->pads_cookie++;
   GST_OBJECT_UNLOCK (element);
 
-  g_signal_emit (G_OBJECT (element), gst_element_signals[PAD_REMOVED], 0, pad);
+  /* emit the PAD_REMOVED signal before unparenting and losing the last ref. */
+  g_signal_emit (element, gst_element_signals[PAD_REMOVED], 0, pad);
 
-  gst_object_unparent (GST_OBJECT (pad));
+  gst_object_unparent (GST_OBJECT_CAST (pad));
 
   return TRUE;
 
+  /* ERRORS */
 not_our_pad:
   {
     /* FIXME, locking order? */
     GST_OBJECT_LOCK (element);
     g_critical ("Padname %s:%s does not belong to element %s when removing",
-        GST_ELEMENT_NAME (GST_PAD_PARENT (pad)), GST_PAD_NAME (pad),
-        GST_ELEMENT_NAME (element));
+        GST_DEBUG_PAD_NAME (pad), GST_ELEMENT_NAME (element));
     GST_OBJECT_UNLOCK (element);
     GST_OBJECT_UNLOCK (pad);
     return FALSE;
@@ -757,6 +787,8 @@ not_our_pad:
  * pad templates use this in combination with autopluggers to figure out that
  * the element is done initializing its pads.
  *
+ * This function emits the #GstElement::no-more-pads signal.
+ *
  * MT safe.
  */
 void
@@ -845,9 +877,10 @@ gst_element_request_pad (GstElement * element, GstPadTemplate * templ,
  * @name: the name of the request #GstPad to retrieve.
  *
  * Retrieves a pad from the element by name. This version only retrieves
- * request pads.
+ * request pads. The pad should be released with 
+ * gst_element_release_request_pad().
  *
- * Returns: requested #GstPad if found, otherwise NULL. Unref after usage.
+ * Returns: requested #GstPad if found, otherwise NULL. Release after usage.
  */
 GstPad *
 gst_element_get_request_pad (GstElement * element, const gchar * name)
@@ -923,7 +956,12 @@ gst_element_get_request_pad (GstElement * element, const gchar * name)
  * Retrieves a pad from @element by name. Tries gst_element_get_static_pad()
  * first, then gst_element_get_request_pad().
  *
- * Returns: the #GstPad if found, otherwise %NULL. Unref after usage.
+ * <note>Usage of this function is not recommended as it is unclear if the reference
+ * to the result pad should be released with gst_object_unref() in case of a static pad
+ * or gst_element_release_request_pad() in case of a request pad.</note>
+ *
+ * Returns: the #GstPad if found, otherwise %NULL. Unref or Release after usage,
+ * depending on the type of the pad.
  */
 GstPad *
 gst_element_get_pad (GstElement * element, const gchar * name)
@@ -1052,6 +1090,8 @@ gst_element_class_add_pad_template (GstElementClass * klass,
  *
  * Sets the detailed information for a #GstElementClass.
  * <note>This function is for use in _base_init functions only.</note>
+ *
+ * The @details are copied.
  */
 void
 gst_element_class_set_details (GstElementClass * klass,
@@ -1119,8 +1159,12 @@ gst_element_class_get_pad_template (GstElementClass * element_class,
   return NULL;
 }
 
+/* get a random pad on element of the given direction.
+ * The pad is random in a sense that it is the first pad that is (optionaly) linked.
+ */
 static GstPad *
-gst_element_get_random_pad (GstElement * element, GstPadDirection dir)
+gst_element_get_random_pad (GstElement * element, gboolean need_linked,
+    GstPadDirection dir)
 {
   GstPad *result = NULL;
   GList *pads;
@@ -1140,21 +1184,27 @@ gst_element_get_random_pad (GstElement * element, GstPadDirection dir)
       goto wrong_direction;
   }
   for (; pads; pads = g_list_next (pads)) {
-    GstPad *pad = GST_PAD (pads->data);
+    GstPad *pad = GST_PAD_CAST (pads->data);
 
     GST_OBJECT_LOCK (pad);
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "checking pad %s:%s",
         GST_DEBUG_PAD_NAME (pad));
 
-    if (GST_PAD_IS_LINKED (pad)) {
+    if (need_linked && !GST_PAD_IS_LINKED (pad)) {
+      /* if we require a linked pad, and it is not linked, continue the
+       * search */
+      GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "pad %s:%s is not linked",
+          GST_DEBUG_PAD_NAME (pad));
+      GST_OBJECT_UNLOCK (pad);
+      continue;
+    } else {
+      /* found a pad, stop search */
+      GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "found pad %s:%s",
+          GST_DEBUG_PAD_NAME (pad));
       GST_OBJECT_UNLOCK (pad);
       result = pad;
       break;
-    } else {
-      GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "pad %s:%s is not linked",
-          GST_DEBUG_PAD_NAME (pad));
     }
-    GST_OBJECT_UNLOCK (pad);
   }
   if (result)
     gst_object_ref (result);
@@ -1176,13 +1226,14 @@ wrong_direction:
  * @element: a #GstElement to send the event to.
  * @event: the #GstEvent to send to the element.
  *
- * Sends an event to an element. If the element doesn't
- * implement an event handler, the event will be forwarded
- * to a random sink pad. This function takes owership of the
- * provided event so you should gst_event_ref() it if you want to reuse
- * the event after this call.
+ * Sends an event to an element. If the element doesn't implement an
+ * event handler, the event will be pushed on a random linked sink pad for
+ * upstream events or a random linked source pad for downstream events. 
  *
- * Returns: TRUE if the event was handled.
+ * This function takes owership of the provided event so you should
+ * gst_event_ref() it if you want to reuse the event after this call.
+ *
+ * Returns: %TRUE if the event was handled.
  *
  * MT safe.
  */
@@ -1203,8 +1254,8 @@ gst_element_send_event (GstElement * element, GstEvent * event)
     result = oclass->send_event (element, event);
   } else {
     GstPad *pad = GST_EVENT_IS_DOWNSTREAM (event) ?
-        gst_element_get_random_pad (element, GST_PAD_SRC) :
-        gst_element_get_random_pad (element, GST_PAD_SINK);
+        gst_element_get_random_pad (element, TRUE, GST_PAD_SRC) :
+        gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
 
     if (pad) {
       GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS,
@@ -1234,7 +1285,9 @@ gst_element_send_event (GstElement * element, GstEvent * event)
  * @stop_type: The type and flags for the new stop position
  * @stop: The value of the new stop position
  *
- * Sends a seek event to an element.
+ * Sends a seek event to an element. See gst_event_new_seek() for the details of
+ * the parameters. The seek event is sent to the element using
+ * gst_element_send_event().
  *
  * Returns: %TRUE if the event was handled.
  *
@@ -1263,7 +1316,7 @@ gst_element_seek (GstElement * element, gdouble rate, GstFormat format,
  *
  * Get an array of query types from the element.
  * If the element doesn't implement a query types function,
- * the query will be forwarded to a random sink pad.
+ * the query will be forwarded to the peer of a random linked sink pad.
  *
  * Returns: An array of #GstQueryType elements that should not
  * be freed or modified.
@@ -1283,7 +1336,7 @@ gst_element_get_query_types (GstElement * element)
   if (oclass->get_query_types) {
     result = oclass->get_query_types (element);
   } else {
-    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    GstPad *pad = gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
 
     if (pad) {
       GstPad *peer = gst_pad_get_peer (pad);
@@ -1304,11 +1357,11 @@ gst_element_get_query_types (GstElement * element)
  * @element: a #GstElement to perform the query on.
  * @query: the #GstQuery.
  *
- * Performs a query on the given element. If the format is set
- * to #GST_FORMAT_DEFAULT and this function returns TRUE, the
- * format pointer will hold the default format.
- * For element that don't implement a query handler, this function
- * forwards the query to a random usable sinkpad of this element.
+ * Performs a query on the given element.
+ *
+ * For elements that don't implement a query handler, this function
+ * forwards the query to a random srcpad or to the peer of a
+ * random linked sinkpad of this element.
  *
  * Returns: TRUE if the query could be performed.
  *
@@ -1330,14 +1383,14 @@ gst_element_query (GstElement * element, GstQuery * query)
         GST_ELEMENT_NAME (element));
     result = oclass->query (element, query);
   } else {
-    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SRC);
+    GstPad *pad = gst_element_get_random_pad (element, FALSE, GST_PAD_SRC);
 
     if (pad) {
       result = gst_pad_query (pad, query);
 
       gst_object_unref (pad);
     } else {
-      pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+      pad = gst_element_get_random_pad (element, TRUE, GST_PAD_SINK);
       if (pad) {
         GstPad *peer = gst_pad_get_peer (pad);
 
@@ -1362,7 +1415,8 @@ gst_element_query (GstElement * element, GstQuery * query)
  * message; if you want to access the message after this call, you should add an
  * additional reference before calling.
  *
- * Returns: TRUE if the message was successfully posted.
+ * Returns: %TRUE if the message was successfully posted. The function returns
+ * %FALSE if the element did not have a bus.
  *
  * MT safe.
  */
@@ -1384,14 +1438,17 @@ gst_element_post_message (GstElement * element, GstMessage * message)
   gst_object_ref (bus);
   GST_OBJECT_UNLOCK (element);
 
+  /* we release the element lock when posting the message so that any
+   * (synchronous) message handlers can operate on the element */
   result = gst_bus_post (bus, message);
   gst_object_unref (bus);
 
   return result;
 
+  /* ERRORS */
 no_bus:
   {
-    GST_DEBUG ("not posting message %p: no bus", message);
+    GST_DEBUG_OBJECT (element, "not posting message %p: no bus", message);
     GST_OBJECT_UNLOCK (element);
     gst_message_unref (message);
     return FALSE;
@@ -1475,7 +1532,7 @@ void gst_element_message_full
     has_debug = FALSE;
   }
 
-  name = gst_object_get_path_string (GST_OBJECT (element));
+  name = gst_object_get_path_string (GST_OBJECT_CAST (element));
   if (has_debug)
     sent_debug = g_strdup_printf ("%s(%d): %s (): %s:\n%s",
         file, line, function, name, debug);
@@ -1491,17 +1548,18 @@ void gst_element_message_full
   gerror = g_error_new_literal (domain, code, sent_text);
 
   if (type == GST_MESSAGE_ERROR) {
-    message = gst_message_new_error (GST_OBJECT (element), gerror, sent_debug);
+    message =
+        gst_message_new_error (GST_OBJECT_CAST (element), gerror, sent_debug);
   } else if (type == GST_MESSAGE_WARNING) {
-    message = gst_message_new_warning (GST_OBJECT (element), gerror,
+    message = gst_message_new_warning (GST_OBJECT_CAST (element), gerror,
         sent_debug);
   } else {
     g_assert_not_reached ();
   }
   gst_element_post_message (element, message);
 
-  GST_CAT_INFO_OBJECT (GST_CAT_ERROR_SYSTEM, element, "posted message: %s",
-      sent_text);
+  GST_CAT_INFO_OBJECT (GST_CAT_ERROR_SYSTEM, element, "posted %s message: %s",
+      (type == GST_MESSAGE_ERROR ? "error" : "warning"), sent_text);
 
   /* cleanup */
   g_error_free (gerror);
@@ -1716,9 +1774,10 @@ done:
     *pending = GST_STATE_PENDING (element);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
-      "state current: %s, pending: %s, result: %d",
+      "state current: %s, pending: %s, result: %s",
       gst_element_state_get_name (GST_STATE (element)),
-      gst_element_state_get_name (GST_STATE_PENDING (element)), ret);
+      gst_element_state_get_name (GST_STATE_PENDING (element)),
+      gst_element_state_change_return_get_name (ret));
   GST_OBJECT_UNLOCK (element);
 
   return ret;
@@ -1741,7 +1800,7 @@ interrupted:
 /**
  * gst_element_get_state:
  * @element: a #GstElement to get the state of.
- * @state: a pointer to #GstState to hold the state. Can be NULL.
+ * @state: a pointer to #GstState to hold the state. Can be %NULL.
  * @pending: a pointer to #GstState to hold the pending state.
  *           Can be %NULL.
  * @timeout: a #GstClockTime to specify the timeout for an async
@@ -1855,9 +1914,9 @@ nothing_aborted:
  * pending state if any. This function is used   
  * by elements that do asynchronous state changes.       
  * The core will normally call this method automatically when an         
- * element returned SUCCESS from the state change function.      
- * Elements that return ASYNC from the change_state function should      
- * eventually call this method from the streaming thread to signal       
+ * element returned %GST_STATE_CHANGE_SUCCESS from the state change function.      
+ * Elements that return %GST_STATE_CHANGE_ASYNC from the change_state function
+ * should eventually call this method from the streaming thread to signal       
  * successfull state change completion.          
  *       
  * If after calling this method the element still has not reached        
@@ -1877,7 +1936,7 @@ gst_element_continue_state (GstElement * element, GstStateChangeReturn ret)
   GstStateChange transition;
 
   GST_OBJECT_LOCK (element);
-  old_ret = GST_STATE_RETURN (element);
+  old_ret = (GstState) GST_STATE_RETURN (element);
   GST_STATE_RETURN (element) = ret;
   pending = GST_STATE_PENDING (element);
 
@@ -1896,7 +1955,7 @@ gst_element_continue_state (GstElement * element, GstStateChangeReturn ret)
     goto complete;
 
   next = GST_STATE_GET_NEXT (current, pending);
-  transition = GST_STATE_TRANSITION (current, next);
+  transition = (GstStateChange) GST_STATE_TRANSITION (current, next);
 
   GST_STATE_NEXT (element) = next;
   /* mark busy */
@@ -1909,7 +1968,7 @@ gst_element_continue_state (GstElement * element, GstStateChangeReturn ret)
       gst_element_state_get_name (old_next),
       gst_element_state_get_name (pending));
 
-  message = gst_message_new_state_changed (GST_OBJECT (element),
+  message = gst_message_new_state_changed (GST_OBJECT_CAST (element),
       old_state, old_next, pending);
   gst_element_post_message (element, message);
 
@@ -1943,7 +2002,7 @@ complete:
      * We do signal the cond though as a _get_state() might be blocking 
      * on it. */
     if (old_state != old_next || old_ret == GST_STATE_CHANGE_ASYNC) {
-      message = gst_message_new_state_changed (GST_OBJECT (element),
+      message = gst_message_new_state_changed (GST_OBJECT_CAST (element),
           old_state, old_next, GST_STATE_VOID_PENDING);
       gst_element_post_message (element, message);
     }
@@ -1960,11 +2019,11 @@ complete:
  *
  * Brings the element to the lost state. The current state of the
  * element is copied to the pending state so that any call to
- * gst_element_get_state() will return ASYNC.
+ * gst_element_get_state() will return %GST_STATE_CHANGE_ASYNC.
  *
  * This is mostly used for elements that lost their preroll buffer
- * in the PAUSED state after a flush, they become PAUSED again
- * if a new preroll buffer is queued.
+ * in the %GST_STATE_PAUSED state after a flush, they become %GST_STATE_PAUSED
+ * again if a new preroll buffer is queued.
  * This function can only be called when the element is currently
  * not in error or an async state change.
  *
@@ -1995,12 +2054,12 @@ gst_element_lost_state (GstElement * element)
   GST_STATE_RETURN (element) = GST_STATE_CHANGE_ASYNC;
   GST_OBJECT_UNLOCK (element);
 
-  message = gst_message_new_state_changed (GST_OBJECT (element),
+  message = gst_message_new_state_changed (GST_OBJECT_CAST (element),
       current_state, current_state, current_state);
   gst_element_post_message (element, message);
 
   /* and mark us dirty */
-  message = gst_message_new_state_dirty (GST_OBJECT (element));
+  message = gst_message_new_state_dirty (GST_OBJECT_CAST (element));
   gst_element_post_message (element, message);
 
   return;
@@ -2088,10 +2147,11 @@ gst_element_set_state_func (GstElement * element, GstState state)
   GST_STATE_PENDING (element) = state;
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
-      "current %s, old_pending %s, next %s, old return %d",
+      "current %s, old_pending %s, next %s, old return %s",
       gst_element_state_get_name (current),
       gst_element_state_get_name (old_pending),
-      gst_element_state_get_name (next), old_ret);
+      gst_element_state_get_name (next),
+      gst_element_state_change_return_get_name (old_ret));
 
   /* if the element was busy doing a state change, we just update the
    * target state, it'll get to it async then. */
@@ -2120,7 +2180,7 @@ gst_element_set_state_func (GstElement * element, GstState state)
   if (current != next)
     GST_STATE_RETURN (element) = GST_STATE_CHANGE_ASYNC;
 
-  transition = GST_STATE_TRANSITION (current, next);
+  transition = (GstStateChange) GST_STATE_TRANSITION (current, next);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
       "%s: setting state from %s to %s",
@@ -2136,7 +2196,8 @@ gst_element_set_state_func (GstElement * element, GstState state)
 
   GST_STATE_UNLOCK (element);
 
-  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "returned %d", ret);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "returned %s",
+      gst_element_state_change_return_get_name (ret));
 
   return ret;
 
@@ -2165,7 +2226,7 @@ gst_element_change_state (GstElement * element, GstStateChange transition)
   oclass = GST_ELEMENT_GET_CLASS (element);
 
   /* start with the current state. */
-  current = GST_STATE_TRANSITION_CURRENT (transition);
+  current = (GstState) GST_STATE_TRANSITION_CURRENT (transition);
   next = GST_STATE_TRANSITION_NEXT (transition);
 
   /* call the state change function so it can set the state */
@@ -2242,88 +2303,126 @@ invalid_return:
   }
 }
 
-/* gst_iterator_fold functions for pads_activate */
-
+/* gst_iterator_fold functions for pads_activate
+ * Note how we don't stop the iterator when we fail an activation. This is
+ * probably a FIXME since when one pad activation fails, we don't want to
+ * continue our state change. */
 static gboolean
 activate_pads (GstPad * pad, GValue * ret, gboolean * active)
 {
   if (!gst_pad_set_active (pad, *active))
     g_value_set_boolean (ret, FALSE);
-  else if (!*active)
-    gst_pad_set_caps (pad, NULL);
 
+  /* unref the object that was reffed for us by _fold */
   gst_object_unref (pad);
   return TRUE;
 }
 
-/* returns false on error or early cutout of the fold, true otherwise */
+/* set the caps on the pad to NULL */
 static gboolean
-iterator_fold_with_resync (GstIterator * iter, GstIteratorFoldFunction func,
-    GValue * ret, gpointer user_data)
+clear_caps (GstPad * pad, GValue * ret, gboolean * active)
 {
-  GstIteratorResult ires;
-  gboolean res = TRUE;
-
-  while (1) {
-    ires = gst_iterator_fold (iter, func, ret, user_data);
-
-    switch (ires) {
-      case GST_ITERATOR_RESYNC:
-        gst_iterator_resync (iter);
-        break;
-      case GST_ITERATOR_DONE:
-        res = TRUE;
-        goto done;
-      default:
-        res = FALSE;
-        goto done;
-    }
-  }
-
-done:
-  return res;
+  gst_pad_set_caps (pad, NULL);
+  gst_object_unref (pad);
+  return TRUE;
 }
 
-/* is called with STATE_LOCK
- */
+/* returns false on error or early cutout (will never happen because the fold
+ * function always returns TRUE, see FIXME above) of the fold, true if all
+ * pads in @iter were (de)activated successfully. */
 static gboolean
-gst_element_pads_activate (GstElement * element, gboolean active)
+iterator_activate_fold_with_resync (GstIterator * iter,
+    GstIteratorFoldFunction func, gpointer user_data)
 {
-  GValue ret = { 0, };
-  GstIterator *iter;
-  gboolean fold_ok;
+  GstIteratorResult ires;
+  GValue ret = { 0 };
 
-  GST_DEBUG_OBJECT (element, "pads_activate with active %d", active);
   /* no need to unset this later, it's just a boolean */
   g_value_init (&ret, G_TYPE_BOOLEAN);
   g_value_set_boolean (&ret, TRUE);
 
-  if (active)
-    iter = gst_element_iterate_src_pads (element);
-  else
-    iter = gst_element_iterate_sink_pads (element);
-  fold_ok = iterator_fold_with_resync
-      (iter, (GstIteratorFoldFunction) activate_pads, &ret, &active);
-  gst_iterator_free (iter);
-  if (!fold_ok || !g_value_get_boolean (&ret)) {
-    GST_DEBUG_OBJECT (element, "pads_activate failed");
-    return FALSE;
+  while (1) {
+    ires = gst_iterator_fold (iter, func, &ret, user_data);
+    switch (ires) {
+      case GST_ITERATOR_RESYNC:
+        /* need to reset the result again */
+        g_value_set_boolean (&ret, TRUE);
+        gst_iterator_resync (iter);
+        break;
+      case GST_ITERATOR_DONE:
+        /* all pads iterated, return collected value */
+        goto done;
+      default:
+        /* iterator returned _ERROR or premature end with _OK, 
+         * mark an error and exit */
+        g_value_set_boolean (&ret, FALSE);
+        goto done;
+    }
   }
+done:
+  /* return collected value */
+  return g_value_get_boolean (&ret);
+}
 
-  if (active)
-    iter = gst_element_iterate_sink_pads (element);
-  else
-    iter = gst_element_iterate_src_pads (element);
-  fold_ok = iterator_fold_with_resync
-      (iter, (GstIteratorFoldFunction) activate_pads, &ret, &active);
+/* is called with STATE_LOCK
+ *
+ * Pads are activated from source pads to sinkpads. 
+ */
+static gboolean
+gst_element_pads_activate (GstElement * element, gboolean active)
+{
+  GstIterator *iter;
+  gboolean res;
+
+  GST_DEBUG_OBJECT (element, "pads_activate with active %d", active);
+
+  iter = gst_element_iterate_src_pads (element);
+  res =
+      iterator_activate_fold_with_resync (iter,
+      (GstIteratorFoldFunction) activate_pads, &active);
   gst_iterator_free (iter);
-  if (!fold_ok || !g_value_get_boolean (&ret)) {
-    GST_DEBUG_OBJECT (element, "pads_activate failed");
-    return FALSE;
+  if (G_UNLIKELY (!res))
+    goto src_failed;
+
+  iter = gst_element_iterate_sink_pads (element);
+  res =
+      iterator_activate_fold_with_resync (iter,
+      (GstIteratorFoldFunction) activate_pads, &active);
+  gst_iterator_free (iter);
+  if (G_UNLIKELY (!res))
+    goto sink_failed;
+
+  if (!active) {
+    /* clear the caps on all pads, this should never fail */
+    iter = gst_element_iterate_pads (element);
+    res =
+        iterator_activate_fold_with_resync (iter,
+        (GstIteratorFoldFunction) clear_caps, &active);
+    gst_iterator_free (iter);
+    if (G_UNLIKELY (!res))
+      goto caps_failed;
   }
 
   GST_DEBUG_OBJECT (element, "pads_activate successful");
+
   return TRUE;
+
+  /* ERRORS */
+src_failed:
+  {
+    GST_DEBUG_OBJECT (element, "source pads_activate failed");
+    return FALSE;
+  }
+sink_failed:
+  {
+    GST_DEBUG_OBJECT (element, "sink pads_activate failed");
+    return FALSE;
+  }
+caps_failed:
+  {
+    GST_DEBUG_OBJECT (element, "failed to clear caps on pads");
+    return FALSE;
+  }
 }
 
 /* is called with STATE_LOCK */
@@ -2335,7 +2434,7 @@ gst_element_change_state_func (GstElement * element, GstStateChange transition)
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), GST_STATE_CHANGE_FAILURE);
 
-  state = GST_STATE_TRANSITION_CURRENT (transition);
+  state = (GstState) GST_STATE_TRANSITION_CURRENT (transition);
   next = GST_STATE_TRANSITION_NEXT (transition);
 
   /* if the element already is in the given state, we just return success */
@@ -2416,20 +2515,17 @@ static void
 gst_element_dispose (GObject * object)
 {
   GstElement *element = GST_ELEMENT (object);
+  GstClock **clock_p;
+  GstBus **bus_p;
 
   GST_CAT_INFO_OBJECT (GST_CAT_REFCOUNTING, element, "dispose");
 
-  if (GST_STATE (element) != GST_STATE_NULL) {
-    g_critical
-        ("\nTrying to dispose element %s, but it is not in the NULL state.\n"
-        "You need to explicitly set elements to the NULL state before\n"
-        "dropping the final reference, to allow them to clean up.\n",
-        GST_OBJECT_NAME (element));
-    return;
-  }
+  if (GST_STATE (element) != GST_STATE_NULL)
+    goto not_null;
+
   g_return_if_fail (GST_STATE_PENDING (element) == GST_STATE_VOID_PENDING);
 
-  GST_DEBUG ("removing %d pads", g_list_length (element->pads));
+  GST_DEBUG_OBJECT (element, "removing %d pads", g_list_length (element->pads));
   /* first we break all our links with the outside */
   while (element->pads && element->pads->data) {
     /* don't call _remove_pad with NULL */
@@ -2441,13 +2537,28 @@ gst_element_dispose (GObject * object)
   }
 
   GST_OBJECT_LOCK (element);
-  gst_object_replace ((GstObject **) & element->clock, NULL);
-  gst_object_replace ((GstObject **) & element->bus, NULL);
+  clock_p = &element->clock;
+  bus_p = &element->bus;
+  gst_object_replace ((GstObject **) clock_p, NULL);
+  gst_object_replace ((GstObject **) bus_p, NULL);
   GST_OBJECT_UNLOCK (element);
 
   GST_CAT_INFO_OBJECT (GST_CAT_REFCOUNTING, element, "parent class dispose");
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
+
+  return;
+
+  /* ERRORS */
+not_null:
+  {
+    g_critical
+        ("\nTrying to dispose element %s, but it is not in the NULL state.\n"
+        "You need to explicitly set elements to the NULL state before\n"
+        "dropping the final reference, to allow them to clean up.\n",
+        GST_OBJECT_NAME (element));
+    return;
+  }
 }
 
 static void
@@ -2545,13 +2656,13 @@ gst_element_save_thyself (GstObject * object, xmlNodePtr parent)
   pads = GST_ELEMENT_PADS (element);
 
   while (pads) {
-    GstPad *pad = GST_PAD (pads->data);
+    GstPad *pad = GST_PAD_CAST (pads->data);
 
     /* figure out if it's a direct pad or a ghostpad */
     if (GST_ELEMENT (GST_OBJECT_PARENT (pad)) == element) {
       xmlNodePtr padtag = xmlNewChild (parent, NULL, (xmlChar *) "pad", NULL);
 
-      gst_object_save_thyself (GST_OBJECT (pad), padtag);
+      gst_object_save_thyself (GST_OBJECT_CAST (pad), padtag);
     }
     pads = g_list_next (pads);
   }
@@ -2597,7 +2708,7 @@ gst_element_restore_thyself (GstObject * object, xmlNodePtr self)
   children = self->xmlChildrenNode;
   while (children) {
     if (!strcmp ((char *) children->name, "pad")) {
-      gst_pad_load_and_link (children, GST_OBJECT (element));
+      gst_pad_load_and_link (children, GST_OBJECT_CAST (element));
     }
     children = children->next;
   }
@@ -2610,13 +2721,15 @@ gst_element_restore_thyself (GstObject * object, xmlNodePtr self)
 static void
 gst_element_set_bus_func (GstElement * element, GstBus * bus)
 {
+  GstBus **bus_p;
+
   g_return_if_fail (GST_IS_ELEMENT (element));
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_PARENTAGE, element, "setting bus to %p", bus);
 
   GST_OBJECT_LOCK (element);
-  gst_object_replace ((GstObject **) & GST_ELEMENT_BUS (element),
-      GST_OBJECT_CAST (bus));
+  bus_p = &GST_ELEMENT_BUS (element);
+  gst_object_replace ((GstObject **) bus_p, GST_OBJECT_CAST (bus));
   GST_OBJECT_UNLOCK (element);
 }
 
@@ -2625,8 +2738,8 @@ gst_element_set_bus_func (GstElement * element, GstBus * bus)
  * @element: a #GstElement to set the bus of.
  * @bus: the #GstBus to set.
  *
- * Sets the bus of the element.  For internal use only, unless you're
- * testing elements.
+ * Sets the bus of the element. Increases the refcount on the bus.
+ * For internal use only, unless you're testing elements.
  *
  * MT safe.
  */
@@ -2661,8 +2774,8 @@ gst_element_get_bus (GstElement * element)
   g_return_val_if_fail (GST_IS_ELEMENT (element), result);
 
   GST_OBJECT_LOCK (element);
-  result = GST_ELEMENT_BUS (element);
-  gst_object_ref (result);
+  if ((result = GST_ELEMENT_BUS (element)))
+    gst_object_ref (result);
   GST_OBJECT_UNLOCK (element);
 
   GST_DEBUG_OBJECT (element, "got bus %" GST_PTR_FORMAT, result);
