@@ -77,9 +77,12 @@ G_BEGIN_DECLS
  * @GST_STATE_CHANGE_FAILURE   : the state change failed
  * @GST_STATE_CHANGE_SUCCESS   : the state change succeeded
  * @GST_STATE_CHANGE_ASYNC     : the state change will happen asynchronously
- * @GST_STATE_CHANGE_NO_PREROLL: the state change cannot be prerolled
+ * @GST_STATE_CHANGE_NO_PREROLL: the state change succeeded but the element cannot
+ *                               produce data in PAUSED. This typically happens
+ *                               with live sources.
  *
- * the possible return values from a state change function.
+ * The possible return values from a state change function. Only
+ * @GST_STATE_CHANGE_FAILURE is a real failure.
  */
 typedef enum {
   GST_STATE_CHANGE_FAILURE             = 0,
@@ -142,21 +145,21 @@ typedef enum {
  * Given a current state @cur and a next state @next, calculate the associated
  * #GstStateChange transition.
  */
-#define GST_STATE_TRANSITION(cur,next)		(((cur)<<3)|(next))
+#define GST_STATE_TRANSITION(cur,next)		((GstStateChange)(((cur)<<3)|(next)))
 /**
  * GST_STATE_TRANSITION_CURRENT:
  * @trans: A #GstStateChange
  *
  * Given a state transition @trans, extract the current #GstState.
  */
-#define GST_STATE_TRANSITION_CURRENT(trans)	((trans)>>3)
+#define GST_STATE_TRANSITION_CURRENT(trans)	((GstState)((trans)>>3))
 /**
  * GST_STATE_TRANSITION_NEXT:
  * @trans: A #GstStateChange
  *
  * Given a state transition @trans, extract the next #GstState.
  */
-#define GST_STATE_TRANSITION_NEXT(trans)	((trans)&0x7)
+#define GST_STATE_TRANSITION_NEXT(trans)	((GstState)((trans)&0x7))
 
 /**
  * GstStateChange:
@@ -570,7 +573,12 @@ gboolean		gst_element_query		(GstElement *element, GstQuery *query);
 gboolean		gst_element_post_message	(GstElement * element, GstMessage * message);
 
 /* error handling */
+/* gcc versions < 3.3 warn about NULL being passed as format to printf */
+#if (defined(GST_USING_PRINTF_EXTENSION) || !defined(__GNUC__) || (__GNUC__ < 3) || (__GNUC__ == 3 && __GNUC_MINOR__ < 3))
 gchar *			_gst_element_error_printf	(const gchar *format, ...);
+#else
+gchar *			_gst_element_error_printf	(const gchar *format, ...) G_GNUC_PRINTF (1, 2);
+#endif
 void			gst_element_message_full	(GstElement * element, GstMessageType type,
 							 GQuark domain, gint code, gchar * text,
 							 gchar * debug, const gchar * file,
