@@ -72,7 +72,7 @@
  *   </programlisting>
  * </example>
  *
- * Last reviewed on 2006-01-24 (0.10.2)
+ * Last reviewed on 2006-09-6 (0.10.10)
  */
 
 
@@ -99,8 +99,8 @@ _gst_event_initialize (void)
 
 typedef struct
 {
-  gint type;
-  gchar *name;
+  const gint type;
+  const gchar *name;
   GQuark quark;
 } GstEventQuarks;
 
@@ -185,7 +185,7 @@ gst_event_type_get_flags (GstEventType type)
 GType
 gst_event_get_type (void)
 {
-  static GType _gst_event_type;
+  static GType _gst_event_type = 0;
   int i;
 
   if (G_UNLIKELY (_gst_event_type == 0)) {
@@ -269,7 +269,7 @@ _gst_event_copy (GstEvent * event)
   if (event->structure) {
     copy->structure = gst_structure_copy (event->structure);
     gst_structure_set_parent_refcount (copy->structure,
-        &event->mini_object.refcount);
+        &copy->mini_object.refcount);
   }
   return copy;
 }
@@ -315,6 +315,10 @@ gst_event_new_custom (GstEventType type, GstStructure * structure)
 {
   GstEvent *event;
 
+  /* structure must not have a parent */
+  if (structure)
+    g_return_val_if_fail (structure->parent_refcount == NULL, NULL);
+
   event = gst_event_new (type);
   if (structure) {
     gst_structure_set_parent_refcount (structure, &event->mini_object.refcount);
@@ -348,7 +352,12 @@ gst_event_get_structure (GstEvent * event)
  *
  * Allocate a new flush start event. The flush start event can be send
  * upstream and downstream and travels out-of-bounds with the dataflow.
- * It marks pads as being in a WRONG_STATE to process more data.
+ *
+ * It marks pads as being flushing and will make them return
+ * #GST_FLOW_WRONG_STATE when used for data flow with gst_pad_push(),
+ * gst_pad_chain(), gst_pad_alloc_buffer(), gst_pad_get_range() and
+ * gst_pad_pull_range(). Any event (except a #GST_EVENT_FLUSH_STOP) received
+ * on a flushing pad will return %FALSE immediatly.
  *
  * Elements unlock and blocking functions and exit their streaming functions
  * as fast as possible.
@@ -391,11 +400,11 @@ gst_event_new_flush_stop (void)
  *
  * Create a new EOS event. The eos event can only travel downstream
  * synchronized with the buffer flow. Elements that receive the EOS
- * event on a pad can return UNEXPECTED as a GstFlowReturn when data
- * after the EOS event arrives.
+ * event on a pad can return #GST_FLOW_UNEXPECTED as a #GstFlowReturn
+ * when data after the EOS event arrives.
  *
  * The EOS event will travel down to the sink elements in the pipeline
- * which will then post the GST_MESSAGE_EOS on the bus after they have
+ * which will then post the #GST_MESSAGE_EOS on the bus after they have
  * finished playing any buffered data.
  *
  * When all sinks have posted an EOS message, the EOS message is
@@ -473,7 +482,7 @@ gst_event_parse_new_segment (GstEvent * event, gboolean * update,
  * The newsegment event marks the range of buffers to be processed. All
  * data not within the segment range is not to be processed. This can be
  * used intelligently by plugins to apply more efficient methods of skipping
- * unneeded packets.
+ * unneeded data.
  *
  * The position value of the segment is used in conjunction with the start
  * value to convert the buffer timestamps into the stream time. This is 
@@ -483,7 +492,7 @@ gst_event_parse_new_segment (GstEvent * event, gboolean * update,
  *
  * @start cannot be -1, @stop can be -1. If there
  * is a valid @stop given, it must be greater or equal the @start, including 
- * when the indicated playback @rate is < 0
+ * when the indicated playback @rate is < 0.
  *
  * The @applied_rate value provides information about any rate adjustment that
  * has already been made to the timestamps and content on the buffers of the 
@@ -599,6 +608,7 @@ GstEvent *
 gst_event_new_tag (GstTagList * taglist)
 {
   g_return_val_if_fail (taglist != NULL, NULL);
+
   return gst_event_new_custom (GST_EVENT_TAG, (GstStructure *) taglist);
 }
 
@@ -614,6 +624,7 @@ gst_event_parse_tag (GstEvent * event, GstTagList ** taglist)
 {
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_TAG);
+
   if (taglist)
     *taglist = (GstTagList *) event->structure;
 }
@@ -629,7 +640,7 @@ gst_event_parse_tag (GstEvent * event, GstTagList ** taglist)
  * Create a new buffersize event. The event is sent downstream and notifies
  * elements that they should provide a buffer of the specified dimensions.
  *
- * When the async flag is set, a thread boundary is prefered.
+ * When the @async flag is set, a thread boundary is prefered.
  *
  * Returns: a new #GstEvent
  */
@@ -641,6 +652,7 @@ gst_event_new_buffer_size (GstFormat format, gint64 minsize,
       "creating buffersize format %d, minsize %" G_GINT64_FORMAT
       ", maxsize %" G_GINT64_FORMAT ", async %d", format,
       minsize, maxsize, async);
+
   return gst_event_new_custom (GST_EVENT_BUFFERSIZE,
       gst_structure_new ("GstEventBufferSize",
           "format", GST_TYPE_FORMAT, format,
@@ -667,6 +679,7 @@ gst_event_parse_buffer_size (GstEvent * event, GstFormat * format,
 
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_BUFFERSIZE);
+
   structure = gst_event_get_structure (event);
   if (format)
     *format = g_value_get_enum (gst_structure_get_value (structure, "format"));
@@ -690,24 +703,31 @@ gst_event_parse_buffer_size (GstEvent * event, GstFormat * format,
  * The QOS event is generated in an element that wants an upstream
  * element to either reduce or increase its rate because of
  * high/low CPU load or other resource usage such as network performance.
+ * Typically sinks generate these events for each buffer they receive.
  *
- * @proportion is the requested adjustment in datarate, 1.0 is the normal
- * datarate, 0.75 means increase datarate by 75%, 1.5 is 150%. Negative
- * values request a slow down, so -0.75 means a decrease by 75%.
+ * @proportion indicates the real-time performance of the streaming in the
+ * element that generated the QoS event (usually the sink). The value is
+ * generally computed based on more long term statistics about the streams
+ * timestamps compared to the clock.
+ * A value < 1.0 indicates that the upstream element is producing data faster
+ * than real-time. A value > 1.0 indicates that the upstream element is not
+ * producing data fast enough. 1.0 is the ideal @proportion value. The
+ * proportion value can safely be used to lower or increase the quality of
+ * the element.
  *
- * @diff is the difference against the clock in stream time of the last
- * buffer that caused the element to generate the QOS event.
+ * @diff is the difference against the clock in running time of the last
+ * buffer that caused the element to generate the QOS event. A negative value
+ * means that the buffer with @timestamp arrived in time. A positive value
+ * indicates how late the buffer with @timestamp was.
  *
  * @timestamp is the timestamp of the last buffer that cause the element
- * to generate the QOS event. 
+ * to generate the QOS event. It is expressed in running time and thus an ever
+ * increasing value.
  *
  * The upstream element can use the @diff and @timestamp values to decide
- * whether to process more buffers. All buffers with timestamp <=
- * @timestamp + @diff will certainly arrive late in the sink as well. 
- *
- * The @proportion value is generally computed based on more long
- * term statistics about the stream quality and can be used in various ways
- * such as lowering or increasing processing quality.
+ * whether to process more buffers. For possitive @diff, all buffers with
+ * timestamp <= @timestamp + @diff will certainly arrive late in the sink
+ * as well. 
  *
  * The application can use general event probes to intercept the QoS
  * event and implement custom application specific QoS handling.
@@ -737,7 +757,8 @@ gst_event_new_qos (gdouble proportion, GstClockTimeDiff diff,
  * @diff: A pointer to store the diff in
  * @timestamp: A pointer to store the timestamp in
  *
- * Get the proportion, diff and timestamp in the qos event.
+ * Get the proportion, diff and timestamp in the qos event. See
+ * gst_event_new_qos() for more information about the different QoS values.
  */
 void
 gst_event_parse_qos (GstEvent * event, gdouble * proportion,
@@ -747,6 +768,7 @@ gst_event_parse_qos (GstEvent * event, gdouble * proportion,
 
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_QOS);
+
   structure = gst_event_get_structure (event);
   if (proportion)
     *proportion =
@@ -763,59 +785,72 @@ gst_event_parse_qos (GstEvent * event, gdouble * proportion,
  * @rate: The new playback rate
  * @format: The format of the seek values
  * @flags: The optional seek flags
- * @cur_type: The type and flags for the new current position
- * @cur: The value of the new current position
+ * @start_type: The type and flags for the new start position
+ * @start: The value of the new start position
  * @stop_type: The type and flags for the new stop position
  * @stop: The value of the new stop position
  *
  * Allocate a new seek event with the given parameters.
  *
- * The seek event configures playback of the pipeline from
- * @cur to @stop at the speed given in @rate, also called a segment.
- * The @cur and @stop values are expressed in format @format.
+ * The seek event configures playback of the pipeline between @start to @stop
+ * at the speed given in @rate, also called a playback segment.
+ * The @start and @stop values are expressed in @format.
  *
  * A @rate of 1.0 means normal playback rate, 2.0 means double speed.
  * Negatives values means backwards playback. A value of 0.0 for the
- * rate is not allowed.
+ * rate is not allowed and should be accomplished instead by PAUSING the
+ * pipeline.
  *
- * @cur_type and @stop_type specify how to adjust the current and stop
- * time, relative or absolute. A type of #GST_SEEK_TYPE_NONE means that
- * the position should not be updated. The currently configured playback
- * segment can be queried with #GST_QUERY_SEGMENT.
+ * A pipeline has a default playback segment configured with a start
+ * position of 0, a stop position of -1 and a rate of 1.0. The currently
+ * configured playback segment can be queried with #GST_QUERY_SEGMENT. 
  *
- * Note that updating the @cur position will actually move the current
- * playback pointer to that new position. It is not possible to seek 
- * relative to the current playing position, to do this, pause the pipeline,
- * get the current position and perform a GST_SEEK_TYPE_SET to the desired
- * position.
+ * @start_type and @stop_type specify how to adjust the currently configured 
+ * start and stop fields in @segment. Adjustments can be made relative or
+ * absolute to the last configured values. A type of #GST_SEEK_TYPE_NONE means
+ * that the position should not be updated.
+ *
+ * When the rate is positive and @start has been updated, playback will start
+ * from the newly configured start position. 
+ *
+ * For negative rates, playback will start from the newly configured stop
+ * position (if any). If the stop position if updated, it must be different from
+ * -1 for negative rates.
+ *
+ * It is not possible to seek relative to the current playback position, to do
+ * this, PAUSE the pipeline, query the current playback position with
+ * #GST_QUERY_POSITION and update the playback segment current position with a
+ * #GST_SEEK_TYPE_SET to the desired position.
  *
  * Returns: A new seek event.
  */
 GstEvent *
 gst_event_new_seek (gdouble rate, GstFormat format, GstSeekFlags flags,
-    GstSeekType cur_type, gint64 cur, GstSeekType stop_type, gint64 stop)
+    GstSeekType start_type, gint64 start, GstSeekType stop_type, gint64 stop)
 {
+  g_return_val_if_fail (rate != 0.0, NULL);
+
   if (format == GST_FORMAT_TIME) {
     GST_CAT_INFO (GST_CAT_EVENT,
         "creating seek rate %lf, format TIME, flags %d, "
-        "cur_type %d, cur %" GST_TIME_FORMAT ", "
+        "start_type %d, start %" GST_TIME_FORMAT ", "
         "stop_type %d, stop %" GST_TIME_FORMAT,
-        rate, flags, cur_type, GST_TIME_ARGS (cur),
+        rate, flags, start_type, GST_TIME_ARGS (start),
         stop_type, GST_TIME_ARGS (stop));
   } else {
     GST_CAT_INFO (GST_CAT_EVENT,
         "creating seek rate %lf, format %d, flags %d, "
-        "cur_type %d, cur %" G_GINT64_FORMAT ", "
+        "start_type %d, start %" G_GINT64_FORMAT ", "
         "stop_type %d, stop %" G_GINT64_FORMAT,
-        rate, format, flags, cur_type, cur, stop_type, stop);
+        rate, format, flags, start_type, start, stop_type, stop);
   }
 
   return gst_event_new_custom (GST_EVENT_SEEK,
       gst_structure_new ("GstEventSeek", "rate", G_TYPE_DOUBLE, rate,
           "format", GST_TYPE_FORMAT, format,
           "flags", GST_TYPE_SEEK_FLAGS, flags,
-          "cur_type", GST_TYPE_SEEK_TYPE, cur_type,
-          "cur", G_TYPE_INT64, cur,
+          "cur_type", GST_TYPE_SEEK_TYPE, start_type,
+          "cur", G_TYPE_INT64, start,
           "stop_type", GST_TYPE_SEEK_TYPE, stop_type,
           "stop", G_TYPE_INT64, stop, NULL));
 }
@@ -826,8 +861,8 @@ gst_event_new_seek (gdouble rate, GstFormat format, GstSeekFlags flags,
  * @rate: result location for the rate
  * @format: result location for the stream format
  * @flags:  result location for the #GstSeekFlags
- * @cur_type: result location for the #GstSeekType of the current position
- * @cur: result location for the current postion expressed in @format
+ * @start_type: result location for the #GstSeekType of the start position
+ * @start: result location for the start postion expressed in @format
  * @stop_type:  result location for the #GstSeekType of the stop position
  * @stop: result location for the stop postion expressed in @format
  *
@@ -835,13 +870,14 @@ gst_event_new_seek (gdouble rate, GstFormat format, GstSeekFlags flags,
  */
 void
 gst_event_parse_seek (GstEvent * event, gdouble * rate,
-    GstFormat * format, GstSeekFlags * flags, GstSeekType * cur_type,
-    gint64 * cur, GstSeekType * stop_type, gint64 * stop)
+    GstFormat * format, GstSeekFlags * flags, GstSeekType * start_type,
+    gint64 * start, GstSeekType * stop_type, gint64 * stop)
 {
   const GstStructure *structure;
 
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_SEEK);
+
   structure = gst_event_get_structure (event);
   if (rate)
     *rate = g_value_get_double (gst_structure_get_value (structure, "rate"));
@@ -849,11 +885,11 @@ gst_event_parse_seek (GstEvent * event, gdouble * rate,
     *format = g_value_get_enum (gst_structure_get_value (structure, "format"));
   if (flags)
     *flags = g_value_get_flags (gst_structure_get_value (structure, "flags"));
-  if (cur_type)
-    *cur_type =
+  if (start_type)
+    *start_type =
         g_value_get_enum (gst_structure_get_value (structure, "cur_type"));
-  if (cur)
-    *cur = g_value_get_int64 (gst_structure_get_value (structure, "cur"));
+  if (start)
+    *start = g_value_get_int64 (gst_structure_get_value (structure, "cur"));
   if (stop_type)
     *stop_type =
         g_value_get_enum (gst_structure_get_value (structure, "stop_type"));
@@ -873,5 +909,6 @@ GstEvent *
 gst_event_new_navigation (GstStructure * structure)
 {
   g_return_val_if_fail (structure != NULL, NULL);
+
   return gst_event_new_custom (GST_EVENT_NAVIGATION, structure);
 }
