@@ -48,7 +48,7 @@ G_BEGIN_DECLS
 typedef enum {
   GST_EVENT_TYPE_UPSTREAM	= 1 << 0,
   GST_EVENT_TYPE_DOWNSTREAM	= 1 << 1,
-  GST_EVENT_TYPE_SERIALIZED	= 1 << 2,
+  GST_EVENT_TYPE_SERIALIZED	= 1 << 2
 } GstEventTypeFlags;
 
 /**
@@ -230,10 +230,10 @@ typedef struct _GstEventClass GstEventClass;
  * @GST_SEEK_TYPE_SET: absolute position is requested
  * @GST_SEEK_TYPE_END: relative position to duration is requested
  *
- * The different types of seek events. When constructing a seek event, a format,
- * a seek method and optional flags are to be provided. The seek event is then
- * inserted into the graph with gst_pad_send_event() or
- * gst_element_send_event().
+ * The different types of seek events. When constructing a seek event with
+ * gst_event_new_seek(), a format, a seek method and optional flags are to
+ * be provided. The seek event is then inserted into the graph with
+ * gst_pad_send_event() or gst_element_send_event().
  */
 typedef enum {
   /* one of these */
@@ -253,7 +253,8 @@ typedef enum {
  *		       faster but less accurate.
  * @GST_SEEK_FLAG_SEGMENT: perform a segment seek.
  *
- * Flags to be used with gst_element_seek() or gst_event_new_seek()
+ * Flags to be used with gst_element_seek() or gst_event_new_seek(). All flags
+ * can be used together.
  *
  * A non flushing seek might take some time to perform as the currently
  * playing data in the pipeline will not be cleared.
@@ -264,10 +265,10 @@ typedef enum {
  *
  * When performing a segment seek: after the playback of the segment completes,
  * no EOS will be emmited by the element that performed the seek, but a
- * SEGMENT_DONE message will be posted on the bus by the element. When this
- * message is posted, it is possible to send a new seek event to continue
- * playback. With this seek method it is possible to perform seemless looping
- * or simple linear editing.
+ * #GST_MESSAGE_SEGMENT_DONE message will be posted on the bus by the element.
+ * When this message is posted, it is possible to send a new seek event to
+ * continue playback. With this seek method it is possible to perform seemless
+ * looping or simple linear editing.
  */
 typedef enum {
   GST_SEEK_FLAG_NONE		= 0,
@@ -401,7 +402,7 @@ void		gst_event_parse_tag		(GstEvent *event, GstTagList **taglist);
 /* buffer */
 GstEvent *	gst_event_new_buffer_size	(GstFormat format, gint64 minsize, gint64 maxsize,
 						 gboolean async);
-void 		gst_event_parse_buffer_size	(GstEvent *event, GstFormat *format, gint64 *minsize,
+void		gst_event_parse_buffer_size	(GstEvent *event, GstFormat *format, gint64 *minsize,
 						 gint64 *maxsize, gboolean *async);
 
 /* QOS events */
@@ -411,11 +412,11 @@ void		gst_event_parse_qos		(GstEvent *event, gdouble *proportion, GstClockTimeDi
 						 GstClockTime *timestamp);
 /* seek event */
 GstEvent*	gst_event_new_seek		(gdouble rate, GstFormat format, GstSeekFlags flags,
-						 GstSeekType cur_type, gint64 cur,
+						 GstSeekType start_type, gint64 start,
 						 GstSeekType stop_type, gint64 stop);
 void		gst_event_parse_seek		(GstEvent *event, gdouble *rate, GstFormat *format,
 		                                 GstSeekFlags *flags,
-						 GstSeekType *cur_type, gint64 *cur,
+						 GstSeekType *start_type, gint64 *start,
 						 GstSeekType *stop_type, gint64 *stop);
 /* navigation event */
 GstEvent*	gst_event_new_navigation	(GstStructure *structure);
