@@ -227,68 +227,84 @@ typedef struct _GstDebugMessage GstDebugMessage;
  * gst_debug_add_log_function().
  * Use G_GNUC_NO_INSTRUMENT on that function.
  */
-typedef void (*GstLogFunction)	(GstDebugCategory *	category,
-				 GstDebugLevel		level,
-				 const gchar *		file,
-				 const gchar *		function,
-				 gint			line,
-				 GObject *		object,
-				 GstDebugMessage *	message,
-				 gpointer		data);
+typedef void (*GstLogFunction)  (GstDebugCategory * category,
+                                 GstDebugLevel      level,
+                                 const gchar      * file,
+                                 const gchar      * function,
+                                 gint               line,
+                                 GObject          * object,
+                                 GstDebugMessage  * message,
+                                 gpointer           data);
 
 #ifndef GST_DISABLE_GST_DEBUG
 
-void		_gst_debug_init			(void);
+void            _gst_debug_init (void);
 
-/* note we can't use G_GNUC_PRINTF (7, 8) because gcc chokes on %P, which
- * we use for GST_PTR_FORMAT. */
-void		gst_debug_log		(GstDebugCategory *	category,
-					 GstDebugLevel		level,
-					 const gchar *		file,
-					 const gchar *		function,
-					 gint			line,
-					 GObject *		object,
-					 const gchar *		format,
-					 ...) G_GNUC_NO_INSTRUMENT;
-void		gst_debug_log_valist	(GstDebugCategory *	category,
-					 GstDebugLevel		level,
-					 const gchar *		file,
-					 const gchar *		function,
-					 gint			line,
-					 GObject *		object,
-					 const gchar *		format,
-					 va_list		args) G_GNUC_NO_INSTRUMENT;
 
-const gchar *	gst_debug_message_get	(GstDebugMessage *	message);
+#ifdef GST_USING_PRINTF_EXTENSION
 
-void		gst_debug_log_default	(GstDebugCategory *	category,
-					 GstDebugLevel		level,
-					 const gchar *		file,
-					 const gchar *		function,
-					 gint			line,
-					 GObject *		object,
-					 GstDebugMessage *	message,
-					 gpointer		unused) G_GNUC_NO_INSTRUMENT;
+/* not using G_GNUC_PRINTF, since gcc will choke on GST_PTR_FORMAT being %P */
+void		    gst_debug_log            (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint               line,
+                                          GObject          * object,
+                                          const gchar      * format,
+                                          ...) G_GNUC_NO_INSTRUMENT;
+
+#else /* GST_USING_PRINTF_EXTENSION */
+
+void		    gst_debug_log            (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint               line,
+                                          GObject          * object,
+                                          const gchar      * format,
+                                          ...) G_GNUC_PRINTF (7, 8) G_GNUC_NO_INSTRUMENT;
+
+#endif /* GST_USING_PRINTF_EXTENSION */
+
+void            gst_debug_log_valist     (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint	              line,
+                                          GObject          * object,
+                                          const gchar      * format,
+                                          va_list            args) G_GNUC_NO_INSTRUMENT;
+
+const gchar   * gst_debug_message_get    (GstDebugMessage  * message);
+
+void            gst_debug_log_default    (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint               line,
+                                          GObject          * object,
+                                          GstDebugMessage  * message,
+                                          gpointer           unused) G_GNUC_NO_INSTRUMENT;
 
 G_CONST_RETURN gchar *
-		gst_debug_level_get_name	(GstDebugLevel		level);
+                gst_debug_level_get_name (GstDebugLevel level);
 
-void		gst_debug_add_log_function	(GstLogFunction		func,
-						 gpointer data);
-guint		gst_debug_remove_log_function	(GstLogFunction		func);
-guint		gst_debug_remove_log_function_by_data (gpointer		data);
+void            gst_debug_add_log_function            (GstLogFunction func,
+                                                       gpointer       data);
+guint           gst_debug_remove_log_function         (GstLogFunction func);
+guint           gst_debug_remove_log_function_by_data (gpointer       data);
 
-void		gst_debug_set_active		(gboolean active);
-gboolean	gst_debug_is_active		(void);
+void            gst_debug_set_active  (gboolean active);
+gboolean        gst_debug_is_active   (void);
 
-void		gst_debug_set_colored		(gboolean colored);
-gboolean	gst_debug_is_colored		(void);
+void            gst_debug_set_colored (gboolean colored);
+gboolean        gst_debug_is_colored  (void);
 
-void		gst_debug_set_default_threshold	(GstDebugLevel		level);
-GstDebugLevel	gst_debug_get_default_threshold	(void);
-void		gst_debug_set_threshold_for_name (const gchar *		name,
-						 GstDebugLevel		level);
-void		gst_debug_unset_threshold_for_name (const gchar *	name);
+void            gst_debug_set_default_threshold	    (GstDebugLevel level);
+GstDebugLevel   gst_debug_get_default_threshold	    (void);
+void            gst_debug_set_threshold_for_name    (const gchar * name,
+                                                     GstDebugLevel level);
+void            gst_debug_unset_threshold_for_name  (const gchar * name);
 
 /**
  * GST_DEBUG_CATEGORY:
@@ -305,7 +321,13 @@ void		gst_debug_unset_threshold_for_name (const gchar *	name);
  * Declares a GstDebugCategory variable as extern. Use in header files.
  * This macro expands to nothing if debugging is disabled.
  */
+#ifndef WIN32
 #define GST_DEBUG_CATEGORY_EXTERN(cat) extern GstDebugCategory *cat
+#else
+#define GST_DEBUG_CATEGORY_EXTERN(cat) \
+  extern __declspec (dllimport) GstDebugCategory *cat;
+#endif
+
 /**
  * GST_DEBUG_CATEGORY_STATIC:
  * @cat: the category
@@ -384,6 +406,10 @@ GST_EXPORT GstDebugCategory *	GST_CAT_DEFAULT;
 /* this symbol may not be used */
 extern gboolean			__gst_debug_enabled;
 
+/* since 0.10.7, the min debug level, used for quickly discarding debug
+ * messages that fall under the threshold. */
+extern GstDebugLevel            __gst_debug_min; 
+
 /**
  * GST_CAT_LEVEL_LOG:
  * @cat: category to use
@@ -397,7 +423,7 @@ extern gboolean			__gst_debug_enabled;
  */
 #ifdef G_HAVE_ISO_VARARGS
 #define GST_CAT_LEVEL_LOG(cat,level,object,...) G_STMT_START{		\
-  if (__gst_debug_enabled) {						\
+  if (G_UNLIKELY (level <= __gst_debug_min)) {						\
     gst_debug_log ((cat), (level), __FILE__, GST_FUNCTION, __LINE__,	\
         (GObject *) (object), __VA_ARGS__);				\
   }									\
@@ -405,7 +431,7 @@ extern gboolean			__gst_debug_enabled;
 #else /* G_HAVE_GNUC_VARARGS */
 #ifdef G_HAVE_GNUC_VARARGS
 #define GST_CAT_LEVEL_LOG(cat,level,object,args...) G_STMT_START{	\
-  if (__gst_debug_enabled) {						\
+  if (G_UNLIKELY (level <= __gst_debug_min)) {						\
     gst_debug_log ((cat), (level), __FILE__, GST_FUNCTION, __LINE__,	\
         (GObject *) (object), ##args );					\
   }									\
@@ -423,7 +449,7 @@ static inline void
 GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
     gpointer object, const char *format, ...)
 {
-  if (__gst_debug_enabled) {
+  if (G_UNLIKELY (level <= __gst_debug_min)) {						\
     va_list varargs;
 
     va_start (varargs, format);
@@ -861,7 +887,7 @@ GST_LOG (const char *format, ...)
 
 typedef	void (* GstDebugFuncPtr)	(void);
 void	_gst_debug_register_funcptr	(GstDebugFuncPtr	func,
-					 gchar *		ptrname);
+					 const gchar *		ptrname);
 G_CONST_RETURN gchar *
 	_gst_debug_nameof_funcptr	(GstDebugFuncPtr	func);
 
@@ -877,12 +903,13 @@ G_CONST_RETURN gchar *
 
 /**
  * GST_DEBUG_FUNCPTR_NAME:
- * @ptr: pointer to the function to look up the name
+ * @ptr: address of the function of which to look up the name
  *
  * Retrieves the name of the function, if it was previously registered with
  * GST_DEBUG_FUNCPTR(). If not, it returns a description of the pointer.
  *
- * Make sure you free the string after use.
+ * This macro returns a constant string which must not be modified or
+ * freed by the caller.
  */
 #define GST_DEBUG_FUNCPTR_NAME(ptr) \
   _gst_debug_nameof_funcptr((GstDebugFuncPtr)ptr)
@@ -900,10 +927,11 @@ G_CONST_RETURN gchar *
 #define gst_debug_set_default_threshold(level)		/* NOP */
 #define gst_debug_get_default_threshold()		(GST_LEVEL_NONE)
 
-#define gst_debug_level_get_name(level)			("NONE")
+#define gst_debug_level_get_name(level)				("NONE")
+#define gst_debug_message_get(message)  			("NONE")
 #define gst_debug_add_log_function(func,data)		/* NOP */
-guint		gst_debug_remove_log_function	(GstLogFunction		func);
-guint		gst_debug_remove_log_function_by_data (gpointer		data);
+guint gst_debug_remove_log_function (GstLogFunction func);
+guint gst_debug_remove_log_function_by_data (gpointer data);
 #define gst_debug_set_active(active)			/* NOP */
 #define gst_debug_is_active()				(FALSE)
 #define gst_debug_set_colored(colored)			/* NOP */
