@@ -68,7 +68,7 @@ typedef enum {
   GST_PAD_LINK_WRONG_DIRECTION  = -3,
   GST_PAD_LINK_NOFORMAT         = -4,
   GST_PAD_LINK_NOSCHED          = -5,
-  GST_PAD_LINK_REFUSED          = -6,
+  GST_PAD_LINK_REFUSED          = -6
 } GstPadLinkReturn;
 
 /**
@@ -171,7 +171,7 @@ GQuark			gst_flow_to_quark	(GstFlowReturn ret);
 typedef enum {
   GST_ACTIVATE_NONE,
   GST_ACTIVATE_PUSH,
-  GST_ACTIVATE_PULL,
+  GST_ACTIVATE_PULL
 } GstActivateMode;
 
 /**
@@ -211,27 +211,62 @@ typedef gboolean		(*GstPadActivateModeFunction)	(GstPad *pad, gboolean active);
 /* data passing */
 /**
  * GstPadChainFunction:
- * @pad: the #GstPad that performed the chain.
- * @buffer: the #GstBuffer that is chained.
+ * @pad: the sink #GstPad that performed the chain.
+ * @buffer: the #GstBuffer that is chained, not %NULL.
  *
  * A function that will be called on sinkpads when chaining buffers.
+ * The function typically processes the data contained in the buffer and 
+ * either consumes the data or passes it on to the internally linked pad(s).
  *
- * Returns: GST_FLOW_OK for success
+ * The implementer of this function receives a refcount to @buffer and should
+ * gst_buffer_unref() when the buffer is no longer needed.
+ *
+ * When a chain function detects an error in the data stream, it must post an
+ * error on the buffer and return an appropriate #GstFlowReturn value.
+ *
+ * Returns: #GST_FLOW_OK for success
  */
 typedef GstFlowReturn		(*GstPadChainFunction)		(GstPad *pad, GstBuffer *buffer);
 /**
  * GstPadGetRangeFunction:
- * @pad: the #GstPad to perform the getrange on.
+ * @pad: the src #GstPad to perform the getrange on.
  * @offset: the offset of the range
  * @length: the length of the range
- * @buffer: a memory location to hold the result buffer
+ * @buffer: a memory location to hold the result buffer, cannot be NULL.
  *
- * This function will be called on sourcepads when a peer element
- * request a buffer at the specified offset and length. If this function
- * returns GST_FLOW_OK, the result buffer will be stored in @buffer. The 
+ * This function will be called on source pads when a peer element
+ * request a buffer at the specified @offset and @length. If this function
+ * returns #GST_FLOW_OK, the result buffer will be stored in @buffer. The 
  * contents of @buffer is invalid for any other return value.
  *
- * Returns: GST_FLOW_OK for success
+ * This function is installed on a source pad with
+ * gst_pad_set_getrange_function() and can only be called on source pads after 
+ * they are successfully activated with gst_pad_activate_pull().
+ *
+ * @offset and @length are always given in byte units. @offset must normally be a value
+ * between 0 and the length in bytes of the data available on @pad. The
+ * length (duration in bytes) can be retrieved with a #GST_QUERY_DURATION or with a
+ * #GST_QUERY_SEEKING. 
+ *
+ * Any @offset larger or equal than the length will make the function return
+ * #GST_FLOW_UNEXPECTED, which corresponds to EOS. In this case @buffer does not
+ * contain a valid buffer.
+ *
+ * The buffer size of @buffer might be smaller than @length when @offset is near
+ * the end of the stream.
+ *
+ * It is allowed to call this function with a 0 @length and valid @offset, in 
+ * which case @buffer will contain a 0-sized buffer and the function returns
+ * #GST_FLOW_OK.
+ *
+ * When this function is called with a -1 @offset, the sequentially next buffer
+ * of length @length in the stream is returned. 
+ *
+ * When this function is called with a -1 @length, a buffer with a default
+ * optimal length is returned in @buffer. The length might depend on the value
+ * of @offset.
+ *
+ * Returns: #GST_FLOW_OK for success
  */
 typedef GstFlowReturn		(*GstPadGetRangeFunction)	(GstPad *pad, guint64 offset,
 		                                                 guint length, GstBuffer **buffer);
@@ -383,15 +418,18 @@ typedef void			(*GstPadFixateCapsFunction)	(GstPad *pad, GstCaps *caps);
  * provide a hardware buffer in order to avoid additional memcpy operations.
  *
  * The function can return a buffer that does not have @caps, in which case the
- * upstream element requests a format change. 
+ * upstream element requests a format change. If a format change was requested,
+ * the returned buffer will be one to hold the data of said new caps, so its
+ * size might be different from @size.
  *
- * When this function returns anything else than GST_FLOW_OK, the buffer allocation
+ * When this function returns anything else than #GST_FLOW_OK, the buffer allocation
  * failed and @buf does not contain valid data.
  *
  * By default this function returns a new buffer of @size and with @caps containing
- * purely malloced data.
+ * purely malloced data. The buffer should be freed with gst_buffer_unref()
+ * after usage.
  *
- * Returns: GST_FLOW_OK if @buf contains a valid buffer, any other return
+ * Returns: #GST_FLOW_OK if @buf contains a valid buffer, any other return
  *  value means @buf does not hold a valid buffer.
  */
 typedef GstFlowReturn		(*GstPadBufferAllocFunction)	(GstPad *pad, guint64 offset, guint size,
@@ -441,6 +479,7 @@ typedef enum {
  * @GST_PAD_FLUSHING: is pad refusing buffers
  * @GST_PAD_IN_GETCAPS: GstPadGetCapsFunction() is running now
  * @GST_PAD_IN_SETCAPS: GstPadSetCapsFunction() is running now
+ * @GST_PAD_BLOCKING: is pad currently blocking on a buffer or event
  * @GST_PAD_FLAG_LAST: offset to define more flags
  *
  * Pad state flags
@@ -450,6 +489,7 @@ typedef enum {
   GST_PAD_FLUSHING      = (GST_OBJECT_FLAG_LAST << 1),
   GST_PAD_IN_GETCAPS    = (GST_OBJECT_FLAG_LAST << 2),
   GST_PAD_IN_SETCAPS    = (GST_OBJECT_FLAG_LAST << 3),
+  GST_PAD_BLOCKING	= (GST_OBJECT_FLAG_LAST << 4),
   /* padding */
   GST_PAD_FLAG_LAST     = (GST_OBJECT_FLAG_LAST << 8)
 } GstPadFlags;
@@ -617,6 +657,7 @@ struct _GstPadClass {
 
 #define GST_PAD_IS_LINKED(pad)		(GST_PAD_PEER(pad) != NULL)
 #define GST_PAD_IS_BLOCKED(pad)		(GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_BLOCKED))
+#define GST_PAD_IS_BLOCKING(pad)	(GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_BLOCKING))
 #define GST_PAD_IS_FLUSHING(pad)	(GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_FLUSHING))
 #define GST_PAD_IS_IN_GETCAPS(pad)	(GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_IN_GETCAPS))
 #define GST_PAD_IS_IN_SETCAPS(pad)	(GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_IN_SETCAPS))
@@ -689,6 +730,7 @@ struct _GstPadClass {
 #define GST_PAD_BLOCK_GET_COND(pad)     (GST_PAD_CAST(pad)->block_cond)
 #define GST_PAD_BLOCK_WAIT(pad)         (g_cond_wait(GST_PAD_BLOCK_GET_COND (pad), GST_OBJECT_GET_LOCK (pad)))
 #define GST_PAD_BLOCK_SIGNAL(pad)       (g_cond_signal(GST_PAD_BLOCK_GET_COND (pad)))
+#define GST_PAD_BLOCK_BROADCAST(pad)    (g_cond_broadcast(GST_PAD_BLOCK_GET_COND (pad)))
 
 /* FIXME: this awful circular dependency need to be resolved properly (see padtemplate.h) */
 #include <gst/gstpadtemplate.h>
@@ -733,6 +775,7 @@ gboolean		gst_pad_set_blocked			(GstPad *pad, gboolean blocked);
 gboolean		gst_pad_set_blocked_async		(GstPad *pad, gboolean blocked,
 								 GstPadBlockCallback callback, gpointer user_data);
 gboolean		gst_pad_is_blocked			(GstPad *pad);
+gboolean		gst_pad_is_blocking			(GstPad *pad);
 
 void			gst_pad_set_element_private		(GstPad *pad, gpointer priv);
 gpointer		gst_pad_get_element_private		(GstPad *pad);
