@@ -111,7 +111,6 @@ enum
   PROP_0,
   PROP_DELAY,
   PROP_AUTO_FLUSH_BUS
-      /* FILL ME */
 };
 
 #define GST_PIPELINE_GET_PRIVATE(obj)  \
@@ -320,7 +319,6 @@ gst_pipeline_get_property (GObject * object, guint prop_id,
  * A flushing seek also resets the stream time to 0 so that when
  * we go back to PLAYING after the seek, the base_time is recalculated
  * and redistributed to the elements.
- *
  */
 static gboolean
 do_pipeline_seek (GstElement * element, GstEvent * event)
@@ -343,12 +341,12 @@ do_pipeline_seek (GstElement * element, GstEvent * event)
     /* need to call _get_state() since a bin state is only updated
      * with this call. */
     gst_element_get_state (element, &state, NULL, 0);
-    was_playing = state == GST_STATE_PLAYING;
+    was_playing = (state == GST_STATE_PLAYING);
 
     if (was_playing) {
       /* and PAUSE when the pipeline was PLAYING, we don't need
        * to wait for the state change to complete since we are going
-       * to flush out any preroll sample anyway */
+       * to flush out any preroll sample anyway. */
       gst_element_set_state (element, GST_STATE_PAUSED);
     }
   }
@@ -506,8 +504,9 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
 
       if (need_reset)
         gst_pipeline_set_new_stream_time (pipeline, 0);
-    }
+
       break;
+    }
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -734,7 +733,7 @@ gst_pipeline_use_clock (GstPipeline * pipeline, GstClock * clock)
  * @clock: the clock to set
  *
  * Set the clock for @pipeline. The clock will be distributed
- * to all the elements managed by the pipeline.
+ * to all the elements managed by the pipeline. 
  *
  * Returns: TRUE if the clock could be set on the pipeline. FALSE if
  *   some element did not accept the clock.
