@@ -124,7 +124,7 @@ gst_plugin_init (GstPlugin * plugin)
 static void
 gst_plugin_finalize (GObject * object)
 {
-  GstPlugin *plugin = GST_PLUGIN (object);
+  GstPlugin *plugin = GST_PLUGIN_CAST (object);
   GstRegistry *registry = gst_registry_get_default ();
   GList *g;
 
@@ -279,6 +279,10 @@ _gst_plugin_fault_handler_restore (void)
 {
   struct sigaction action;
 
+  /* if asked to leave segfaults alone, just return */
+  if (_gst_disable_segtrap)
+    return;
+
   memset (&action, 0, sizeof (action));
   action.sa_handler = SIG_DFL;
 
@@ -399,8 +403,8 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
     GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "problem accessing file");
     g_set_error (error,
         GST_PLUGIN_ERROR,
-        GST_PLUGIN_ERROR_MODULE, "Problem accessing file %s: %s\n", filename,
-        strerror (errno));
+        GST_PLUGIN_ERROR_MODULE, "Problem accessing file %s: %s", filename,
+        g_strerror (errno));
     goto return_error;
   }
 
@@ -935,7 +939,7 @@ gst_plugin_list_free (GList * list)
   GList *g;
 
   for (g = list; g; g = g->next) {
-    gst_object_unref (GST_PLUGIN (g->data));
+    gst_object_unref (GST_PLUGIN_CAST (g->data));
   }
   g_list_free (list);
 }
