@@ -72,7 +72,7 @@ typedef enum
 
 typedef enum
 {
-  GST_PLUGIN_FLAG_CACHED = (1<<0),
+  GST_PLUGIN_FLAG_CACHED = (1<<0)
 } GstPluginFlags;
 
 /**
@@ -127,6 +127,7 @@ struct _GstPluginDesc {
 #define GST_PLUGIN_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_PLUGIN, GstPluginClass))
 #define GST_PLUGIN(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PLUGIN, GstPlugin))
 #define GST_PLUGIN_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PLUGIN, GstPluginClass))
+#define GST_PLUGIN_CAST(obj)           ((GstPlugin*)(obj))
 
 /**
  * GstPlugin:
