@@ -25,6 +25,7 @@
 #define __GST_REGISTRY_H__
 
 #include <stdio.h> /* FIXME: because of cache_file below */
+#include <gst/gstconfig.h>
 #include <gst/gstplugin.h>
 #include <gst/gstpluginfeature.h>
 
@@ -78,7 +79,7 @@ GType			gst_registry_get_type		(void);
 
 GstRegistry *           gst_registry_get_default        (void);
 
-void			gst_registry_scan_path		(GstRegistry *registry, const gchar *path);
+gboolean			gst_registry_scan_path		(GstRegistry *registry, const gchar *path);
 GList*			gst_registry_get_path_list	(GstRegistry *registry);
 
 gboolean		gst_registry_add_plugin		(GstRegistry *registry, GstPlugin *plugin);
@@ -108,10 +109,6 @@ GstPluginFeature * 	gst_registry_lookup_feature 	(GstRegistry *registry, const c
 gboolean 		gst_registry_xml_read_cache 	(GstRegistry * registry, const char *location);
 gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *location);
 
-void 			_gst_registry_remove_cache_plugins (GstRegistry *registry);
-
-void                    _gst_registry_cleanup           (void);
-
 /* convinience defines for the default registry */
 
 /**
