@@ -107,18 +107,25 @@ add_to_char_array (gchar *** array, gchar * value)
 
 /* read a string and copy it into the given location */
 static gboolean
-read_string (xmlTextReaderPtr reader, gchar ** write_to)
+read_string (xmlTextReaderPtr reader, gchar ** write_to, gboolean allow_blank)
 {
   int depth = xmlTextReaderDepth (reader);
   gboolean found = FALSE;
 
   while (xmlTextReaderRead (reader) == 1) {
-    if (xmlTextReaderDepth (reader) == depth)
+    if (xmlTextReaderDepth (reader) == depth) {
+      if (allow_blank && !found &&
+          xmlTextReaderNodeType (reader) == XML_READER_TYPE_END_ELEMENT) {
+        /* Allow blank strings */
+        *write_to = g_strdup ("");
+        found = TRUE;
+      }
       return found;
+    }
     if (xmlTextReaderNodeType (reader) == XML_READER_TYPE_TEXT) {
       if (found)
         return FALSE;
-      *write_to = g_strdup ((gchar *) xmlTextReaderConstValue (reader));
+      *write_to = (gchar *) xmlTextReaderValue (reader);
       found = TRUE;
     }
   }
@@ -213,13 +220,13 @@ load_pad_template (xmlTextReaderPtr reader)
       const gchar *tag = (gchar *) xmlTextReaderConstName (reader);
 
       if (g_str_equal (tag, "nametemplate")) {
-        read_string (reader, &name);
+        read_string (reader, &name, FALSE);
       } else if (g_str_equal (tag, "direction")) {
         read_enum (reader, GST_TYPE_PAD_DIRECTION, &direction);
       } else if (g_str_equal (tag, "presence")) {
         read_enum (reader, GST_TYPE_PAD_PRESENCE, &presence);
       } else if (!strncmp (tag, "caps", 4)) {
-        read_string (reader, &caps_str);
+        read_string (reader, &caps_str, FALSE);
       }
     }
   }
@@ -276,28 +283,29 @@ load_feature (xmlTextReaderPtr reader)
       const gchar *tag = (gchar *) xmlTextReaderConstName (reader);
 
       if (g_str_equal (tag, "name"))
-        read_string (reader, &feature->name);
-      if (g_str_equal (tag, "rank"))
+        read_string (reader, &feature->name, FALSE);
+      else if (g_str_equal (tag, "rank"))
         read_uint (reader, &feature->rank);
+
       if (GST_IS_ELEMENT_FACTORY (feature)) {
-        GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
+        GstElementFactory *factory = GST_ELEMENT_FACTORY_CAST (feature);
 
         if (g_str_equal (tag, "longname")) {
           int ret;
 
-          ret = read_string (reader, &factory->details.longname);
+          ret = read_string (reader, &factory->details.longname, TRUE);
           GST_DEBUG ("longname ret=%d, name=%s",
               ret, factory->details.longname);
         } else if (g_str_equal (tag, "class")) {
-          read_string (reader, &factory->details.klass);
+          read_string (reader, &factory->details.klass, TRUE);
         } else if (g_str_equal (tag, "description")) {
-          read_string (reader, &factory->details.description);
+          read_string (reader, &factory->details.description, TRUE);
         } else if (g_str_equal (tag, "author")) {
-          read_string (reader, &factory->details.author);
+          read_string (reader, &factory->details.author, TRUE);
         } else if (g_str_equal (tag, "uri_type")) {
           gchar *s = NULL;
 
-          if (read_string (reader, &s)) {
+          if (read_string (reader, &s, FALSE)) {
             if (g_ascii_strncasecmp (s, "sink", 4) == 0) {
               factory->uri_type = GST_URI_SINK;
             } else if (g_ascii_strncasecmp (s, "source", 5) == 0) {
@@ -308,12 +316,12 @@ load_feature (xmlTextReaderPtr reader)
         } else if (g_str_equal (tag, "uri_protocol")) {
           gchar *s = NULL;
 
-          if (read_string (reader, &s))
+          if (read_string (reader, &s, FALSE))
             add_to_char_array (&factory->uri_protocols, s);
         } else if (g_str_equal (tag, "interface")) {
           gchar *s = NULL;
 
-          if (read_string (reader, &s)) {
+          if (read_string (reader, &s, FALSE)) {
             __gst_element_factory_add_interface (factory, s);
             /* add_interface strdup's s */
             g_free (s);
@@ -334,21 +342,23 @@ load_feature (xmlTextReaderPtr reader)
         if (g_str_equal (tag, "extension")) {
           gchar *s = NULL;
 
-          if (read_string (reader, &s))
+          if (read_string (reader, &s, TRUE))
             add_to_char_array (&factory->extensions, s);
         } else if (g_str_equal (tag, "caps")) {
           gchar *s = NULL;
 
-          if (read_string (reader, &s)) {
+          if (read_string (reader, &s, FALSE)) {
             factory->caps = gst_caps_from_string (s);
             g_free (s);
           }
         }
+#ifndef GST_DISABLE_INDEX
       } else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
 
         if (g_str_equal (tag, "longdesc"))
-          read_string (reader, &factory->longdesc);
+          read_string (reader, &factory->longdesc, TRUE);
+#endif
       }
     }
   }
@@ -381,44 +391,56 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
       if (g_str_equal (tag, "name")) {
         int ret;
 
-        ret = read_string (reader, &plugin->desc.name);
+        ret = read_string (reader, &plugin->desc.name, FALSE);
         GST_DEBUG ("name ret=%d, name=%s", ret, plugin->desc.name);
         if (!ret)
           break;
       } else if (g_str_equal (tag, "description")) {
-        if (!read_string (reader, &plugin->desc.description))
+        if (!read_string (reader, &plugin->desc.description, TRUE)) {
+          GST_DEBUG ("description field was invalid in registry");
           break;
+        }
         GST_DEBUG ("description %s", plugin->desc.description);
       } else if (g_str_equal (tag, "filename")) {
-        if (!read_string (reader, &plugin->filename))
+        if (!read_string (reader, &plugin->filename, FALSE)) {
+          GST_DEBUG ("filename field was invalid in registry");
           break;
+        }
         GST_DEBUG ("filename %s", plugin->filename);
         plugin->basename = g_path_get_basename (plugin->filename);
       } else if (g_str_equal (tag, "version")) {
-        if (!read_string (reader, &plugin->desc.version))
+        if (!read_string (reader, &plugin->desc.version, TRUE)) {
+          GST_DEBUG ("version field was invalid in registry");
           break;
+        }
         GST_DEBUG ("version %s", plugin->desc.version);
       } else if (g_str_equal (tag, "license")) {
-        if (!read_string (reader, &plugin->desc.license))
+        if (!read_string (reader, &plugin->desc.license, TRUE)) {
+          GST_DEBUG ("license field was invalid in registry");
           break;
+        }
         GST_DEBUG ("license %s", plugin->desc.license);
       } else if (g_str_equal (tag, "source")) {
-        if (!read_string (reader, &plugin->desc.source))
+        if (!read_string (reader, &plugin->desc.source, TRUE)) {
+          GST_DEBUG ("source field was invalid in registry");
           break;
+        }
         GST_DEBUG ("source %s", plugin->desc.source);
       } else if (g_str_equal (tag, "package")) {
-        if (!read_string (reader, &plugin->desc.package))
+        if (!read_string (reader, &plugin->desc.package, TRUE)) {
+          GST_DEBUG ("package field was invalid in registry");
           break;
+        }
         GST_DEBUG ("package %s", plugin->desc.package);
       } else if (g_str_equal (tag, "origin")) {
-        if (!read_string (reader, &plugin->desc.origin)) {
+        if (!read_string (reader, &plugin->desc.origin, TRUE)) {
           GST_DEBUG ("failed to read origin");
           break;
         }
       } else if (g_str_equal (tag, "m32p")) {
         char *s;
 
-        if (!read_string (reader, &s)) {
+        if (!read_string (reader, &s, FALSE)) {
           GST_DEBUG ("failed to read mtime");
           break;
         }
@@ -430,7 +452,7 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
 
         if (read_uint (reader, &x)) {
           plugin->file_size = x;
-          GST_DEBUG ("file_size %d", plugin->file_size);
+          GST_DEBUG ("file_size %" G_GINT64_FORMAT, (gint64) plugin->file_size);
         } else {
           GST_DEBUG ("failed to read size");
         }
@@ -477,7 +499,9 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
   /* make sure these types exist */
   GST_TYPE_ELEMENT_FACTORY;
   GST_TYPE_TYPE_FIND_FACTORY;
+#ifndef GST_DISABLE_INDEX
   GST_TYPE_INDEX_FACTORY;
+#endif
 
   timer = g_timer_new ();
 
@@ -525,7 +549,8 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
             GST_DEBUG ("adding plugin %s", plugin->desc.name);
             gst_registry_add_plugin (registry, plugin);
             for (g = feature_list; g; g = g_list_next (g)) {
-              gst_registry_add_feature (registry, GST_PLUGIN_FEATURE (g->data));
+              gst_registry_add_feature (registry,
+                  GST_PLUGIN_FEATURE_CAST (g->data));
             }
             g_list_free (feature_list);
           }
@@ -692,18 +717,22 @@ gst_registry_xml_save_feature (GstRegistry * registry,
     }
 
     if (GST_URI_TYPE_IS_VALID (factory->uri_type)) {
-      gchar **protocol;
-
       if (!gst_registry_save_escaped (registry, "  ", "uri_type",
               factory->uri_type == GST_URI_SINK ? "sink" : "source"))
         return FALSE;
-      g_assert (factory->uri_protocols);
-      protocol = factory->uri_protocols;
-      while (*protocol) {
-        if (!gst_registry_save_escaped (registry, "  ", "uri_protocol",
-                *protocol))
-          return FALSE;
-        protocol++;
+      if (factory->uri_protocols) {
+        gchar **protocol;
+
+        protocol = factory->uri_protocols;
+        while (*protocol) {
+          if (!gst_registry_save_escaped (registry, "  ", "uri_protocol",
+                  *protocol))
+            return FALSE;
+          protocol++;
+        }
+      } else {
+        g_warning ("GStreamer feature '%s' is URI handler but does not provide"
+            " any protocols it can handle", feature->name);
       }
     }
   } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
@@ -722,10 +751,12 @@ gst_registry_xml_save_feature (GstRegistry * registry,
         i++;
       }
     }
+#ifndef GST_DISABLE_INDEX
   } else if (GST_IS_INDEX_FACTORY (feature)) {
     if (!gst_registry_save_escaped (registry, "  ", "longdesc",
             GST_INDEX_FACTORY (feature)->longdesc))
       return FALSE;
+#endif
   }
   return TRUE;
 }
@@ -819,11 +850,16 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
     g_mkdir_with_parents (dir, 0777);
     g_free (dir);
 
-    registry->cache_file = g_mkstemp (tmp_location);
-  }
-  if (registry->cache_file == -1) {
+    /* the previous g_mkstemp call overwrote the XXXXXX placeholder ... */
     g_free (tmp_location);
-    return FALSE;
+    tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
+    registry->cache_file = g_mkstemp (tmp_location);
+
+    if (registry->cache_file == -1) {
+      GST_DEBUG ("g_mkstemp() failed: %s", g_strerror (errno));
+      g_free (tmp_location);
+      return FALSE;
+    }
   }
 
   if (!gst_registry_save (registry, "<?xml version=\"1.0\"?>\n"))
@@ -834,7 +870,7 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
 
   for (walk = g_list_last (registry->plugins); walk;
       walk = g_list_previous (walk)) {
-    GstPlugin *plugin = GST_PLUGIN (walk->data);
+    GstPlugin *plugin = GST_PLUGIN_CAST (walk->data);
 
     if (!plugin->filename)
       continue;
@@ -866,9 +902,9 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
 
   if (g_file_test (tmp_location, G_FILE_TEST_EXISTS)) {
 #ifdef WIN32
-    remove (location);
+    g_remove (location);
 #endif
-    rename (tmp_location, location);
+    g_rename (tmp_location, location);
   }
 
   g_free (tmp_location);
