@@ -24,27 +24,29 @@
  * @short_description: Generic structure containing fields of names and values
  * @see_also: #GstCaps, #GstMessage, #GstEvent, #GstQuery
  *
- * A #GstStructure is a collection of key/value pairs. The keys are expressed as
- * GQuarks and the values can be of any GType.
+ * A #GstStructure is a collection of key/value pairs. The keys are expressed
+ * as GQuarks and the values can be of any GType.
  *
  * In addition to the key/value pairs, a #GstStructure also has a name.
  * 
- * #GstStructure is used by various GStreamer subsystems to store information in
- * a flexible an extensible way. A #GstStructure does not have a refcount because it
- * usually is part of a higher level object such as #GstCaps. It provides a means to 
- * enforce mutability using the refcount of the parent with the 
- * gst_structure_set_parent_refcount() method.
+ * #GstStructure is used by various GStreamer subsystems to store information
+ * in a flexible and extensible way. A #GstStructure does not have a refcount
+ * because it usually is part of a higher level object such as #GstCaps. It
+ * provides a means to enforce mutability using the refcount of the parent
+ * with the gst_structure_set_parent_refcount() method.
  *
- * A #GstStructure can be created with gst_structure_empty_new() or gst_structure_new(),
- * which both take a name and an optional set of key/value pairs along with the types
- * of the values.
+ * A #GstStructure can be created with gst_structure_empty_new() or
+ * gst_structure_new(), which both take a name and an optional set of
+ * key/value pairs along with the types of the values.
  * 
- * Field values can be changed with gst_structure_set_value() or gst_structure_set().
+ * Field values can be changed with gst_structure_set_value() or
+ * gst_structure_set().
  *
- * Field values can be retrieved with gst_structure_get_value() or the more convenient
- * gst_structure_get_*() functions.
+ * Field values can be retrieved with gst_structure_get_value() or the more
+ * convenient gst_structure_get_*() functions.
  *
- * Fields can be removed with gst_structure_remove_field() or gst_structure_remove_fields().
+ * Fields can be removed with gst_structure_remove_field() or
+ * gst_structure_remove_fields().
  *
  * Last reviewed on 2005-11-09 (0.9.4)
  */
@@ -398,6 +400,7 @@ gst_structure_id_set_value (GstStructure * structure,
     if (s != NULL && !g_utf8_validate (s, -1, NULL)) {
       g_warning ("Trying to set string field '%s' on structure, but string is "
           "not valid UTF-8. Please file a bug.", g_quark_to_string (field));
+      return;
     }
   }
 #endif
@@ -497,6 +500,76 @@ gst_structure_set_valist (GstStructure * structure,
   }
 }
 
+/**
+ * gst_structure_id_set:
+ * @structure: a #GstStructure
+ * @fieldname: the GQuark for the name of the field to set
+ * @...: variable arguments
+ *
+ * Identical to gst_structure_set, except that field names are
+ * passed using the GQuark for the field name. This allows more efficient
+ * setting of the structure if the caller already knows the associated
+ * quark values.
+ * The last variable argument must be NULL.
+ *
+ * Since: 0.10.10
+ */
+void
+gst_structure_id_set (GstStructure * structure, GQuark field, ...)
+{
+  va_list varargs;
+
+  g_return_if_fail (structure != NULL);
+
+  va_start (varargs, field);
+  gst_structure_id_set_valist (structure, field, varargs);
+  va_end (varargs);
+}
+
+/**
+ * gst_structure_id_set_valist:
+ * @structure: a #GstStructure
+ * @fieldname: the name of the field to set
+ * @varargs: variable arguments
+ *
+ * va_list form of gst_structure_id_set().
+ *
+ * Since: 0.10.10
+ */
+void
+gst_structure_id_set_valist (GstStructure * structure,
+    GQuark fieldname, va_list varargs)
+{
+  gchar *err = NULL;
+  GType type;
+
+  g_return_if_fail (structure != NULL);
+  g_return_if_fail (IS_MUTABLE (structure));
+
+  while (fieldname) {
+    GstStructureField field = { 0 };
+
+    field.name = fieldname;
+
+    type = va_arg (varargs, GType);
+
+    if (type == G_TYPE_DATE) {
+      g_warning ("Don't use G_TYPE_DATE, use GST_TYPE_DATE instead\n");
+      type = GST_TYPE_DATE;
+    }
+
+    g_value_init (&field.value, type);
+    G_VALUE_COLLECT (&field.value, varargs, 0, &err);
+    if (err) {
+      g_critical ("%s", err);
+      return;
+    }
+    gst_structure_set_field (structure, &field);
+
+    fieldname = va_arg (varargs, GQuark);
+  }
+}
+
 /* If the structure currently contains a field with the same name, it is
  * replaced with the provided field. Otherwise, the field is added to the
  * structure. The field's value is not deeply copied.
@@ -768,7 +841,11 @@ gst_structure_nth_field_name (const GstStructure * structure, guint index)
 {
   GstStructureField *field;
 
+  g_return_val_if_fail (structure != NULL, NULL);
+  g_return_val_if_fail (index < structure->fields->len, NULL);
+
   field = GST_STRUCTURE_FIELD (structure, index);
+
   return g_quark_to_string (field->name);
 }
 
@@ -793,6 +870,7 @@ gst_structure_foreach (const GstStructure * structure,
   gboolean ret;
 
   g_return_val_if_fail (structure != NULL, FALSE);
+  g_return_val_if_fail (func != NULL, FALSE);
 
   for (i = 0; i < structure->fields->len; i++) {
     field = GST_STRUCTURE_FIELD (structure, i);
@@ -828,6 +906,7 @@ gst_structure_map_in_place (GstStructure * structure,
 
   g_return_val_if_fail (structure != NULL, FALSE);
   g_return_val_if_fail (IS_MUTABLE (structure), FALSE);
+  g_return_val_if_fail (func != NULL, FALSE);
 
   for (i = 0; i < structure->fields->len; i++) {
     field = GST_STRUCTURE_FIELD (structure, i);
@@ -903,8 +982,8 @@ gst_structure_has_field_typed (const GstStructure * structure,
  * and has the correct type.
  *
  * Returns: TRUE if the value could be set correctly. If there was no field
- * with @fieldname or the existing field did not contain a boolean, this function
- * returns FALSE.
+ * with @fieldname or the existing field did not contain a boolean, this
+ * function returns FALSE.
  */
 gboolean
 gst_structure_get_boolean (const GstStructure * structure,
