@@ -35,7 +35,7 @@
  * #GstBaseSink infrastructure should be sufficient.
  *
  * #GstBaseSink provides support for exactly one sink pad, which should be
- * named "sink". A sink implementation (subclass of GstBaseSink) should
+ * named "sink". A sink implementation (subclass of #GstBaseSink) should
  * install a pad template in its base_init function, like so:
  * <programlisting>
  * static void
@@ -53,16 +53,16 @@
  * </programlisting>
  *
  * #GstBaseSink will handle the prerolling correctly. This means that it will
- * return GST_STATE_CHANGE_ASYNC from a state change to PAUSED until the first
+ * return #GST_STATE_CHANGE_ASYNC from a state change to PAUSED until the first
  * buffer arrives in this element. The base class will call the
- * GstBaseSink::preroll vmethod with this preroll buffer and will then commit
+ * #GstBaseSink::preroll vmethod with this preroll buffer and will then commit
  * the state change to the next asynchronously pending state.
  *
  * When the element is set to PLAYING, #GstBaseSink will synchronise on the
  * clock using the times returned from ::get_times. If this function returns
  * #GST_CLOCK_TIME_NONE for the start time, no synchronisation will be done.
  * Synchronisation can be disabled entirely by setting the object "sync"
- * property to FALSE.
+ * property to %FALSE.
  *
  * After synchronisation the virtual method #GstBaseSink::render will be called.
  * Subclasses should minimally implement this method.
@@ -70,7 +70,8 @@
  * Since 0.10.3 subclasses that synchronise on the clock in the ::render method
  * are supported as well. These classes typically receive a buffer in the render
  * method and can then potentially block on the clock while rendering. A typical
- * example is an audiosink.
+ * example is an audiosink. Since 0.10.11 these subclasses can use
+ * gst_base_sink_wait_preroll() to perform the blocking wait.
  *
  * Upon receiving the EOS event in the PLAYING state, #GstBaseSink will wait
  * for the clock to reach the time indicated by the stop time of the last
@@ -78,14 +79,14 @@
  * EOS in PAUSED, preroll completes, the event is queued and an EOS message is
  * posted when going to PLAYING.
  *
- * #GstBaseSink will internally use the GST_EVENT_NEW_SEGMENT events to schedule
+ * #GstBaseSink will internally use the #GST_EVENT_NEWSEGMENT events to schedule
  * synchronisation and clipping of buffers. Buffers that fall completely outside
  * of the current segment are dropped. Buffers that fall partially in the
  * segment are rendered (and prerolled). Subclasses should do any subbuffer
  * clipping themselves when needed.
  *
  * #GstBaseSink will by default report the current playback position in
- * GST_FORMAT_TIME based on the current clock time and segment information.
+ * #GST_FORMAT_TIME based on the current clock time and segment information.
  * If no clock has been set on the element, the query will be forwarded
  * upstream.
  *
@@ -127,7 +128,7 @@
  * synchronisation. For each dropped buffer it will also send a QoS message
  * upstream.
  *
- * Last reviewed on 2006-03-31 (0.10.5)
+ * Last reviewed on 2006-09-27 (0.10.11)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -237,7 +238,7 @@ gst_base_sink_get_type (void)
 {
   static GType base_sink_type = 0;
 
-  if (!base_sink_type) {
+  if (G_UNLIKELY (base_sink_type == 0)) {
     static const GTypeInfo base_sink_info = {
       sizeof (GstBaseSinkClass),
       (GBaseInitFunc) gst_base_sink_base_init,
@@ -709,11 +710,10 @@ gst_base_sink_configure_segment (GstBaseSink * basesink, GstPad * pad,
   gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
       &start, &stop, &time);
 
+  /* The segment is protected with both the STREAM_LOCK and the OBJECT_LOCK.
+   * We protect with the OBJECT_LOCK so that we can use the values to
+   * safely answer a POSITION query. */
   GST_OBJECT_LOCK (basesink);
-
-  if (segment->format != format)
-    gst_segment_init (segment, format);
-
   gst_segment_set_newsegment_full (segment, update, rate, arate, format, start,
       stop, time);
 
@@ -748,6 +748,9 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
   gboolean post_paused = FALSE;
   gboolean post_playing = FALSE;
 
+  /* we are certainly not playing async anymore now */
+  basesink->playing_async = FALSE;
+
   GST_OBJECT_LOCK (basesink);
   current = GST_STATE (basesink);
   next = GST_STATE_NEXT (basesink);
@@ -821,7 +824,25 @@ gst_base_sink_commit_state (GstBaseSink * basesink)
 
 nothing_pending:
   {
-    GST_DEBUG_OBJECT (basesink, "nothing to commit");
+    /* Depending on the state, set our vars. We get in this situation when the
+     * state change function got a change to update the state vars before the
+     * streaming thread did. This is fine but we need to make sure that we
+     * update the need_preroll var since it was TRUE when we got here and might
+     * become FALSE if we got to PLAYING. */
+    GST_DEBUG_OBJECT (basesink, "nothing to commit, now in %s",
+        gst_element_state_get_name (current));
+    switch (current) {
+      case GST_STATE_PLAYING:
+        basesink->need_preroll = FALSE;
+        break;
+      case GST_STATE_PAUSED:
+        basesink->need_preroll = TRUE;
+        break;
+      default:
+        basesink->need_preroll = FALSE;
+        basesink->flushing = TRUE;
+        break;
+    }
     GST_OBJECT_UNLOCK (basesink);
     return TRUE;
   }
@@ -934,6 +955,9 @@ gst_base_sink_get_sync_times (GstBaseSink * basesink, GstMiniObject * obj,
         GST_TIME_ARGS (cstop));
   }
 
+  /* set last stop position */
+  gst_segment_set_last_stop (segment, GST_FORMAT_TIME, cstop);
+
 do_times:
   /* this can produce wrong values if we accumulated non-TIME segments. If this happens,
    * upstream is behaving very badly */
@@ -1032,6 +1056,47 @@ no_clock:
   }
 }
 
+/**
+ * gst_base_sink_wait_preroll:
+ * @sink: the sink
+ *
+ * If the #GstBaseSinkClass::render method performs its own synchronisation against
+ * the clock it must unblock when going from PLAYING to the PAUSED state and call
+ * this method before continuing to render the remaining data.
+ *
+ * This function will block until a state change to PLAYING happens (in which
+ * case this function returns #GST_FLOW_OK) or the processing must be stopped due
+ * to a state change to READY or a FLUSH event (in which case this function
+ * returns #GST_FLOW_WRONG_STATE).
+ *
+ * Since: 0.10.11
+ *
+ * Returns: #GST_FLOW_OK if the preroll completed and processing can
+ * continue. Any other return value should be returned from the render vmethod.
+ */
+GstFlowReturn
+gst_base_sink_wait_preroll (GstBaseSink * sink)
+{
+  /* block until the state changes, or we get a flush, or something */
+  GST_DEBUG_OBJECT (sink, "wait for preroll...");
+  sink->have_preroll = TRUE;
+  GST_PAD_PREROLL_WAIT (sink->sinkpad);
+  sink->have_preroll = FALSE;
+  GST_DEBUG_OBJECT (sink, "preroll done");
+  if (G_UNLIKELY (sink->flushing))
+    goto stopping;
+  GST_DEBUG_OBJECT (sink, "continue after preroll");
+
+  return GST_FLOW_OK;
+
+  /* ERRORS */
+stopping:
+  {
+    GST_DEBUG_OBJECT (sink, "preroll interrupted");
+    return GST_FLOW_WRONG_STATE;
+  }
+}
+
 /* with STREAM_LOCK, PREROLL_LOCK
  *
  * Make sure we are in PLAYING and synchronize an object to the clock.
@@ -1100,7 +1165,6 @@ again:
     GST_DEBUG_OBJECT (basesink, "prerolling object %p", obj);
 
     if (G_LIKELY (basesink->playing_async)) {
-      basesink->playing_async = FALSE;
       /* commit state */
       if (G_UNLIKELY (!gst_base_sink_commit_state (basesink)))
         goto stopping;
@@ -1110,12 +1174,7 @@ again:
      * made us not need the preroll anymore */
     if (G_LIKELY (basesink->need_preroll)) {
       /* block until the state changes, or we get a flush, or something */
-      GST_DEBUG_OBJECT (basesink, "waiting to finish preroll");
-      basesink->have_preroll = TRUE;
-      GST_PAD_PREROLL_WAIT (pad);
-      basesink->have_preroll = FALSE;
-      GST_DEBUG_OBJECT (basesink, "done preroll");
-      if (G_UNLIKELY (basesink->flushing))
+      if (gst_base_sink_wait_preroll (basesink) != GST_FLOW_OK)
         goto flushing;
     }
   }
@@ -1573,12 +1632,10 @@ gst_base_sink_preroll_object (GstBaseSink * basesink, GstPad * pad,
     if (bclass->preroll)
       if ((ret = bclass->preroll (basesink, buf)) != GST_FLOW_OK)
         goto preroll_failed;
-  } else {
   }
 
   /* commit state */
   if (G_LIKELY (basesink->playing_async)) {
-    basesink->playing_async = FALSE;
     if (G_UNLIKELY (!gst_base_sink_commit_state (basesink)))
       goto stopping;
   }
@@ -1856,7 +1913,7 @@ gst_base_sink_chain_unlocked (GstBaseSink * basesink, GstPad * pad,
     GstBuffer * buf)
 {
   GstFlowReturn result;
-  GstClockTime start = -1, end = -1;
+  GstClockTime start = GST_CLOCK_TIME_NONE, end = GST_CLOCK_TIME_NONE;
   GstSegment *clip_segment;
 
   if (G_UNLIKELY (basesink->flushing))
@@ -1982,6 +2039,8 @@ gst_base_sink_loop (GstPad * pad)
   if (G_UNLIKELY (buf == NULL))
     goto no_buffer;
 
+  basesink->offset += GST_BUFFER_SIZE (buf);
+
   GST_PAD_PREROLL_LOCK (pad);
   result = gst_base_sink_chain_unlocked (basesink, pad, buf);
   GST_PAD_PREROLL_UNLOCK (pad);
@@ -2267,7 +2326,7 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
     case GST_FORMAT_TIME:
     {
       GstClockTime now, base;
-      gint64 time, accum;
+      gint64 time, accum, duration;
       gdouble rate;
       gint64 last;
 
@@ -2302,6 +2361,11 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
       else
         time = 0;
 
+      if (GST_CLOCK_TIME_IS_VALID (basesink->segment.stop))
+        duration = basesink->segment.stop - basesink->segment.start;
+      else
+        duration = 0;
+
       base = GST_ELEMENT_CAST (basesink)->base_time;
       accum = basesink->segment.accum;
       rate = basesink->segment.rate * basesink->segment.applied_rate;
@@ -2320,7 +2384,11 @@ gst_base_sink_get_position (GstBaseSink * basesink, GstFormat format,
        * rate and applied rate. */
       base += accum;
       base = MIN (now, base);
-      /* for negative rate this will count back from the segment time */
+
+      /* for negative rates we need to count back from from the segment
+       * duration. */
+      if (rate < 0.0)
+        time += duration;
       *cur = time + gst_guint64_to_gdouble (now - base) * rate;
 
       /* never report more than last seen position */
