@@ -37,41 +37,48 @@
  * <para>
  * The source can be configured to operate in any #GstFormat with the
  * gst_base_src_set_format() method. The currently set format determines 
- * the format of the internal #GstSegment and any #GST_EVENT_NEW_SEGMENT 
+ * the format of the internal #GstSegment and any #GST_EVENT_NEWSEGMENT 
  * events. The default format for #GstBaseSrc is #GST_FORMAT_BYTES.
  * </para>
  * <para>
  * #GstBaseSrc always supports push mode scheduling. If the following
  * conditions are met, it also supports pull mode scheduling:
  * <itemizedlist>
- *   <listitem><para>The format is set to GST_FORMAT_BYTES (default).</para>
+ *   <listitem><para>The format is set to #GST_FORMAT_BYTES (default).</para>
  *   </listitem>
- *   <listitem><para>#GstBaseSrc::is_seekable returns TRUE.</para>
+ *   <listitem><para>#GstBaseSrc::is_seekable returns %TRUE.</para>
  *   </listitem>
  * </itemizedlist>
  * </para>
  * <para>
+ * Since 0.10.9, any #GstBaseSrc can enable pull based scheduling at any 
+ * time by overriding #GstBaseSrc::check_get_range so that it returns %TRUE. 
+ * </para>
+ * <para>
  * If all the conditions are met for operating in pull mode, #GstBaseSrc is
  * automatically seekable in push mode as well. The following conditions must 
  * be met to make the element seekable in push mode when the format is not
- * GST_FORMAT_BYTES:
+ * #GST_FORMAT_BYTES:
  * <itemizedlist>
  *   <listitem><para>
- *     #GstBaseSrc::is_seekable returns TRUE.
+ *     #GstBaseSrc::is_seekable returns %TRUE.
  *   </para></listitem>
  *   <listitem><para>
  *     #GstBaseSrc::query can convert all supported seek formats to the
  *     internal format as set with gst_base_src_set_format().
  *   </para></listitem>
  *   <listitem><para>
- *     #GstBaseSrc::do_seek is implemented, performs the seek and returns TRUE.
+ *     #GstBaseSrc::do_seek is implemented, performs the seek and returns %TRUE.
  *   </para></listitem>
  * </itemizedlist>
  * </para>
  * <para>
- * When the default format is not GST_FORMAT_BYTES, the subclass should ignore 
- * the offset and length in the #GstBaseSrc::create method. It is recommended 
- * to subclass #GstPushSrc instead in this situation.
+ * When the element does not meet the requirements to operate in pull mode,
+ * the offset and length in the #GstBaseSrc::create method should be ignored.
+ * It is recommended to subclass #GstPushSrc instead, in this situation. If the
+ * element can operate in pull mode but only with specific offsets and
+ * lengths, it is allowed to generate an error when the wrong values are passed
+ * to the #GstBaseSrc::create function.
  * </para>
  * <para>
  * #GstBaseSrc has support for live sources. Live sources are sources that 
@@ -84,7 +91,7 @@
  * A live source does not produce data in the PAUSED state. This means that the 
  * #GstBaseSrc::create method will not be called in PAUSED but only in PLAYING.
  * To signal the pipeline that the element will not produce data, the return
- * value from the READY to PAUSED state will be GST_STATE_CHANGE_NO_PREROLL.
+ * value from the READY to PAUSED state will be #GST_STATE_CHANGE_NO_PREROLL.
  * </para>
  * <para>
  * A typical live source will timestamp the buffers it creates with the 
@@ -93,6 +100,11 @@
  * distributed and running.
  * </para>
  * <para>
+ * Live sources that synchronize and block on the clock (and audio source, for
+ * example) can since 0.10.12 use gst_base_src_wait_playing() when the ::create
+ * function was interrupted by a state change to PAUSED.
+ * </para>
+ * <para>
  * The #GstBaseSrc::get_times method can be used to implement pseudo-live 
  * sources. The base source will wait for the specified stream time returned in 
  * #GstBaseSrc::get_times before pushing out the buffer. 
@@ -100,8 +112,8 @@
  * a live source.
  * </para>
  * <para>
- * There is only support in GstBaseSrc for exactly one source pad, which 
- * should be named "src". A source implementation (subclass of GstBaseSrc) 
+ * There is only support in #GstBaseSrc for exactly one source pad, which 
+ * should be named "src". A source implementation (subclass of #GstBaseSrc) 
  * should install a pad template in its base_init function, like so:
  * </para>
  * <para>
@@ -139,8 +151,8 @@
  * <programlisting>
  * ...
  * // stop recording
- * gst_element_set_state (audio_source, GST_STATE_NULL);
- * gst_element_set_locked_state (audio_source, TRUE);
+ * gst_element_set_state (audio_source, #GST_STATE_NULL);
+ * gst_element_set_locked_state (audio_source, %TRUE);
  * ...
  * </programlisting>
  * Now the application should wait for an EOS message
@@ -149,8 +161,8 @@
  * <programlisting>
  * ...
  * // everything done - shut down pipeline
- * gst_element_set_state (pipeline, GST_STATE_NULL);
- * gst_element_set_locked_state (audio_source, FALSE);
+ * gst_element_set_state (pipeline, #GST_STATE_NULL);
+ * gst_element_set_locked_state (audio_source, %FALSE);
  * ...
  * </programlisting>
  * </para>
@@ -160,7 +172,7 @@
  * thread might be blocked in PREROLL.
  * </para>
  * <para>
- * Last reviewed on 2006-04-14 (0.10.6)
+ * Last reviewed on 2006-09-27 (0.10.11)
  * </para>
  * </refsect2>
  */
@@ -237,7 +249,7 @@ gst_base_src_get_type (void)
 {
   static GType base_src_type = 0;
 
-  if (!base_src_type) {
+  if (G_UNLIKELY (base_src_type == 0)) {
     static const GTypeInfo base_src_info = {
       sizeof (GstBaseSrcClass),
       (GBaseInitFunc) gst_base_src_base_init,
@@ -316,8 +328,8 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_BLOCKSIZE,
       g_param_spec_ulong ("blocksize", "Block size",
-          "Size in bytes to read per buffer", 1, G_MAXULONG, DEFAULT_BLOCKSIZE,
-          G_PARAM_READWRITE));
+          "Size in bytes to read per buffer (0 = default)", 0, G_MAXULONG,
+          DEFAULT_BLOCKSIZE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_NUM_BUFFERS,
       g_param_spec_int ("num-buffers", "num-buffers",
           "Number of buffers to output before sending EOS", -1, G_MAXINT,
@@ -413,6 +425,58 @@ gst_base_src_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * gst_base_src_wait_playing:
+ * @src: the src
+ *
+ * If the #GstBaseSrcClass::create method performs its own synchronisation against
+ * the clock it must unblock when going from PLAYING to the PAUSED state and call
+ * this method before continuing to produce the remaining data.
+ *
+ * This function will block until a state change to PLAYING happens (in which
+ * case this function returns #GST_FLOW_OK) or the processing must be stopped due
+ * to a state change to READY or a FLUSH event (in which case this function
+ * returns #GST_FLOW_WRONG_STATE).
+ *
+ * Since: 0.10.12
+ *
+ * Returns: #GST_FLOW_OK if @src is PLAYING and processing can
+ * continue. Any other return value should be returned from the create vmethod.
+ */
+GstFlowReturn
+gst_base_src_wait_playing (GstBaseSrc * src)
+{
+  /* block until the state changes, or we get a flush, or something */
+  GST_LIVE_LOCK (src);
+  if (src->is_live) {
+    while (G_UNLIKELY (!src->live_running)) {
+      GST_DEBUG ("live source signal waiting");
+      GST_LIVE_SIGNAL (src);
+      GST_DEBUG ("live source waiting for running state");
+      GST_LIVE_WAIT (src);
+      GST_DEBUG ("live source unlocked");
+    }
+    /* FIXME, use another variable to signal stopping so that we don't
+     * have to grab another lock. */
+    GST_OBJECT_LOCK (src->srcpad);
+    if (G_UNLIKELY (GST_PAD_IS_FLUSHING (src->srcpad)))
+      goto flushing;
+    GST_OBJECT_UNLOCK (src->srcpad);
+  }
+  GST_LIVE_UNLOCK (src);
+
+  return GST_FLOW_OK;
+
+  /* ERRORS */
+flushing:
+  {
+    GST_DEBUG_OBJECT (src, "pad is flushing");
+    GST_OBJECT_UNLOCK (src->srcpad);
+    GST_LIVE_UNLOCK (src);
+    return GST_FLOW_WRONG_STATE;
+  }
+}
+
 /**
  * gst_base_src_set_live:
  * @src: base source instance
@@ -635,9 +699,6 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
       res = TRUE;
       break;
     }
-    case GST_QUERY_LATENCY:
-    case GST_QUERY_JITTER:
-    case GST_QUERY_RATE:
     case GST_QUERY_CONVERT:
     {
       GstFormat src_fmt, dest_fmt;
@@ -655,6 +716,9 @@ gst_base_src_default_query (GstBaseSrc * src, GstQuery * query)
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
     }
+    case GST_QUERY_LATENCY:
+    case GST_QUERY_JITTER:
+    case GST_QUERY_RATE:
     default:
       res = FALSE;
       break;
@@ -848,6 +912,7 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
     GST_DEBUG_OBJECT (src, "closing running segment %" G_GINT64_FORMAT
         " to %" G_GINT64_FORMAT, src->segment.start, src->segment.last_stop);
 
+    /* queue the segment for sending in the stream thread */
     if (src->priv->close_segment)
       gst_event_unref (src->priv->close_segment);
     src->priv->close_segment =
@@ -867,13 +932,16 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
               src->segment.format, src->segment.last_stop));
     }
 
+    /* for deriving a stop position for the playback segment form the seek
+     * segment, we must take the duration when the stop is not set */
     if ((stop = src->segment.stop) == -1)
       stop = src->segment.duration;
 
-    /* now send the newsegment */
     GST_DEBUG_OBJECT (src, "Sending newsegment from %" G_GINT64_FORMAT
         " to %" G_GINT64_FORMAT, src->segment.start, stop);
 
+    /* now replace the old segment so that we send it in the stream thread the
+     * next time it is scheduled. */
     if (src->priv->start_segment)
       gst_event_unref (src->priv->start_segment);
     src->priv->start_segment =
@@ -1107,7 +1175,7 @@ gst_base_src_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
-/* with STREAM_LOCK and LOCK*/
+/* with STREAM_LOCK and LOCK */
 static GstClockReturn
 gst_base_src_wait (GstBaseSrc * basesrc, GstClockTime time)
 {
@@ -1119,10 +1187,6 @@ gst_base_src_wait (GstBaseSrc * basesrc, GstClockTime time)
   if ((clock = GST_ELEMENT_CLOCK (basesrc)) == NULL)
     return GST_CLOCK_OK;
 
-  /* clock_id should be NULL outside of this function */
-  g_assert (basesrc->clock_id == NULL);
-  g_assert (GST_CLOCK_TIME_IS_VALID (time));
-
   id = gst_clock_new_single_shot_id (clock, time);
 
   basesrc->clock_id = id;
@@ -1195,6 +1259,7 @@ gst_base_src_update_length (GstBaseSrc * src, guint64 offset, guint * length)
   if (src->segment.format != GST_FORMAT_BYTES)
     return TRUE;
 
+  /* get total file size */
   size = (guint64) src->segment.duration;
 
   /* the max amount of bytes to read is the total size or
@@ -1209,30 +1274,37 @@ gst_base_src_update_length (GstBaseSrc * src, guint64 offset, guint * length)
       ", segment.stop %" G_GINT64_FORMAT ", maxsize %" G_GINT64_FORMAT, offset,
       *length, size, src->segment.stop, maxsize);
 
-  /* check size */
+  /* check size if we have one */
   if (maxsize != -1) {
-    if (offset > maxsize)
-      goto unexpected_length;
-
-    if (offset + *length > maxsize) {
+    /* if we run past the end, check if the file became bigger and 
+     * retry. */
+    if (G_UNLIKELY (offset + *length >= maxsize)) {
       /* see if length of the file changed */
       if (bclass->get_size)
-        bclass->get_size (src, &size);
+        if (!bclass->get_size (src, &size))
+          size = -1;
 
+      gst_segment_set_duration (&src->segment, GST_FORMAT_BYTES, size);
+
+      /* make sure we don't exceed the configured segment stop
+       * if it was set */
       if (src->segment.stop != -1)
         maxsize = MIN (size, src->segment.stop);
       else
         maxsize = size;
 
-      if (offset + *length > maxsize) {
+      /* if we are at or past the end, EOS */
+      if (G_UNLIKELY (offset >= maxsize))
+        goto unexpected_length;
+
+      /* else we can clip to the end */
+      if (G_UNLIKELY (offset + *length >= maxsize))
         *length = maxsize - offset;
-      }
+
     }
   }
-  if (*length == 0)
-    goto unexpected_length;
 
-  /* keep track f current position. segment is in bytes, we checked 
+  /* keep track of current position. segment is in bytes, we checked 
    * that above. */
   gst_segment_set_last_stop (&src->segment, GST_FORMAT_BYTES, offset);
 
@@ -1255,48 +1327,36 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
 
   bclass = GST_BASE_SRC_GET_CLASS (src);
 
-  GST_LIVE_LOCK (src);
-  if (src->is_live) {
-    while (!src->live_running) {
-      GST_DEBUG ("live source signal waiting");
-      GST_LIVE_SIGNAL (src);
-      GST_DEBUG ("live source waiting for running state");
-      GST_LIVE_WAIT (src);
-      GST_DEBUG ("live source unlocked");
-    }
-    /* FIXME, use another variable to signal stopping */
-    GST_OBJECT_LOCK (src->srcpad);
-    if (GST_PAD_IS_FLUSHING (src->srcpad))
-      goto flushing;
-    GST_OBJECT_UNLOCK (src->srcpad);
-  }
-  GST_LIVE_UNLOCK (src);
+  ret = gst_base_src_wait_playing (src);
+  if (ret != GST_FLOW_OK)
+    goto stopped;
 
-  if (!GST_OBJECT_FLAG_IS_SET (src, GST_BASE_SRC_STARTED))
+  if (G_UNLIKELY (!GST_OBJECT_FLAG_IS_SET (src, GST_BASE_SRC_STARTED)))
     goto not_started;
 
   if (G_UNLIKELY (!bclass->create))
     goto no_function;
 
-  if (!gst_base_src_update_length (src, offset, &length))
+  if (G_UNLIKELY (!gst_base_src_update_length (src, offset, &length)))
     goto unexpected_length;
 
-  if (src->num_buffers_left == 0) {
-    goto reached_num_buffers;
-  } else {
-    if (src->num_buffers_left > 0)
+  /* normally we don't count buffers */
+  if (G_UNLIKELY (src->num_buffers_left >= 0)) {
+    if (src->num_buffers_left == 0)
+      goto reached_num_buffers;
+    else
       src->num_buffers_left--;
   }
 
   GST_DEBUG_OBJECT (src,
-      "calling create offset %" G_GUINT64_FORMAT " %" G_GINT64_FORMAT, offset,
-      src->segment.time);
+      "calling create offset %" G_GUINT64_FORMAT " length %u, time %"
+      G_GINT64_FORMAT, offset, length, src->segment.time);
 
   ret = bclass->create (src, offset, length, buf);
-  if (ret != GST_FLOW_OK)
+  if (G_UNLIKELY (ret != GST_FLOW_OK))
     goto done;
 
-  /* no timestamp set and we are at offset 0 */
+  /* no timestamp set and we are at offset 0, we can timestamp with 0 */
   if (offset == 0 && src->segment.time == 0
       && GST_BUFFER_TIMESTAMP (*buf) == -1)
     GST_BUFFER_TIMESTAMP (*buf) = 0;
@@ -1311,6 +1371,10 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
       GST_DEBUG_OBJECT (src, "buffer ok");
       break;
     default:
+      /* this case is triggered when we were waiting for the clock and
+       * it got unlocked because we did a state change. We return 
+       * WRONG_STATE in this case to stop the dataflow also get rid of the
+       * produced buffer. */
       GST_DEBUG_OBJECT (src, "clock returned %d, not returning", status);
       gst_buffer_unref (*buf);
       *buf = NULL;
@@ -1321,12 +1385,10 @@ done:
   return ret;
 
   /* ERROR */
-flushing:
+stopped:
   {
-    GST_DEBUG_OBJECT (src, "pad is flushing");
-    GST_OBJECT_UNLOCK (src->srcpad);
-    GST_LIVE_UNLOCK (src);
-    return GST_FLOW_WRONG_STATE;
+    GST_DEBUG_OBJECT (src, "wait_playing returned %d", ret);
+    return ret;
   }
 not_started:
   {
@@ -1367,31 +1429,6 @@ gst_base_src_pad_get_range (GstPad * pad, guint64 offset, guint length,
   return res;
 }
 
-static gboolean
-gst_base_src_pad_check_get_range (GstPad * pad)
-{
-  GstBaseSrcClass *bclass;
-  GstBaseSrc *src;
-  gboolean res;
-
-  src = GST_BASE_SRC (gst_pad_get_parent (pad));
-
-  bclass = GST_BASE_SRC_GET_CLASS (src);
-
-  if (bclass->check_get_range == NULL) {
-    GST_WARNING_OBJECT (src, "no check_get_range function set");
-    return FALSE;
-  }
-
-  res = bclass->check_get_range (src);
-  GST_LOG_OBJECT (src, "%s() returned %d",
-      GST_DEBUG_FUNCPTR_NAME (bclass->check_get_range), (gint) res);
-
-  gst_object_unref (src);
-
-  return res;
-}
-
 static gboolean
 gst_base_src_default_check_get_range (GstBaseSrc * src)
 {
@@ -1399,8 +1436,8 @@ gst_base_src_default_check_get_range (GstBaseSrc * src)
 
   if (!GST_OBJECT_FLAG_IS_SET (src, GST_BASE_SRC_STARTED)) {
     GST_LOG_OBJECT (src, "doing start/stop to check get_range support");
-    gst_base_src_start (src);
-    gst_base_src_stop (src);
+    if (G_LIKELY (gst_base_src_start (src)))
+      gst_base_src_stop (src);
   }
 
   /* we can operate in getrange mode if the native format is bytes
@@ -1411,6 +1448,46 @@ gst_base_src_default_check_get_range (GstBaseSrc * src)
   return res;
 }
 
+static gboolean
+gst_base_src_check_get_range (GstBaseSrc * src)
+{
+  GstBaseSrcClass *bclass;
+  gboolean res;
+
+  bclass = GST_BASE_SRC_GET_CLASS (src);
+
+  if (bclass->check_get_range == NULL)
+    goto no_function;
+
+  res = bclass->check_get_range (src);
+  GST_LOG_OBJECT (src, "%s() returned %d",
+      GST_DEBUG_FUNCPTR_NAME (bclass->check_get_range), (gint) res);
+
+  return res;
+
+  /* ERRORS */
+no_function:
+  {
+    GST_WARNING_OBJECT (src, "no check_get_range function set");
+    return FALSE;
+  }
+}
+
+static gboolean
+gst_base_src_pad_check_get_range (GstPad * pad)
+{
+  GstBaseSrc *src;
+  gboolean res;
+
+  src = GST_BASE_SRC (gst_pad_get_parent (pad));
+
+  res = gst_base_src_check_get_range (src);
+
+  gst_object_unref (src);
+
+  return res;
+}
+
 static void
 gst_base_src_loop (GstPad * pad)
 {
@@ -1444,7 +1521,7 @@ gst_base_src_loop (GstPad * pad)
 
   ret = gst_base_src_get_range (src, position, src->blocksize, &buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
-    GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() =  %d", ret);
+    GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() = %d", ret);
     goto pause;
   }
   if (G_UNLIKELY (buf == NULL))
@@ -1496,7 +1573,7 @@ gst_base_src_loop (GstPad * pad)
 
   ret = gst_pad_push (pad, buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
-    GST_INFO_OBJECT (src, "pausing after gst_pad_push() =  %d", ret);
+    GST_INFO_OBJECT (src, "pausing after gst_pad_push() = %d", ret);
     goto pause;
   }
 
@@ -1522,18 +1599,19 @@ pause:
       if (ret == GST_FLOW_UNEXPECTED) {
         /* perform EOS logic */
         if (src->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-          gst_element_post_message (GST_ELEMENT (src),
-              gst_message_new_segment_done (GST_OBJECT (src),
+          gst_element_post_message (GST_ELEMENT_CAST (src),
+              gst_message_new_segment_done (GST_OBJECT_CAST (src),
                   src->segment.format, src->segment.last_stop));
         } else {
           gst_pad_push_event (pad, gst_event_new_eos ());
           src->priv->last_sent_eos = TRUE;
         }
       } else {
-        /* for fatal errors we post an error message */
+        /* for fatal errors we post an error message, post the error
+         * first so the app knows about the error first. */
         GST_ELEMENT_ERROR (src, STREAM, FAILED,
             (_("Internal data flow error.")),
-            ("streaming task paused, reason %s", reason));
+            ("streaming task paused, reason %s (%d)", reason, ret));
         gst_pad_push_event (pad, gst_event_new_eos ());
         src->priv->last_sent_eos = TRUE;
       }
@@ -1707,6 +1785,8 @@ gst_base_src_start (GstBaseSrc * basesrc)
     /* only update the size when operating in bytes, subclass is supposed
      * to set duration in the start method for other formats */
     gst_segment_set_duration (&basesrc->segment, GST_FORMAT_BYTES, size);
+  } else {
+    size = -1;
   }
 
   GST_DEBUG_OBJECT (basesrc,
@@ -1747,6 +1827,7 @@ gst_base_src_start (GstBaseSrc * basesrc)
 could_not_start:
   {
     GST_DEBUG_OBJECT (basesrc, "could not start");
+    /* subclass is supposed to post a message. We don't have to call _stop. */
     return FALSE;
   }
 could_not_negotiate:
@@ -1754,6 +1835,7 @@ could_not_negotiate:
     GST_DEBUG_OBJECT (basesrc, "could not negotiate, stopping");
     GST_ELEMENT_ERROR (basesrc, STREAM, FORMAT,
         ("Could not negotiate format"), ("Check your filtered caps, if any"));
+    /* we must call stop */
     gst_base_src_stop (basesrc);
     return FALSE;
   }
@@ -1803,20 +1885,18 @@ static gboolean
 gst_base_src_activate_push (GstPad * pad, gboolean active)
 {
   GstBaseSrc *basesrc;
-  gboolean res;
+  GstEvent *event;
 
   basesrc = GST_BASE_SRC (GST_OBJECT_PARENT (pad));
 
   /* prepare subclass first */
   if (active) {
-    GstEvent *event;
-
     GST_DEBUG_OBJECT (basesrc, "Activating in push mode");
 
-    if (!basesrc->can_activate_push)
+    if (G_UNLIKELY (!basesrc->can_activate_push))
       goto no_push_activation;
 
-    if (!gst_base_src_start (basesrc))
+    if (G_UNLIKELY (!gst_base_src_start (basesrc)))
       goto error_start;
 
     basesrc->priv->last_sent_eos = FALSE;
@@ -1828,27 +1908,52 @@ gst_base_src_activate_push (GstPad * pad, gboolean active)
     GST_OBJECT_UNLOCK (basesrc);
 
     /* no need to unlock anything, the task is certainly
-     * not running here. */
-    res = gst_base_src_perform_seek (basesrc, event, FALSE);
+     * not running here. The perform seek code will start the task when
+     * finished. */
+    if (G_UNLIKELY (!gst_base_src_perform_seek (basesrc, event, FALSE)))
+      goto seek_failed;
 
     if (event)
       gst_event_unref (event);
   } else {
     GST_DEBUG_OBJECT (basesrc, "Deactivating in push mode");
-    res = gst_base_src_deactivate (basesrc, pad);
+    /* call the unlock function and stop the task */
+    if (G_UNLIKELY (!gst_base_src_deactivate (basesrc, pad)))
+      goto deactivate_failed;
+
+    /* now we can stop the source */
+    if (G_UNLIKELY (!gst_base_src_stop (basesrc)))
+      goto error_stop;
   }
-  return res;
+  return TRUE;
 
   /* ERRORS */
 no_push_activation:
   {
-    GST_DEBUG_OBJECT (basesrc, "Subclass disabled push-mode activation");
+    GST_ERROR_OBJECT (basesrc, "Subclass disabled push-mode activation");
     return FALSE;
   }
 error_start:
   {
+    GST_ERROR_OBJECT (basesrc, "Failed to start in push mode");
+    return FALSE;
+  }
+seek_failed:
+  {
+    GST_ERROR_OBJECT (basesrc, "Failed to perform initial seek");
     gst_base_src_stop (basesrc);
-    GST_DEBUG_OBJECT (basesrc, "Failed to start in push mode");
+    if (event)
+      gst_event_unref (event);
+    return FALSE;
+  }
+deactivate_failed:
+  {
+    GST_ERROR_OBJECT (basesrc, "Failed to deactivate in push mode");
+    return FALSE;
+  }
+error_stop:
+  {
+    GST_DEBUG_OBJECT (basesrc, "Failed to stop in push mode");
     return FALSE;
   }
 }
@@ -1863,36 +1968,46 @@ gst_base_src_activate_pull (GstPad * pad, gboolean active)
   /* prepare subclass first */
   if (active) {
     GST_DEBUG_OBJECT (basesrc, "Activating in pull mode");
-    if (!gst_base_src_start (basesrc))
+    if (G_UNLIKELY (!gst_base_src_start (basesrc)))
       goto error_start;
 
     /* if not random_access, we cannot operate in pull mode for now */
-    if (!basesrc->random_access) {
-      gst_base_src_stop (basesrc);
-      return FALSE;
-    }
-    return TRUE;
+    if (G_UNLIKELY (!gst_base_src_check_get_range (basesrc)))
+      goto no_get_range;
   } else {
     GST_DEBUG_OBJECT (basesrc, "Deactivating in pull mode");
+    /* call the unlock function. We have no task to stop. */
+    if (G_UNLIKELY (!gst_base_src_deactivate (basesrc, pad)))
+      goto deactivate_failed;
 
     /* don't send EOS when going from PAUSED => READY when in pull mode */
     basesrc->priv->last_sent_eos = TRUE;
 
-    if (!gst_base_src_stop (basesrc))
+    if (G_UNLIKELY (!gst_base_src_stop (basesrc)))
       goto error_stop;
-
-    return gst_base_src_deactivate (basesrc, pad);
   }
+  return TRUE;
 
+  /* ERRORS */
 error_start:
   {
+    GST_ERROR_OBJECT (basesrc, "Failed to start in pull mode");
+    return FALSE;
+  }
+no_get_range:
+  {
+    GST_ERROR_OBJECT (basesrc, "Cannot operate in pull mode, stopping");
     gst_base_src_stop (basesrc);
-    GST_DEBUG_OBJECT (basesrc, "Failed to start in pull mode");
+    return FALSE;
+  }
+deactivate_failed:
+  {
+    GST_ERROR_OBJECT (basesrc, "Failed to deactivate in pull mode");
     return FALSE;
   }
 error_stop:
   {
-    GST_DEBUG_OBJECT (basesrc, "Failed to stop in pull mode");
+    GST_ERROR_OBJECT (basesrc, "Failed to stop in pull mode");
     return FALSE;
   }
 }
@@ -1949,9 +2064,6 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
     {
       GstEvent **event_p;
 
-      if (!gst_base_src_stop (basesrc))
-        goto error_stop;
-
       /* FIXME, deprecate this behaviour, it is very dangerous.
        * the prefered way of sending EOS downstream is by sending
        * the EOS event to the element */
@@ -1983,12 +2095,6 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
 failure:
   {
     GST_DEBUG_OBJECT (basesrc, "parent failed state change");
-    gst_base_src_stop (basesrc);
     return result;
   }
-error_stop:
-  {
-    GST_DEBUG_OBJECT (basesrc, "Failed to stop");
-    return GST_STATE_CHANGE_FAILURE;
-  }
 }
