@@ -60,18 +60,22 @@
  *     element negotiates the same caps on both pads.
  *   </para></listitem>
  *   <listitem><para>
- *     passthrough_on_same_caps on an element that doesn't implement a transform_caps
- *     function is useful for elements that only inspect data (such as level)
+ *     passthrough_on_same_caps on an element that doesn't implement a
+ *     transform_caps function is useful for elements that only inspect data
+ *     (such as level)
  *   </para></listitem>
  *   </itemizedlist>
  *   <itemizedlist>
  *   <title>Example elements</title>
  *     <listitem>Level</listitem>
- *     <listitem>Videoscale, audioconvert, ffmpegcolorspace, audioresample in certain modes.</listitem>
+ *     <listitem>Videoscale, audioconvert, ffmpegcolorspace, audioresample in
+ *     certain modes.</listitem>
  *   </itemizedlist>
  * </listitem>
  * <listitem>
- *   <itemizedlist><title>Modifications in-place - input buffer and output buffer are the same thing.</title>
+ *   <itemizedlist>
+ *     <title>Modifications in-place - input buffer and output buffer are the
+ *     same thing.</title>
  *   <listitem><para>
  *     The element must implement a transform_ip function.
  *   </para></listitem>
@@ -79,41 +83,46 @@
  *     Output buffer size must <= input buffer size
  *   </para></listitem>
  *   <listitem><para>
- *     If the always_in_place flag is set, non-writable buffers will be copied and
- *     passed to the transform_ip function, otherwise a new buffer will be created
- *     and the transform function called.
+ *     If the always_in_place flag is set, non-writable buffers will be copied
+ *     and passed to the transform_ip function, otherwise a new buffer will be
+ *     created and the transform function called.
  *   </para></listitem>
  *   <listitem><para>
- *     Incoming writable buffers will be passed to the transform_ip function immediately.
- *   </para></listitem>
+ *     Incoming writable buffers will be passed to the transform_ip function
+ *     immediately.  </para></listitem>
  *   <listitem><para>
- *     only implementing transform_ip and not transform implies always_in_place =
- *     TRUE
+ *     only implementing transform_ip and not transform implies always_in_place
+ *     = TRUE
  *   </para></listitem>
  *   </itemizedlist>
  *   <itemizedlist>
  *   <title>Example elements</title>
  *     <listitem>Volume</listitem>
- *     <listitem>Audioconvert in certain modes (signed/unsigned conversion)</listitem>
- *     <listitem>ffmpegcolorspace in certain modes (endianness swapping)</listitem>
+ *     <listitem>Audioconvert in certain modes (signed/unsigned
+ *     conversion)</listitem>
+ *     <listitem>ffmpegcolorspace in certain modes (endianness
+ *     swapping)</listitem>
  *   </itemizedlist>
  *  </listitem>
  * <listitem>
- *   <itemizedlist><title>Modifications only to the caps/metadata of a buffer</title>
+ *   <itemizedlist>
+ *   <title>Modifications only to the caps/metadata of a buffer</title>
  *   <listitem><para>
- *     The element does not require writable data, but non-writable buffers should
- *     be subbuffered so that the meta-information can be replaced.
+ *     The element does not require writable data, but non-writable buffers
+ *     should be subbuffered so that the meta-information can be replaced.
  *   </para></listitem>
  *   <listitem><para>
  *     Elements wishing to operate in this mode should replace the
- *     prepare_output_buffer method to create subbuffers of the input buffer and
- *     set always_in_place to TRUE
+ *     prepare_output_buffer method to create subbuffers of the input buffer
+ *     and set always_in_place to TRUE
  *   </para></listitem>
  *   </itemizedlist>
  *   <itemizedlist>
  *   <title>Example elements</title>
- *     <listitem>Capsfilter when setting caps on outgoing buffers that have none.</listitem>
- *     <listitem>identity when it is going to re-timestamp buffers by datarate.</listitem>
+ *     <listitem>Capsfilter when setting caps on outgoing buffers that have
+ *     none.</listitem>
+ *     <listitem>identity when it is going to re-timestamp buffers by
+ *     datarate.</listitem>
  *   </itemizedlist>
  * </listitem>
  * <listitem>
@@ -130,14 +139,15 @@
  *   </itemizedlist>
  *   <itemizedlist>
  *   <title>Example elements</title>
- *     <listitem>Videoscale, ffmpegcolorspace, audioconvert when doing scaling/conversions</listitem>
+ *     <listitem>Videoscale, ffmpegcolorspace, audioconvert when doing
+ *     scaling/conversions</listitem>
  *   </itemizedlist>
  * </listitem>
  * <listitem>
  *   <itemizedlist><title>Special output buffer allocations</title>
  *   <listitem><para>
- *     Elements which need to do special allocation of their output buffers other
- *     than what gst_buffer_pad_alloc allows should implement a
+ *     Elements which need to do special allocation of their output buffers
+ *     other than what gst_buffer_pad_alloc allows should implement a
  *     prepare_output_buffer method, which calls the parent implementation and
  *     passes the newly allocated buffer.
  *   </para></listitem>
@@ -157,8 +167,8 @@
  *       interested in modifying the buffers.
  *     </para></listitem>
  *     <listitem><para>
- *       Elements which are always in passthrough mode whenever the same caps has
- *       been negotiated on both pads can set the class variable
+ *       Elements which are always in passthrough mode whenever the same caps
+ *       has been negotiated on both pads can set the class variable
  *       passthrough_on_same_caps to have this behaviour automatically.
  *     </para></listitem>
  *   </itemizedlist>
@@ -188,6 +198,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "../../../gst/gst_private.h"
 #include "../../../gst/gst-i18n-lib.h"
 #include "gstbasetransform.h"
 #include <gst/gstmarshal.h>
@@ -202,9 +213,23 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_PROP_QOS	FALSE
+
 enum
 {
   PROP_0,
+  PROP_QOS
+};
+
+#define GST_BASE_TRANSFORM_GET_PRIVATE(obj)  \
+    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_BASE_TRANSFORM, GstBaseTransformPrivate))
+
+struct _GstBaseTransformPrivate
+{
+  /* QoS *//* with LOCK */
+  gboolean qos_enabled;
+  gdouble proportion;
+  GstClockTime earliest_time;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -255,8 +280,11 @@ static gboolean gst_base_transform_get_unit_size (GstBaseTransform * trans,
 static GstStateChangeReturn gst_base_transform_change_state (GstElement *
     element, GstStateChange transition);
 
-static gboolean gst_base_transform_event (GstPad * pad, GstEvent * event);
-static gboolean gst_base_transform_eventfunc (GstBaseTransform * trans,
+static gboolean gst_base_transform_src_event (GstPad * pad, GstEvent * event);
+static gboolean gst_base_transform_src_eventfunc (GstBaseTransform * trans,
+    GstEvent * event);
+static gboolean gst_base_transform_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_base_transform_sink_eventfunc (GstBaseTransform * trans,
     GstEvent * event);
 static GstFlowReturn gst_base_transform_getrange (GstPad * pad, guint64 offset,
     guint length, GstBuffer ** buffer);
@@ -297,6 +325,8 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
   gobject_class = G_OBJECT_CLASS (klass);
   gstelement_class = GST_ELEMENT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GstBaseTransformPrivate));
+
   parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property =
@@ -304,13 +334,18 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
   gobject_class->get_property =
       GST_DEBUG_FUNCPTR (gst_base_transform_get_property);
 
+  g_object_class_install_property (gobject_class, PROP_QOS,
+      g_param_spec_boolean ("qos", "QoS", "handle QoS messages",
+          DEFAULT_PROP_QOS, G_PARAM_READWRITE));
+
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_base_transform_finalize);
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_base_transform_change_state);
 
   klass->passthrough_on_same_caps = FALSE;
-  klass->event = GST_DEBUG_FUNCPTR (gst_base_transform_eventfunc);
+  klass->event = GST_DEBUG_FUNCPTR (gst_base_transform_sink_eventfunc);
+  klass->src_event = GST_DEBUG_FUNCPTR (gst_base_transform_src_eventfunc);
 }
 
 static void
@@ -321,6 +356,8 @@ gst_base_transform_init (GstBaseTransform * trans,
 
   GST_DEBUG ("gst_base_transform_init");
 
+  trans->priv = GST_BASE_TRANSFORM_GET_PRIVATE (trans);
+
   pad_template =
       gst_element_class_get_pad_template (GST_ELEMENT_CLASS (bclass), "sink");
   g_return_if_fail (pad_template != NULL);
@@ -330,7 +367,7 @@ gst_base_transform_init (GstBaseTransform * trans,
   gst_pad_set_setcaps_function (trans->sinkpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_setcaps));
   gst_pad_set_event_function (trans->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_base_transform_event));
+      GST_DEBUG_FUNCPTR (gst_base_transform_sink_event));
   gst_pad_set_chain_function (trans->sinkpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_chain));
   gst_pad_set_activatepush_function (trans->sinkpad,
@@ -347,6 +384,8 @@ gst_base_transform_init (GstBaseTransform * trans,
       GST_DEBUG_FUNCPTR (gst_base_transform_getcaps));
   gst_pad_set_setcaps_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_setcaps));
+  gst_pad_set_event_function (trans->srcpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_src_event));
   gst_pad_set_getrange_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_getrange));
   gst_pad_set_activatepull_function (trans->srcpad,
@@ -356,6 +395,7 @@ gst_base_transform_init (GstBaseTransform * trans,
   trans->transform_lock = g_mutex_new ();
   trans->delay_configure = FALSE;
   trans->pending_configure = FALSE;
+  trans->priv->qos_enabled = DEFAULT_PROP_QOS;
   trans->cache_caps1 = NULL;
   trans->cache_caps2 = NULL;
 
@@ -391,10 +431,11 @@ gst_base_transform_transform_caps (GstBaseTransform * trans,
 
     /* start with empty caps */
     ret = gst_caps_new_empty ();
+    GST_DEBUG_OBJECT (trans, "transform caps (direction = %d)", direction);
 
     if (gst_caps_is_any (caps)) {
       /* for any caps we still have to call the transform function */
-      GST_DEBUG_OBJECT (trans, "from ANY:");
+      GST_DEBUG_OBJECT (trans, "from: ANY");
       temp = klass->transform_caps (trans, direction, caps);
       GST_DEBUG_OBJECT (trans, "  to: %" GST_PTR_FORMAT, temp);
 
@@ -413,15 +454,25 @@ gst_base_transform_transform_caps (GstBaseTransform * trans,
         GST_DEBUG_OBJECT (trans, "  to[%d]: %" GST_PTR_FORMAT, i, temp);
 
         temp = gst_caps_make_writable (temp);
-        gst_caps_append (ret, temp);
+        /* FIXME: here we need to only append those structures, that are not yet
+         * in there
+         * gst_caps_append (ret, temp);
+         */
+        gst_caps_merge (ret, temp);
       }
+      GST_DEBUG_OBJECT (trans, "merged: (%d)", gst_caps_get_size (ret));
+      /* now simplify caps
+         gst_caps_do_simplify (ret);
+         GST_DEBUG_OBJECT (trans, "simplified: (%d)", gst_caps_get_size (ret));
+       */
     }
   } else {
     /* else use the identity transform */
     ret = gst_caps_ref (caps);
   }
 
-  GST_DEBUG_OBJECT (trans, "to:   %" GST_PTR_FORMAT, ret);
+  GST_DEBUG_OBJECT (trans, "to: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (ret),
+      ret);
 
   return ret;
 }
@@ -730,7 +781,8 @@ gst_base_transform_setcaps (GstPad * pad, GstCaps * caps)
   if (!peer_checked && otherpeer && !gst_pad_accept_caps (otherpeer, othercaps))
     goto peer_no_accept;
 
-  GST_DEBUG_OBJECT (trans, "got final caps %" GST_PTR_FORMAT, othercaps);
+  GST_DEBUG_OBJECT (trans, "Input caps were %" GST_PTR_FORMAT
+      ", and got final caps %" GST_PTR_FORMAT, caps, othercaps);
 
   trans->have_same_caps = gst_caps_is_equal (caps, othercaps);
   GST_DEBUG_OBJECT (trans, "have_same_caps: %d", trans->have_same_caps);
@@ -818,6 +870,10 @@ failed_configure:
 /* Allocate a buffer using gst_pad_alloc_buffer
  *
  * This function does not do renegotiation on the source pad
+ *
+ * The output buffer is always writable. outbuf can be equal to
+ * inbuf, the caller should be prepared for this and perform 
+ * appropriate refcounting.
  */
 static GstFlowReturn
 gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
@@ -833,6 +889,7 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
    * the old buffer. We will therefore delay the reconfiguration of the
    * element until we have processed this last buffer. */
   trans->delay_configure = TRUE;
+
   /* out_caps is the caps of the src pad gathered through the GST_PAD_CAPS 
      macro. If a set_caps occurs during this function this caps will become
      invalid. We want to keep them during preparation of the output buffer. */
@@ -846,6 +903,13 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
         out_buf);
     if (ret != GST_FLOW_OK)
       goto done;
+
+    /* decrease refcount again if vmethod returned refcounted in_buf. This
+     * is because we need to make sure that the buffer is writable for the
+     * in_place transform. The docs of the vmethod say that you should return
+     * a reffed inbuf, which is exactly what we don't want :), oh well.. */
+    if (in_buf == *out_buf)
+      gst_buffer_unref (in_buf);
   }
 
   /* See if we want to prepare the buffer for in place output */
@@ -853,18 +917,24 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
       && bclass->transform_ip) {
     if (gst_buffer_is_writable (in_buf)) {
       if (trans->have_same_caps) {
-        /* Input buffer is already writable and caps are the same, just ref and return it */
+        /* Input buffer is already writable and caps are the same, return input as
+         * output buffer. We don't take an additional ref since that would make the
+         * output buffer not writable anymore. Caller should be prepared to deal
+         * with proper refcounting of input/output buffers. */
         *out_buf = in_buf;
-        gst_buffer_ref (in_buf);
+        GST_LOG_OBJECT (trans, "reuse input buffer");
       } else {
         /* Writable buffer, but need to change caps => subbuffer */
         *out_buf = gst_buffer_create_sub (in_buf, 0, GST_BUFFER_SIZE (in_buf));
         gst_caps_replace (&GST_BUFFER_CAPS (*out_buf), out_caps);
+        GST_LOG_OBJECT (trans, "created sub-buffer of input buffer");
       }
+      /* we are done now */
       goto done;
     } else {
       /* Make a writable buffer below and copy the data */
       copy_inbuf = TRUE;
+      GST_LOG_OBJECT (trans, "need to copy input buffer to new output buffer");
     }
   }
 
@@ -901,6 +971,7 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
 done:
   if (out_caps)
     gst_caps_unref (out_caps);
+
   trans->delay_configure = FALSE;
 
   return ret;
@@ -975,6 +1046,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   GstBaseTransform *trans;
   GstFlowReturn res;
   guint new_size;
+  gboolean issinkcaps = TRUE;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
 
@@ -990,9 +1062,26 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   else
     GST_DEBUG_OBJECT (trans, "... and offset %" G_GUINT64_FORMAT, offset);
 
+  /* if have_same_caps was previously set to TRUE we need to double check if it
+   * hasn't changed */
+  if (trans->have_same_caps) {
+    GstCaps *sinkcaps;
+
+    GST_OBJECT_LOCK (trans->sinkpad);
+    sinkcaps = GST_PAD_CAPS (trans->sinkpad);
+    issinkcaps = sinkcaps && (gst_caps_is_equal (sinkcaps, caps));
+    GST_OBJECT_UNLOCK (trans->sinkpad);
+  }
+
   /* before any buffers are pushed, have_same_caps is TRUE; allocating can trigger
    * a renegotiation and change that to FALSE */
-  if (trans->have_same_caps) {
+
+  /* bilboed: This seems wrong, from all debug logs, have_same_caps is
+   * initialized to FALSE */
+
+  /* checking against trans->have_same_caps is not enough !! It should also
+   *  check to see if the requested caps are equal to the sink caps */
+  if (trans->have_same_caps && issinkcaps) {
     /* request a buffer with the same caps */
     GST_DEBUG_OBJECT (trans, "requesting buffer with same caps, size %d", size);
 
@@ -1001,20 +1090,26 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
         buf);
   } else {
     /* if we are configured, request a buffer with the src caps */
-    GstCaps *srccaps = gst_pad_get_negotiated_caps (trans->srcpad);
-    GstCaps *sinkcaps = gst_pad_get_negotiated_caps (trans->sinkpad);
+    GstCaps *srccaps;
+    GstCaps *sinkcaps;
+    gboolean configured;
 
-    if (!srccaps)
+    /* take lock, peek if the caps are ok */
+    GST_OBJECT_LOCK (trans->sinkpad);
+    sinkcaps = GST_PAD_CAPS (trans->sinkpad);
+    configured = (sinkcaps == NULL || gst_caps_is_equal (sinkcaps, caps));
+    GST_OBJECT_UNLOCK (trans->sinkpad);
+    if (!configured)
       goto not_configured;
 
-    if (sinkcaps != NULL) {
-      if (sinkcaps != caps || !gst_caps_is_equal (sinkcaps, caps)) {
-        gst_caps_unref (sinkcaps);
-        gst_caps_unref (srccaps);
-        goto not_configured;
-      }
-      gst_caps_unref (sinkcaps);
-    }
+    /* take lock on srcpad to grab the caps, caps can change when pushing a
+     * buffer. */
+    GST_OBJECT_LOCK (trans->srcpad);
+    if ((srccaps = GST_PAD_CAPS (trans->srcpad)))
+      gst_caps_ref (srccaps);
+    GST_OBJECT_UNLOCK (trans->srcpad);
+    if (!srccaps)
+      goto not_configured;
 
     GST_DEBUG_OBJECT (trans, "calling transform_size");
     if (!gst_base_transform_transform_size (trans,
@@ -1026,6 +1121,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     res =
         gst_pad_alloc_buffer_and_set_caps (trans->srcpad, offset, new_size,
         srccaps, buf);
+
     gst_caps_unref (srccaps);
   }
 
@@ -1035,12 +1131,25 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
        the alloc_buffer served to transmit caps information but we can't use the
        buffer. fall through and allocate a buffer corresponding to our sink
        caps, if any */
-    GstCaps *sinkcaps = gst_pad_get_negotiated_caps (trans->sinkpad);
-    GstCaps *srccaps = gst_pad_get_negotiated_caps (trans->srcpad);
+    GstCaps *sinkcaps;
+    GstCaps *srccaps;
 
+    GST_OBJECT_LOCK (trans->sinkpad);
+    if ((sinkcaps = GST_PAD_CAPS (trans->sinkpad)))
+      gst_caps_ref (sinkcaps);
+    GST_OBJECT_UNLOCK (trans->sinkpad);
     if (!sinkcaps)
       goto not_configured;
 
+    GST_OBJECT_LOCK (trans->srcpad);
+    if ((srccaps = GST_PAD_CAPS (trans->srcpad)))
+      gst_caps_ref (srccaps);
+    GST_OBJECT_UNLOCK (trans->srcpad);
+    if (!srccaps) {
+      gst_caps_unref (sinkcaps);
+      goto not_configured;
+    }
+
     if (!gst_base_transform_transform_size (trans,
             GST_PAD_DIRECTION (trans->srcpad), srccaps, GST_BUFFER_SIZE (*buf),
             sinkcaps, &new_size)) {
@@ -1048,16 +1157,17 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
       gst_caps_unref (sinkcaps);
       goto unknown_size;
     }
+    /* don't need the caps anymore now */
+    gst_caps_unref (srccaps);
 
     gst_buffer_unref (*buf);
 
     *buf = gst_buffer_new_and_alloc (new_size);
-    gst_buffer_set_caps (*buf, sinkcaps);
     GST_BUFFER_OFFSET (*buf) = offset;
-    res = GST_FLOW_OK;
+    /* set caps, gives away the ref */
+    GST_BUFFER_CAPS (*buf) = sinkcaps;
 
-    gst_caps_unref (srccaps);
-    gst_caps_unref (sinkcaps);
+    res = GST_FLOW_OK;
   }
 
 done:
@@ -1068,24 +1178,38 @@ done:
 
 not_configured:
   {
-    /* let the default allocator handle it */
+    /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "not configured");
     gst_buffer_replace (buf, NULL);
-    res = GST_FLOW_OK;
+    if (trans->passthrough || trans->always_in_place) {
+      /* ...by calling alloc_buffer without setting caps on the src pad, which
+       * will force negotiation in the chain function. */
+      res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
+    } else {
+      /* ...by letting the default handler create a buffer */
+      res = GST_FLOW_OK;
+    }
     goto done;
   }
 unknown_size:
   {
-    /* let the default allocator handle it */
+    /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "unknown size");
     gst_buffer_replace (buf, NULL);
-    res = GST_FLOW_OK;
+    if (trans->passthrough || trans->always_in_place) {
+      /* ...by calling alloc_buffer without setting caps on the src pad, which
+       * will force negotiation in the chain function. */
+      res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
+    } else {
+      /* ...by letting the default handler create a buffer */
+      res = GST_FLOW_OK;
+    }
     goto done;
   }
 }
 
 static gboolean
-gst_base_transform_event (GstPad * pad, GstEvent * event)
+gst_base_transform_sink_event (GstPad * pad, GstEvent * event)
 {
   GstBaseTransform *trans;
   GstBaseTransformClass *bclass;
@@ -1100,7 +1224,7 @@ gst_base_transform_event (GstPad * pad, GstEvent * event)
   /* FIXME, do this in the default event handler so the subclass can do
    * something different. */
   if (ret)
-    ret = gst_pad_event_default (pad, event);
+    ret = gst_pad_push_event (trans->srcpad, event);
 
   gst_object_unref (trans);
 
@@ -1108,13 +1232,19 @@ gst_base_transform_event (GstPad * pad, GstEvent * event)
 }
 
 static gboolean
-gst_base_transform_eventfunc (GstBaseTransform * trans, GstEvent * event)
+gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
 {
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
       break;
     case GST_EVENT_FLUSH_STOP:
+      GST_OBJECT_LOCK (trans);
+      /* reset QoS parameters */
+      trans->priv->proportion = 1.0;
+      trans->priv->earliest_time = -1;
+      GST_OBJECT_UNLOCK (trans);
       /* we need new segment info after the flush. */
+      trans->have_newsegment = FALSE;
       gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
       break;
     case GST_EVENT_EOS:
@@ -1124,23 +1254,20 @@ gst_base_transform_eventfunc (GstBaseTransform * trans, GstEvent * event)
     case GST_EVENT_NEWSEGMENT:
     {
       GstFormat format;
-      gdouble rate;
+      gdouble rate, arate;
       gint64 start, stop, time;
       gboolean update;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
-
-      if (trans->segment.format != format)
-        gst_segment_init (&trans->segment, format);
-
-      gst_segment_set_newsegment (&trans->segment, update, rate, format, start,
-          stop, time);
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
+          &start, &stop, &time);
 
       trans->have_newsegment = TRUE;
 
+      gst_segment_set_newsegment_full (&trans->segment, update, rate, arate,
+          format, start, stop, time);
+
       if (format == GST_FORMAT_TIME) {
-        GST_DEBUG_OBJECT (trans, "received NEW_SEGMENT %" GST_TIME_FORMAT
+        GST_DEBUG_OBJECT (trans, "received TIME NEW_SEGMENT %" GST_TIME_FORMAT
             " -- %" GST_TIME_FORMAT ", time %" GST_TIME_FORMAT
             ", accum %" GST_TIME_FORMAT,
             GST_TIME_ARGS (trans->segment.start),
@@ -1163,6 +1290,53 @@ gst_base_transform_eventfunc (GstBaseTransform * trans, GstEvent * event)
   return TRUE;
 }
 
+static gboolean
+gst_base_transform_src_event (GstPad * pad, GstEvent * event)
+{
+  GstBaseTransform *trans;
+  GstBaseTransformClass *bclass;
+  gboolean ret = TRUE;
+
+  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
+
+  if (bclass->src_event)
+    ret = bclass->src_event (trans, event);
+
+  gst_object_unref (trans);
+
+  return ret;
+}
+
+static gboolean
+gst_base_transform_src_eventfunc (GstBaseTransform * trans, GstEvent * event)
+{
+  gboolean ret;
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK:
+      break;
+    case GST_EVENT_NAVIGATION:
+      break;
+    case GST_EVENT_QOS:
+    {
+      gdouble proportion;
+      GstClockTimeDiff diff;
+      GstClockTime timestamp;
+
+      gst_event_parse_qos (event, &proportion, &diff, &timestamp);
+      gst_base_transform_update_qos (trans, proportion, diff, timestamp);
+      break;
+    }
+    default:
+      break;
+  }
+
+  ret = gst_pad_push_event (trans->sinkpad, event);
+
+  return ret;
+}
+
 static GstFlowReturn
 gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
     GstBuffer ** outbuf)
@@ -1171,6 +1345,7 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
   GstFlowReturn ret = GST_FLOW_OK;
   guint out_size;
   gboolean want_in_place;
+  GstClockTime qostime;
 
   bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
@@ -1189,6 +1364,34 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
   if (!trans->negotiated && !trans->passthrough && (bclass->set_caps != NULL))
     goto not_negotiated;
 
+  /* can only do QoS if the segment is in TIME */
+  if (trans->segment.format != GST_FORMAT_TIME)
+    goto no_qos;
+
+  qostime = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
+      GST_BUFFER_TIMESTAMP (inbuf));
+
+  if (qostime != -1) {
+    gboolean need_skip;
+    GstClockTime earliest_time;
+
+    GST_OBJECT_LOCK (trans);
+    earliest_time = trans->priv->earliest_time;
+    /* check for QoS, don't perform conversion for buffers
+     * that are known to be late. */
+    need_skip = trans->priv->qos_enabled &&
+        earliest_time != -1 && qostime != -1 && qostime <= earliest_time;
+    GST_OBJECT_UNLOCK (trans);
+
+    if (need_skip) {
+      GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, trans, "skipping transform: qostime %"
+          GST_TIME_FORMAT " <= %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (qostime), GST_TIME_ARGS (earliest_time));
+      goto skip;
+    }
+  }
+
+no_qos:
   if (trans->passthrough) {
     /* In passthrough mode, give transform_ip a look at the
      * buffer, without making it writable, or just push the
@@ -1260,7 +1463,11 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
     if (!success)
       goto configure_failed;
   }
-  gst_buffer_unref (inbuf);
+
+skip:
+  /* only unref input buffer if we allocated a new outbuf buffer */
+  if (*outbuf != inbuf)
+    gst_buffer_unref (inbuf);
 
   return ret;
 
@@ -1289,7 +1496,8 @@ no_buffer:
   }
 configure_failed:
   {
-    gst_buffer_unref (inbuf);
+    if (*outbuf != inbuf)
+      gst_buffer_unref (inbuf);
     GST_DEBUG_OBJECT (trans, "could not negotiate");
     return GST_FLOW_NOT_NEGOTIATED;
   }
@@ -1334,10 +1542,13 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
   ret = gst_base_transform_handle_buffer (trans, buffer, &outbuf);
   g_mutex_unlock (trans->transform_lock);
 
-  if (ret == GST_FLOW_OK) {
-    ret = gst_pad_push (trans->srcpad, outbuf);
-  } else if (outbuf != NULL)
-    gst_buffer_unref (outbuf);
+  /* outbuf can be NULL, this means a dropped buffer */
+  if (outbuf != NULL) {
+    if ((ret == GST_FLOW_OK))
+      ret = gst_pad_push (trans->srcpad, outbuf);
+    else
+      gst_buffer_unref (outbuf);
+  }
 
   gst_object_unref (trans);
 
@@ -1353,6 +1564,9 @@ gst_base_transform_set_property (GObject * object, guint prop_id,
   trans = GST_BASE_TRANSFORM (object);
 
   switch (prop_id) {
+    case PROP_QOS:
+      gst_base_transform_set_qos_enabled (trans, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1368,6 +1582,9 @@ gst_base_transform_get_property (GObject * object, guint prop_id,
   trans = GST_BASE_TRANSFORM (object);
 
   switch (prop_id) {
+    case PROP_QOS:
+      g_value_set_boolean (value, gst_base_transform_is_qos_enabled (trans));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1441,6 +1658,8 @@ gst_base_transform_change_state (GstElement * element,
       trans->negotiated = FALSE;
       trans->have_newsegment = FALSE;
       gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
+      trans->priv->proportion = 1.0;
+      trans->priv->earliest_time = -1;
       GST_OBJECT_UNLOCK (trans);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
@@ -1598,3 +1817,82 @@ gst_base_transform_is_in_place (GstBaseTransform * trans)
 
   return result;
 }
+
+/**
+ * gst_base_transform_update_qos:
+ * @trans: a #GstBaseTransform
+ * @proportion: the proportion
+ * @diff: the diff against the clock
+ * @timestamp: the timestamp of the buffer generating the QoS
+ *
+ * Set the QoS parameters in the transform.
+ *
+ * Since: 0.10.5
+ *
+ * MT safe.
+ */
+void
+gst_base_transform_update_qos (GstBaseTransform * trans,
+    gdouble proportion, GstClockTimeDiff diff, GstClockTime timestamp)
+{
+
+  g_return_if_fail (trans != NULL);
+
+  GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, trans,
+      "qos: proportion: %lf, diff %" G_GINT64_FORMAT ", timestamp %"
+      GST_TIME_FORMAT, proportion, diff, GST_TIME_ARGS (timestamp));
+
+  GST_OBJECT_LOCK (trans);
+  trans->priv->proportion = proportion;
+  trans->priv->earliest_time = timestamp + diff;
+  GST_OBJECT_UNLOCK (trans);
+}
+
+/**
+ * gst_base_transform_set_qos_enabled:
+ * @trans: a #GstBaseTransform
+ * @enabled: new state
+ *
+ * Enable or disable QoS handling in the transform.
+ *
+ * Since: 0.10.5
+ *
+ * MT safe.
+ */
+void
+gst_base_transform_set_qos_enabled (GstBaseTransform * trans, gboolean enabled)
+{
+  g_return_if_fail (trans != NULL);
+
+  GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, trans, "enabled: %d", enabled);
+
+  GST_OBJECT_LOCK (trans);
+  trans->priv->qos_enabled = enabled;
+  GST_OBJECT_UNLOCK (trans);
+}
+
+/**
+ * gst_base_transform_is_qos_enabled:
+ * @trans: a #GstBaseTransform
+ *
+ * Queries if the transform will handle QoS.
+ *
+ * Returns: TRUE if QoS is enabled.
+ *
+ * Since: 0.10.5
+ *
+ * MT safe.
+ */
+gboolean
+gst_base_transform_is_qos_enabled (GstBaseTransform * trans)
+{
+  gboolean result;
+
+  g_return_val_if_fail (trans != NULL, FALSE);
+
+  GST_OBJECT_LOCK (trans);
+  result = trans->priv->qos_enabled;
+  GST_OBJECT_UNLOCK (trans);
+
+  return result;
+}
