@@ -297,7 +297,7 @@ buf_helper_find_peek (gpointer data, gint64 off, guint size)
     return NULL;
   }
 
-  if ((off + size) < helper->size)
+  if ((off + size) <= helper->size)
     return helper->data + off;
 
   return NULL;
