@@ -73,23 +73,34 @@ GstPad * gst_check_setup_sink_pad (GstElement *element,
     GstStaticPadTemplate *template, GstCaps *caps);
 void gst_check_teardown_sink_pad (GstElement *element);
 void gst_check_abi_list (GstCheckABIStruct list[], gboolean have_abi_sizes);
-
-
+gint gst_check_run_suite (Suite *suite, const gchar *name, const gchar *fname);
 
 #define fail_unless_message_error(msg, domain, code)		\
 gst_check_message_error (msg, GST_MESSAGE_ERROR,		\
   GST_ ## domain ## _ERROR, GST_ ## domain ## _ERROR_ ## code)
+#define assert_message_error(m, d, c) fail_unless_message_error(m, d, c)
 
 /***
  * wrappers for START_TEST and END_TEST
  */
+#if CHECK_MAJOR_VERSION >= 0 && CHECK_MINOR_VERSION >= 9 && CHECK_MICRO_VERSION >= 4
 #define GST_START_TEST(__testname) \
-static void __testname (void)\
+static void __testname (int __i__)\
 {\
   GST_DEBUG ("test start"); \
   tcase_fn_start (""# __testname, __FILE__, __LINE__);
 
 #define GST_END_TEST END_TEST
+#else
+#define GST_START_TEST(__testname) \
+static void __testname ()\
+{\
+  GST_DEBUG ("test start"); \
+  tcase_fn_start (""# __testname, __FILE__, __LINE__);
+
+#define GST_END_TEST END_TEST
+#endif
+
 
 /* additional fail macros */
 #define fail_unless_equals_int(a, b)					\
@@ -99,6 +110,7 @@ G_STMT_START {								\
   fail_unless(first == second,						\
     "'" #a "' (%d) is not equal to '" #b"' (%d)", first, second);	\
 } G_STMT_END;
+#define assert_equals_int(a, b) fail_unless_equals_int(a, b)
 
 #define fail_unless_equals_uint64(a, b)					\
 G_STMT_START {								\
@@ -108,14 +120,16 @@ G_STMT_START {								\
     "'" #a "' (%" G_GUINT64_FORMAT ") is not equal to '" #b"' (%"	\
     G_GUINT64_FORMAT ")", first, second);				\
 } G_STMT_END;
+#define assert_equals_uint64(a, b) fail_unless_equals_uint64(a, b)
 
-#define fail_unless_equals_string(a, b)					\
-G_STMT_START {								\
-  gchar * first = a;							\
-  gchar * second = b;							\
-  fail_unless(strcmp (first, second) == 0,				\
-    "'" #a "' (%s) is not equal to '" #b"' (%s)", first, second);	\
+#define fail_unless_equals_string(a, b)                             \
+G_STMT_START {                                                      \
+  const gchar * first = a;                                          \
+  const gchar * second = b;                                         \
+  fail_unless(strcmp (first, second) == 0,                          \
+    "'" #a "' (%s) is not equal to '" #b"' (%s)", first, second);   \
 } G_STMT_END;
+#define assert_equals_string(a, b) fail_unless_equals_string(a, b)
 
 
 /***
@@ -217,7 +231,7 @@ G_STMT_START {							\
   _gst_check_raised_critical = FALSE;				\
   code;								\
   _fail_unless (_gst_check_raised_critical, __FILE__, __LINE__, \
-                "Expected g_critical, got nothing");            \
+                "Expected g_critical, got nothing", NULL);      \
   _gst_check_expecting_log = FALSE;				\
 } G_STMT_END
 
@@ -227,7 +241,7 @@ G_STMT_START {							\
   _gst_check_raised_warning = FALSE;				\
   code;								\
   _fail_unless (_gst_check_raised_warning, __FILE__, __LINE__,  \
-                "Expected g_warning, got nothing");             \
+                "Expected g_warning, got nothing", NULL);       \
   _gst_check_expecting_log = FALSE;				\
 } G_STMT_END
 
@@ -277,3 +291,10 @@ fail_unless (gst_element_set_state (element,			\
 
 #endif /* __GST_CHECK_H__ */
 
+#define GST_CHECK_MAIN(name)					\
+int main (int argc, char **argv)				\
+{								\
+  Suite *s = name ## _suite ();					\
+  gst_check_init (&argc, &argv);				\
+  return gst_check_run_suite (s, # name, __FILE__);		\
+}
