@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
+ *                    2006 Wim Taymans <wim@fluendo.com>
  *
  * gstfilesink.c:
  *
@@ -80,6 +81,9 @@ static gboolean gst_file_sink_event (GstBaseSink * sink, GstEvent * event);
 static GstFlowReturn gst_file_sink_render (GstBaseSink * sink,
     GstBuffer * buffer);
 
+static gboolean gst_file_sink_do_seek (GstFileSink * filesink,
+    guint64 new_offset);
+
 static gboolean gst_file_sink_query (GstPad * pad, GstQuery * query);
 
 static void gst_file_sink_uri_handler_init (gpointer g_iface,
@@ -136,7 +140,8 @@ gst_file_sink_class_init (GstFileSinkClass * klass)
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_file_sink_event);
 
   if (sizeof (off_t) < 8) {
-    GST_LOG ("No large file support, sizeof (off_t) = %u", sizeof (off_t));
+    GST_LOG ("No large file support, sizeof (off_t) = %" G_GSIZE_FORMAT "!",
+        sizeof (off_t));
   }
 }
 
@@ -152,7 +157,7 @@ gst_file_sink_init (GstFileSink * filesink, GstFileSinkClass * g_class)
   filesink->filename = NULL;
   filesink->file = NULL;
 
-  GST_BASE_SINK (filesink)->sync = FALSE;
+  gst_base_sink_set_sync (GST_BASE_SINK (filesink), FALSE);
 }
 
 static void
@@ -171,11 +176,8 @@ gst_file_sink_dispose (GObject * object)
 static gboolean
 gst_file_sink_set_location (GstFileSink * sink, const gchar * location)
 {
-  if (sink->file) {
-    g_warning ("Changing the `location' property on filesink when "
-        "a file is open not supported.");
-    return FALSE;
-  }
+  if (sink->file)
+    goto was_open;
 
   g_free (sink->filename);
   g_free (sink->uri);
@@ -188,6 +190,14 @@ gst_file_sink_set_location (GstFileSink * sink, const gchar * location)
   }
 
   return TRUE;
+
+  /* ERRORS */
+was_open:
+  {
+    g_warning ("Changing the `location' property on filesink when "
+        "a file is open not supported.");
+    return FALSE;
+  }
 }
 static void
 gst_file_sink_set_property (GObject * object, guint prop_id,
@@ -225,33 +235,56 @@ static gboolean
 gst_file_sink_open_file (GstFileSink * sink)
 {
   /* open the file */
-  if (sink->filename == NULL || sink->filename[0] == '\0') {
+  if (sink->filename == NULL || sink->filename[0] == '\0')
+    goto no_filename;
+
+  sink->file = fopen (sink->filename, "wb");
+  if (sink->file == NULL)
+    goto open_failed;
+
+  sink->data_written = 0;
+  /* try to seek in the file to figure out if it is seekable */
+  sink->seekable = gst_file_sink_do_seek (sink, 0);
+
+  GST_DEBUG_OBJECT (sink, "opened file %s, seekable %d",
+      sink->filename, sink->seekable);
+
+  return TRUE;
+
+  /* ERRORS */
+no_filename:
+  {
     GST_ELEMENT_ERROR (sink, RESOURCE, NOT_FOUND,
         (_("No file name specified for writing.")), (NULL));
     return FALSE;
   }
-
-  sink->file = fopen (sink->filename, "wb");
-  if (sink->file == NULL) {
+open_failed:
+  {
     GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE,
         (_("Could not open file \"%s\" for writing."), sink->filename),
         GST_ERROR_SYSTEM);
     return FALSE;
   }
-
-  sink->data_written = 0;
-
-  return TRUE;
 }
 
 static void
 gst_file_sink_close_file (GstFileSink * sink)
 {
   if (sink->file) {
-    if (fclose (sink->file) != 0) {
-      GST_ELEMENT_ERROR (sink, RESOURCE, CLOSE,
-          (_("Error closing file \"%s\"."), sink->filename), GST_ERROR_SYSTEM);
-    }
+    if (fclose (sink->file) != 0)
+      goto close_failed;
+
+    GST_DEBUG_OBJECT (sink, "closed file");
+    sink->file = NULL;
+  }
+  return;
+
+  /* ERRORS */
+close_failed:
+  {
+    GST_ELEMENT_ERROR (sink, RESOURCE, CLOSE,
+        (_("Error closing file \"%s\"."), sink->filename), GST_ERROR_SYSTEM);
+    return;
   }
 }
 
@@ -292,7 +325,7 @@ gst_file_sink_query (GstPad * pad, GstQuery * query)
 # define __GST_STDIO_SEEK_FUNCTION "fseek"
 #endif
 
-static void
+static gboolean
 gst_file_sink_do_seek (GstFileSink * filesink, guint64 new_offset)
 {
   GST_DEBUG_OBJECT (filesink, "Seeking to offset %" G_GUINT64_FORMAT
@@ -313,18 +346,18 @@ gst_file_sink_do_seek (GstFileSink * filesink, guint64 new_offset)
     goto seek_failed;
 #endif
 
-  return;
+  return TRUE;
 
   /* ERRORS */
 flush_failed:
   {
     GST_DEBUG_OBJECT (filesink, "Flush failed: %s", g_strerror (errno));
-    return;
+    return FALSE;
   }
 seek_failed:
   {
     GST_DEBUG_OBJECT (filesink, "Seeking failed: %s", g_strerror (errno));
-    return;
+    return FALSE;
   }
 }
 
@@ -342,31 +375,50 @@ gst_file_sink_event (GstBaseSink * sink, GstEvent * event)
   switch (type) {
     case GST_EVENT_NEWSEGMENT:
     {
-      gint64 soffset, eoffset;
+      gint64 start, stop, pos;
       GstFormat format;
 
-      gst_event_parse_new_segment (event, NULL, NULL, &format, &soffset,
-          &eoffset, NULL);
+      gst_event_parse_new_segment (event, NULL, NULL, &format, &start,
+          &stop, &pos);
 
       if (format == GST_FORMAT_BYTES) {
-        gst_file_sink_do_seek (filesink, (guint64) soffset);
+        /* FIXME, the seek should be performed on the pos field, start/stop are
+         * just boundaries for valid bytes offsets. We should also fill the file
+         * with zeroes if the new position extends the current EOF (sparse streams
+         * and segment accumulation). */
+        if (!gst_file_sink_do_seek (filesink, (guint64) start))
+          goto seek_failed;
       } else {
-        GST_DEBUG ("Ignored NEWSEGMENT event of format %u", (guint) format);
+        GST_DEBUG ("Ignored NEWSEGMENT event of format %u (%s)",
+            (guint) format, gst_format_get_name (format));
       }
       break;
     }
     case GST_EVENT_EOS:
-      if (fflush (filesink->file)) {
-        GST_ELEMENT_ERROR (filesink, RESOURCE, WRITE,
-            (_("Error while writing to file \"%s\"."), filesink->filename),
-            GST_ERROR_SYSTEM);
-      }
+      if (fflush (filesink->file))
+        goto flush_failed;
       break;
     default:
       break;
   }
 
   return TRUE;
+
+  /* ERRORS */
+seek_failed:
+  {
+    GST_ELEMENT_ERROR (filesink, RESOURCE, SEEK,
+        (_("Error while seeking in file \"%s\"."), filesink->filename),
+        GST_ERROR_SYSTEM);
+    return FALSE;
+  }
+flush_failed:
+  {
+    GST_ELEMENT_ERROR (filesink, RESOURCE, WRITE,
+        (_("Error while writing to file \"%s\"."), filesink->filename),
+        GST_ERROR_SYSTEM);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -403,8 +455,12 @@ gst_file_sink_render (GstBaseSink * sink, GstBuffer * buffer)
 
   filesink = GST_FILE_SINK (sink);
 
-  if (!gst_file_sink_get_current_offset (filesink, &cur_pos))
-    goto handle_error;
+  if (filesink->seekable) {
+    if (!gst_file_sink_get_current_offset (filesink, &cur_pos))
+      goto handle_error;
+  } else {
+    cur_pos = filesink->data_written;
+  }
 
   if (cur_pos < filesink->data_written)
     back_pending = filesink->data_written - cur_pos;
@@ -453,7 +509,7 @@ gst_file_sink_stop (GstBaseSink * basesink)
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_file_sink_uri_get_type (void)
 {
   return GST_URI_SINK;
