@@ -146,6 +146,7 @@ static void gst_queue_get_property (GObject * object,
 static GstFlowReturn gst_queue_chain (GstPad * pad, GstBuffer * buffer);
 static GstFlowReturn gst_queue_bufferalloc (GstPad * pad, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf);
+static gboolean gst_queue_acceptcaps (GstPad * pad, GstCaps * caps);
 static gboolean gst_queue_push_one (GstQueue * queue);
 static void gst_queue_loop (GstPad * pad);
 
@@ -357,6 +358,8 @@ gst_queue_init (GstQueue * queue)
       GST_DEBUG_FUNCPTR (gst_queue_src_activate_push));
   gst_pad_set_link_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_link_src));
+  gst_pad_set_acceptcaps_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_acceptcaps));
   gst_pad_set_getcaps_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_event_function (queue->srcpad,
@@ -476,13 +479,19 @@ gst_queue_bufferalloc (GstPad * pad, guint64 offset, guint size, GstCaps * caps,
 
   queue = GST_QUEUE (GST_PAD_PARENT (pad));
 
-  result =
-      gst_pad_alloc_buffer_and_set_caps (queue->srcpad, offset, size, caps,
-      buf);
+  /* Forward to src pad, without setting caps on the src pad */
+  result = gst_pad_alloc_buffer (queue->srcpad, offset, size, caps, buf);
 
   return result;
 }
 
+static gboolean
+gst_queue_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  /* The only time our acceptcaps method should be called is on the srcpad
+   * when we push a buffer, in which case we always accept those caps */
+  return TRUE;
+}
 
 static void
 gst_queue_locked_flush (GstQueue * queue)
@@ -645,6 +654,13 @@ gst_queue_chain (GstPad * pad, GstBuffer * buffer)
         GList *item;
         GstMiniObject *leak = NULL;
 
+        if (!gst_queue_is_filled (queue)) {
+          /* Queue was emptied while we sent out the signal, so no need to drop */
+          GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
+              "queue emptied while emitting signal, not leaking buffer");
+          break;
+        }
+
         GST_CAT_DEBUG_OBJECT (queue_dataflow, queue,
             "queue is full, leaking buffer on downstream end");
 
@@ -777,12 +793,7 @@ gst_queue_push_one (GstQueue * queue)
       flowname = gst_flow_get_name (result);
 
       queue->srcresult = result;
-      if (GST_FLOW_IS_FATAL (result)) {
-        GST_ELEMENT_ERROR (queue, STREAM, FAILED,
-            (_("Internal data flow error.")),
-            ("streaming stopped, reason %s", flowname));
-        gst_pad_push_event (queue->srcpad, gst_event_new_eos ());
-      }
+
       GST_DEBUG_OBJECT (queue, "pausing queue, reason %s", flowname);
       gst_pad_pause_task (queue->srcpad);
     }
