@@ -217,6 +217,32 @@ GST_START_TEST (create_custom_events)
     /* The structure should have been duplicated */
     fail_if (gst_event_get_structure (event) ==
         gst_event_get_structure (event2));
+
+    gst_event_unref (event);
+    gst_event_unref (event2);
+  }
+
+  /* Make events writable */
+  {
+    structure = gst_structure_empty_new ("application/x-custom");
+    fail_if (structure == NULL);
+    event = gst_event_new_custom (GST_EVENT_CUSTOM_BOTH, structure);
+    /* ref the event so that it becomes non-writable */
+    gst_event_ref (event);
+    gst_event_ref (event);
+    /* this should fail if the structure isn't writable */
+    ASSERT_CRITICAL (gst_structure_remove_all_fields ((GstStructure *)
+            gst_event_get_structure (event)));
+
+    /* now make writable */
+    event2 =
+        GST_EVENT (gst_mini_object_make_writable (GST_MINI_OBJECT (event)));
+    fail_unless (event != event2);
+    /* this fail if the structure isn't writable */
+    gst_structure_remove_all_fields ((GstStructure *)
+        gst_event_get_structure (event2));
+
+    gst_event_unref (event);
     gst_event_unref (event);
     gst_event_unref (event2);
   }
@@ -274,6 +300,7 @@ static void test_event
     gboolean expect_before_q, GstPad * fake_srcpad)
 {
   GstEvent *event;
+  GstPad *peer;
   gint i;
 
   got_event_before_q = got_event_after_q = NULL;
@@ -288,26 +315,36 @@ static void test_event
   got_event_time.tv_sec = 0;
   got_event_time.tv_usec = 0;
 
+  /* We block the pad so the stream lock is released and we can send the event */
   fail_unless (gst_pad_set_blocked (fake_srcpad, TRUE) == TRUE);
-  gst_pad_push_event (pad, event);
+
+  /* We send on the peer pad, since the pad is blocked */
+  fail_unless ((peer = gst_pad_get_peer (pad)) != NULL);
+  gst_pad_send_event (peer, event);
+  gst_object_unref (peer);
+
   fail_unless (gst_pad_set_blocked (fake_srcpad, FALSE) == TRUE);
 
-  /* Wait up to 5 seconds for the event to appear */
   if (expect_before_q) {
+    /* Wait up to 5 seconds for the event to appear */
     for (i = 0; i < 500; i++) {
       g_usleep (G_USEC_PER_SEC / 100);
       if (got_event_before_q != NULL)
         break;
     }
-    fail_if (got_event_before_q == NULL);
+    fail_if (got_event_before_q == NULL,
+        "Expected event failed to appear upstream of the queue "
+        "within 5 seconds");
     fail_unless (GST_EVENT_TYPE (got_event_before_q) == type);
   } else {
-    for (i = 0; i < 500; i++) {
+    /* Wait up to 10 seconds for the event to appear */
+    for (i = 0; i < 1000; i++) {
       g_usleep (G_USEC_PER_SEC / 100);
       if (got_event_after_q != NULL)
         break;
     }
-    fail_if (got_event_after_q == NULL);
+    fail_if (got_event_after_q == NULL,
+        "Expected event failed to appear after the queue within 10 seconds");
     fail_unless (GST_EVENT_TYPE (got_event_after_q) == type);
   }
 
@@ -419,7 +456,7 @@ GST_START_TEST (send_custom_events)
 GST_END_TEST;
 
 Suite *
-gstevents_suite (void)
+gst_event_suite (void)
 {
   Suite *s = suite_create ("GstEvent");
   TCase *tc_chain = tcase_create ("customevents");
@@ -432,19 +469,4 @@ gstevents_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gstevents_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_event);
