@@ -90,6 +90,7 @@ GST_START_TEST (test_link_unlink_threaded)
   caps = gst_caps_from_string ("foo/bar");
   gst_pad_set_caps (src, caps);
   gst_pad_set_caps (sink, caps);
+  ASSERT_CAPS_REFCOUNT (caps, "caps", 3);
 
   MAIN_START_THREADS (5, thread_link_unlink, NULL);
   for (i = 0; i < 1000; ++i) {
@@ -98,6 +99,13 @@ GST_START_TEST (test_link_unlink_threaded)
     THREAD_SWITCH ();
   }
   MAIN_STOP_THREADS ();
+
+  ASSERT_CAPS_REFCOUNT (caps, "caps", 3);
+  gst_caps_unref (caps);
+
+  ASSERT_CAPS_REFCOUNT (caps, "caps", 2);
+  gst_object_unref (src);
+  gst_object_unref (sink);
 }
 
 GST_END_TEST;
@@ -325,9 +333,21 @@ GST_START_TEST (test_push_linked)
   fail_unless (GST_PAD_LINK_SUCCESSFUL (plr));
   ASSERT_CAPS_REFCOUNT (caps, "caps", 3);
 
+  buffer = gst_buffer_new ();
+#if 0
+  /* FIXME, new pad should be flushing */
+  gst_buffer_ref (buffer);
+  fail_unless (gst_pad_push (src, buffer) == GST_FLOW_WRONG_STATE);
+  gst_buffer_ref (buffer);
+  fail_unless (gst_pad_chain (sink, buffer) == GST_FLOW_WRONG_STATE);
+#endif
+
+  /* activate pads */
+  gst_pad_set_active (src, TRUE);
+  gst_pad_set_active (sink, TRUE);
+
   /* test */
   /* pushing on a linked pad will drop the ref to the buffer */
-  buffer = gst_buffer_new ();
   gst_buffer_ref (buffer);
   fail_unless (gst_pad_push (src, buffer) == GST_FLOW_OK);
   ASSERT_MINI_OBJECT_REFCOUNT (buffer, "buffer", 2);
@@ -437,7 +457,6 @@ GST_START_TEST (test_flowreturn)
   fail_if (strcmp (gst_flow_get_name (ret), "unknown"));
   quark = gst_flow_to_quark (ret);
   fail_unless (quark == 0);
-
 }
 
 GST_END_TEST;
@@ -464,19 +483,4 @@ gst_pad_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gst_pad_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_pad);
