@@ -70,6 +70,13 @@ GST_START_TEST (segment_seek_nosize)
   fail_unless (cstart == 100);
   fail_unless (cstop == 150);
 
+  /* special case, 0 duration */
+  res = gst_segment_clip (&segment, GST_FORMAT_BYTES,
+      100, 100, &cstart, &cstop);
+  fail_unless (res == TRUE);
+  fail_unless (cstart == 100);
+  fail_unless (cstop == 100);
+
   /* completely inside */
   res = gst_segment_clip (&segment, GST_FORMAT_BYTES,
       150, 200, &cstart, &cstop);
@@ -373,6 +380,46 @@ GST_START_TEST (segment_seek_size)
 
 GST_END_TEST;
 
+GST_START_TEST (segment_seek_reverse)
+{
+  GstSegment segment;
+  gboolean update;
+
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+  gst_segment_set_duration (&segment, GST_FORMAT_BYTES, 200);
+
+  /* configure segment to stop 100 */
+  gst_segment_set_seek (&segment, -1.0,
+      GST_FORMAT_BYTES,
+      GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_SET, 0, GST_SEEK_TYPE_SET, 100, &update);
+  fail_unless (segment.start == 0);
+  fail_unless (segment.stop == 100);
+  fail_unless (segment.time == 0);
+  fail_unless (segment.last_stop == 100);
+
+  /* update */
+  gst_segment_set_seek (&segment, -1.0,
+      GST_FORMAT_BYTES,
+      GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_SET, 10, GST_SEEK_TYPE_CUR, -20, &update);
+  fail_unless (segment.start == 10);
+  fail_unless (segment.stop == 80);
+  fail_unless (segment.time == 10);
+  fail_unless (segment.last_stop == 80);
+
+  gst_segment_set_seek (&segment, -1.0,
+      GST_FORMAT_BYTES,
+      GST_SEEK_FLAG_NONE,
+      GST_SEEK_TYPE_SET, 20, GST_SEEK_TYPE_NONE, 0, &update);
+  fail_unless (segment.start == 20);
+  fail_unless (segment.stop == 80);
+  fail_unless (segment.time == 20);
+  fail_unless (segment.last_stop == 80);
+}
+
+GST_END_TEST;
+
 /* mess with the segment structure in the bytes format */
 GST_START_TEST (segment_newsegment_open)
 {
@@ -534,10 +581,10 @@ GST_START_TEST (segment_newsegment_streamtime)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 100);
   fail_unless (result == 100);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 200);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -588,10 +635,10 @@ GST_START_TEST (segment_newsegment_streamtime)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 100);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 700);
-  fail_unless (result == -1);
+  fail_unless (result == 200);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
 
@@ -617,10 +664,10 @@ GST_START_TEST (segment_newsegment_streamtime)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 300);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 700);
-  fail_unless (result == -1);
+  fail_unless (result == 400);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
 }
@@ -665,10 +712,10 @@ GST_START_TEST (segment_newsegment_streamtime_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 150);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 200);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -693,10 +740,10 @@ GST_START_TEST (segment_newsegment_streamtime_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 250);
   fail_unless (result == 150);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
-  fail_unless (result == -1);
+  fail_unless (result == 200);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 400);
   fail_unless (result == -1);
 
@@ -737,10 +784,10 @@ GST_START_TEST (segment_newsegment_streamtime_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
   fail_unless (result == 300);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
-  fail_unless (result == -1);
+  fail_unless (result == 400);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 400);
   fail_unless (result == -1);
 }
@@ -788,10 +835,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 0);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -829,10 +876,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 300);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 400);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -870,10 +917,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 100);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 0);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -912,10 +959,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 0);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 0);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 }
@@ -964,10 +1011,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 300);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 400);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -1005,10 +1052,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 0);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -1046,10 +1093,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 50);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 0);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -1086,10 +1133,10 @@ GST_START_TEST (segment_newsegment_streamtime_applied_rate_rate)
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 150);
   fail_unless (result == 300);
 
-  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 400);
 
+  /* outside of the segment */
   result = gst_segment_to_stream_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 }
@@ -1131,10 +1178,11 @@ GST_START_TEST (segment_newsegment_runningtime)
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 100);
   fail_unless (result == 100);
 
-  /* outside of the segment */
+  /* at edge is exactly the segment duration */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 200);
-  fail_unless (result == -1);
+  fail_unless (result == 200);
 
+  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 300);
   fail_unless (result == -1);
 
@@ -1189,10 +1237,10 @@ GST_START_TEST (segment_newsegment_runningtime)
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 400);
 
-  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 700);
-  fail_unless (result == -1);
+  fail_unless (result == 500);
 
+  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
 
@@ -1221,10 +1269,10 @@ GST_START_TEST (segment_newsegment_runningtime)
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 600);
 
-  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 700);
-  fail_unless (result == -1);
+  fail_unless (result == 500);
 
+  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
 
@@ -1254,10 +1302,10 @@ GST_START_TEST (segment_newsegment_runningtime)
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 600);
   fail_unless (result == 750);
 
-  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 700);
-  fail_unless (result == -1);
+  fail_unless (result == 700);
 
+  /* outside of the segment */
   result = gst_segment_to_running_time (&segment, GST_FORMAT_TIME, 800);
   fail_unless (result == -1);
 
@@ -1272,7 +1320,7 @@ GST_START_TEST (segment_newsegment_runningtime)
 GST_END_TEST;
 
 Suite *
-gstsegments_suite (void)
+gst_segment_suite (void)
 {
   Suite *s = suite_create ("GstSegment");
   TCase *tc_chain = tcase_create ("segments");
@@ -1282,6 +1330,7 @@ gstsegments_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, segment_seek_nosize);
   tcase_add_test (tc_chain, segment_seek_size);
+  tcase_add_test (tc_chain, segment_seek_reverse);
   tcase_add_test (tc_chain, segment_newsegment_open);
   tcase_add_test (tc_chain, segment_newsegment_closed);
   tcase_add_test (tc_chain, segment_newsegment_streamtime);
@@ -1293,19 +1342,4 @@ gstsegments_suite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gstsegments_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_segment);
