@@ -32,6 +32,7 @@
 enum
 {
   ARG_ULONG = 1,
+  ARG_FLOAT,
   ARG_DOUBLE,
   ARG_BOOLEAN,
   ARG_READONLY,
@@ -54,6 +55,7 @@ struct _GstTestMonoSource
 {
   GstElement parent;
   gulong val_ulong;
+  gfloat val_float;
   gdouble val_double;
   gboolean val_boolean;
 };
@@ -74,6 +76,9 @@ gst_test_mono_source_get_property (GObject * object,
     case ARG_ULONG:
       g_value_set_ulong (value, self->val_ulong);
       break;
+    case ARG_FLOAT:
+      g_value_set_float (value, self->val_float);
+      break;
     case ARG_DOUBLE:
       g_value_set_double (value, self->val_double);
       break;
@@ -96,6 +101,9 @@ gst_test_mono_source_set_property (GObject * object,
     case ARG_ULONG:
       self->val_ulong = g_value_get_ulong (value);
       break;
+    case ARG_FLOAT:
+      self->val_float = g_value_get_float (value);
+      break;
     case ARG_DOUBLE:
       self->val_double = g_value_get_double (value);
       break;
@@ -124,6 +132,12 @@ gst_test_mono_source_class_init (GstTestMonoSourceClass * klass)
           "ulong number parameter for the test_mono_source",
           0, G_MAXULONG, 0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
 
+  g_object_class_install_property (gobject_class, ARG_FLOAT,
+      g_param_spec_float ("float",
+          "float prop",
+          "float number parameter for the test_mono_source",
+          0.0, 100.0, 0.0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE));
+
   g_object_class_install_property (gobject_class, ARG_DOUBLE,
       g_param_spec_double ("double",
           "double prop",
@@ -235,6 +249,9 @@ _gst_plugin_static_init__plugin_init (void)
 GST_START_TEST (controller_init)
 {
   gst_controller_init (NULL, NULL);
+  gst_controller_init (NULL, NULL);
+  gst_controller_init (NULL, NULL);
+  gst_controller_init (NULL, NULL);
 }
 
 GST_END_TEST;
@@ -245,6 +262,8 @@ GST_START_TEST (controller_new_fail1)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("fakesrc", "test_source");
 
   /* that property should not exist */
@@ -262,6 +281,8 @@ GST_START_TEST (controller_new_fail2)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* no property given */
@@ -279,6 +300,8 @@ GST_START_TEST (controller_new_fail3)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and but is readonly */
@@ -297,6 +320,8 @@ GST_START_TEST (controller_new_fail4)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and but is not controlable */
@@ -314,6 +339,8 @@ GST_START_TEST (controller_new_fail5)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and but is construct-only */
@@ -333,6 +360,8 @@ GST_START_TEST (controller_new_okay1)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -349,17 +378,40 @@ GST_END_TEST;
 /* tests for an element with several controlled params */
 GST_START_TEST (controller_new_okay2)
 {
-  GstController *ctrl;
+  GstController *ctrl, *ctrl2, *ctrl3;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
-  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", "double", NULL);
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", "double", "float", NULL);
   fail_unless (ctrl != NULL, NULL);
 
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 1);
+
+  ctrl2 = gst_controller_new (G_OBJECT (elem), "boolean", NULL);
+  fail_unless (ctrl2 != NULL, NULL);
+  fail_unless (ctrl2 == ctrl, NULL);
+
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 2);
+
+  /* trying to control the same properties again should correctly
+   * increase the refcount of the object returned as well */
+  ctrl3 =
+      gst_controller_new (G_OBJECT (elem), "ulong", "double", "float", NULL);
+  fail_unless (ctrl3 != NULL, NULL);
+  fail_unless (ctrl3 == ctrl, NULL);
+
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 3);
+
   g_object_unref (ctrl);
+  g_object_unref (ctrl2);
+  g_object_unref (ctrl3);
   gst_object_unref (elem);
 }
 
@@ -368,9 +420,11 @@ GST_END_TEST;
 /* controlling several params should return the same controller */
 GST_START_TEST (controller_new_okay3)
 {
-  GstController *ctrl1, *ctrl2;
+  GstController *ctrl1, *ctrl2, *ctrl3;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -382,8 +436,16 @@ GST_START_TEST (controller_new_okay3)
   fail_unless (ctrl2 != NULL, NULL);
   fail_unless (ctrl1 == ctrl2, NULL);
 
+  /* that property should exist and should be controllable */
+  ctrl3 = gst_controller_new (G_OBJECT (elem), "float", NULL);
+  fail_unless (ctrl3 != NULL, NULL);
+  fail_unless (ctrl1 == ctrl3, NULL);
+
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl1)->ref_count);
+  fail_unless (G_OBJECT (ctrl1)->ref_count == 3);
   g_object_unref (ctrl1);
+  g_object_unref (ctrl2);
+  g_object_unref (ctrl3);
   gst_object_unref (elem);
 }
 
@@ -396,6 +458,8 @@ GST_START_TEST (controller_param_twice)
   GstElement *elem;
   gboolean res;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -423,6 +487,8 @@ GST_START_TEST (controller_finalize)
   GstController *ctrl;
   GstElement *elem;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -449,6 +515,8 @@ GST_START_TEST (controller_interpolate_none)
   gboolean res;
   GValue val_ulong = { 0, };
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -490,6 +558,8 @@ GST_START_TEST (controller_interpolate_trigger)
   gboolean res;
   GValue val_ulong = { 0, };
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -533,6 +603,8 @@ GST_START_TEST (controller_interpolate_linear)
   gboolean res;
   GValue val_ulong = { 0, };
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -574,6 +646,8 @@ GST_START_TEST (controller_unset)
   gboolean res;
   GValue val_ulong = { 0, };
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -628,6 +702,8 @@ GST_START_TEST (controller_unset_all)
   gboolean res;
   GValue val_ulong = { 0, };
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("testmonosource", "test_source");
 
   /* that property should exist and should be controllable */
@@ -667,12 +743,69 @@ GST_START_TEST (controller_unset_all)
 
 GST_END_TEST;
 
+/* test live value handling */
+GST_START_TEST (controller_live)
+{
+  GstController *ctrl;
+  GstElement *elem;
+  gboolean res;
+  GValue val_ulong = { 0, };
+
+  gst_controller_init (NULL, NULL);
+
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  /* set interpolation mode */
+  gst_controller_set_interpolation_mode (ctrl, "ulong", GST_INTERPOLATE_LINEAR);
+
+  /* set control values */
+  g_value_init (&val_ulong, G_TYPE_ULONG);
+  g_value_set_ulong (&val_ulong, 0);
+  res = gst_controller_set (ctrl, "ulong", 0 * GST_SECOND, &val_ulong);
+  fail_unless (res, NULL);
+  g_value_set_ulong (&val_ulong, 100);
+  res = gst_controller_set (ctrl, "ulong", 4 * GST_SECOND, &val_ulong);
+  fail_unless (res, NULL);
+  g_value_set_ulong (&val_ulong, 200);
+  res = gst_controller_set (ctrl, "ulong", 8 * GST_SECOND, &val_ulong);
+  fail_unless (res, NULL);
+
+  /* verify value */
+  gst_controller_sync_values (ctrl, 2 * GST_SECOND);
+  fail_unless (GST_TEST_MONO_SOURCE (elem)->val_ulong == 50, NULL);
+
+  /* set live value */
+  g_object_set (elem, "ulong", 500, NULL);
+
+  /* we should still get the live value */
+  gst_controller_sync_values (ctrl, 3 * GST_SECOND);
+  fail_unless (GST_TEST_MONO_SOURCE (elem)->val_ulong == 500, NULL);
+
+  /* we should not get the live value anymore */
+  gst_controller_sync_values (ctrl, 4 * GST_SECOND);
+  fail_unless (GST_TEST_MONO_SOURCE (elem)->val_ulong == 100, NULL);
+  gst_controller_sync_values (ctrl, 6 * GST_SECOND);
+  fail_unless (GST_TEST_MONO_SOURCE (elem)->val_ulong == 150, NULL);
+
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
 /* tests if we can run helper methods against any GObject */
 GST_START_TEST (controller_helper_any_gobject)
 {
   GstElement *elem;
   gboolean res;
 
+  gst_controller_init (NULL, NULL);
+
   elem = gst_element_factory_make ("bin", "test_elem");
 
   /* that element is not controllable */
@@ -684,8 +817,126 @@ GST_START_TEST (controller_helper_any_gobject)
 
 GST_END_TEST;
 
+GST_START_TEST (controller_misc)
+{
+  GstController *ctrl;
+  GstTimedValue *tval;
+  GstElement *elem;
+  GSList *list = NULL;
 
-Suite *
+  gst_controller_init (NULL, NULL);
+
+  /* test that an invalid timestamp throws a warning of some sort */
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  /* set interpolation mode */
+  gst_controller_set_interpolation_mode (ctrl, "ulong", GST_INTERPOLATE_LINEAR);
+
+  /* set control value */
+  tval = g_new0 (GstTimedValue, 1);
+  tval->timestamp = GST_CLOCK_TIME_NONE;
+  g_value_init (&tval->value, G_TYPE_ULONG);
+  g_value_set_ulong (&tval->value, 0);
+
+  list = g_slist_append (list, tval);
+
+  ASSERT_WARNING (fail_if (gst_controller_set_from_list (ctrl, "ulong", list)));
+
+  /* try again with a valid stamp, should work now */
+  tval->timestamp = 0;
+  fail_unless (gst_controller_set_from_list (ctrl, "ulong", list));
+
+  /* allocated GstTimedValue now belongs to the controller, but list not */
+  g_slist_free (list);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (controller_refcount_new_list)
+{
+  GstController *ctrl, *ctrl2;
+  GstElement *elem;
+  GList *list = NULL;
+
+  gst_controller_init (NULL, NULL);
+
+  /* that property should exist and should be controllable */
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+  list = g_list_append (NULL, "ulong");
+  ctrl = gst_controller_new_list (G_OBJECT (elem), list);
+  fail_unless (ctrl != NULL, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 1);
+  g_list_free (list);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+
+  /* try the same property twice, make sure the refcount is still 1 */
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+  list = g_list_append (NULL, "ulong");
+  list = g_list_append (list, "ulong");
+  ctrl = gst_controller_new_list (G_OBJECT (elem), list);
+  fail_unless (ctrl != NULL, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 1);
+  g_list_free (list);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+
+  /* try two properties, make sure the refcount is still 1 */
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+  list = g_list_append (NULL, "ulong");
+  list = g_list_append (list, "boolean");
+  ctrl = gst_controller_new_list (G_OBJECT (elem), list);
+  fail_unless (ctrl != NULL, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 1);
+  g_list_free (list);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+
+  /* try _new_list with existing controller */
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  list = g_list_append (NULL, "ulong");
+  ctrl2 = gst_controller_new_list (G_OBJECT (elem), list);
+  fail_unless (ctrl2 != NULL, NULL);
+  fail_unless (ctrl == ctrl2, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 2);
+  g_list_free (list);
+  g_object_unref (ctrl);
+  g_object_unref (ctrl2);
+  gst_object_unref (elem);
+
+  /* try _new_list first and then _new */
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+  list = g_list_append (NULL, "ulong");
+  ctrl = gst_controller_new_list (G_OBJECT (elem), list);
+  fail_unless (ctrl != NULL, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  ctrl2 = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl2 != NULL, NULL);
+  fail_unless (ctrl == ctrl2, NULL);
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  fail_unless (G_OBJECT (ctrl)->ref_count == 2);
+  g_list_free (list);
+  g_object_unref (ctrl);
+  g_object_unref (ctrl2);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
+static Suite *
 gst_controller_suite (void)
 {
   Suite *s = suite_create ("Controller");
@@ -693,6 +944,7 @@ gst_controller_suite (void)
 
   suite_add_tcase (s, tc);
   tcase_add_test (tc, controller_init);
+  tcase_add_test (tc, controller_refcount_new_list);
   tcase_add_test (tc, controller_new_fail1);
   tcase_add_test (tc, controller_new_fail2);
   tcase_add_test (tc, controller_new_fail3);
@@ -708,25 +960,11 @@ gst_controller_suite (void)
   tcase_add_test (tc, controller_interpolate_linear);
   tcase_add_test (tc, controller_unset);
   tcase_add_test (tc, controller_unset_all);
+  tcase_add_test (tc, controller_live);
   tcase_add_test (tc, controller_helper_any_gobject);
+  tcase_add_test (tc, controller_misc);
 
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gst_controller_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-  gst_controller_init (NULL, NULL);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_controller);
