@@ -31,7 +31,6 @@
 
 #include <string.h>
 #include <locale.h>
-#include <libintl.h>
 #include <glib/gprintf.h>
 
 static char *_name;
@@ -457,7 +456,7 @@ print_element_properties_info (GstElement * element)
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    %d) %-16s - %s", "",
+            g_print ("\n%s%-23.23s    (%d): %-16s - %s", "",
                 _name, values[j].value, values[j].value_nick,
                 values[j].value_name);
             j++;
@@ -489,8 +488,9 @@ print_element_properties_info (GstElement * element)
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): \t%s", "",
-                _name, values[j].value, values[j].value_nick);
+            g_print ("\n%s%-23.23s    (0x%08x): %-16s - %s", "",
+                _name, values[j].value, values[j].value_nick,
+                values[j].value_name);
             j++;
           }
 
@@ -509,6 +509,8 @@ print_element_properties_info (GstElement * element)
           } else {
             n_print ("%-23.23s Pointer.", "");
           }
+        } else if (param->value_type == G_TYPE_VALUE_ARRAY) {
+          n_print ("%-23.23s Array of GValues", "");
         } else {
           n_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
               g_type_name (param->value_type));
@@ -798,6 +800,9 @@ print_signal_info (GstElement * element)
         if (G_TYPE_IS_FUNDAMENTAL (query->param_types[j])) {
           g_print (",\n%s%s%s arg%d", _name, indent,
               g_type_name (query->param_types[j]), j);
+        } else if (G_TYPE_IS_ENUM (query->param_types[j])) {
+          g_print (",\n%s%s%s arg%d", _name, indent,
+              g_type_name (query->param_types[j]), j);
         } else {
           g_print (",\n%s%s%s* arg%d", _name, indent,
               g_type_name (query->param_types[j]), j);
